/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.ssw.ftcm.handler;

import at.jku.ssw.ftcm.FeatureSelectDialog;
import at.jku.ssw.marker.FeatureMarkerFactory;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jdt.core.*;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.handlers.HandlerUtil;

/**
 * This class handles the assignment of a feature to a method or a class
 * element.
 *
 * @author David Pfeffer
 */
public class MethodClassFeatureAssignHandler extends AbstractHandler {

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.core.commands.AbstractHandler#execute(org.eclipse.core.commands
     * .ExecutionEvent)
     */
    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {

        Shell shell = HandlerUtil.getActiveShell(event);

        // Get selection and first element of it
        IStructuredSelection selection = (IStructuredSelection) HandlerUtil.getActiveMenuSelection(event);
        Object selectedElement = selection.getFirstElement();

        // Check if it is really a element for which we provide marking for
        if (!(selectedElement instanceof IJavaElement) || !(selectedElement instanceof ISourceReference)) {
            return null;
        }

        try {

            IJavaElement javaElement = (IJavaElement) selectedElement;
            ISourceReference sourceReference = (ISourceReference) selectedElement;

            // Open a feature selection dialog to ask the user which feature
            // should be assigned
            FeatureSelectDialog fsd = new FeatureSelectDialog(shell, javaElement.getResource());
            if (fsd.open() == Dialog.OK && fsd.getFeature() != null) {
                ISourceRange range = sourceReference.getSourceRange();
                Position position = new Position(range.getOffset(), range.getLength());

                // Assign the marker depending on the selected type
                switch (javaElement.getElementType()) {
                    case IJavaElement.METHOD:
                        FeatureMarkerFactory.createMethodMarker(javaElement.getResource(), position, (IMethod) javaElement, fsd.getFeature());
                        break;

                    case IJavaElement.TYPE:
                        FeatureMarkerFactory.createClassMarker(javaElement.getResource(), position, (IType) javaElement, fsd.getFeature());
                        break;

                    default:
                        new MessageDialog(HandlerUtil.getActiveShell(event),
                                          "Assignment not supported",
                                          null,
                                          "Assigning features to this type is not supported.",
                                          MessageDialog.ERROR,
                                          new String[]{"Ok"},
                                          0).open();
                        break;
                }
            }
        } catch (CoreException e) {
            e.printStackTrace();
        }

        return null;
    }
}
