/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.ssw.ftcm.proposal;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.ui.text.java.ContentAssistInvocationContext;
import org.eclipse.jdt.ui.text.java.IJavaCompletionProposalComputer;
import org.eclipse.jface.text.contentassist.ICompletionProposal;
import org.eclipse.jface.text.contentassist.IContextInformation;

import java.util.Arrays;
import java.util.List;

/**
 * A computer wrapper for the feature proposal processor.
 *
 * @author David Pfeffer
 */
public class FeatureProposalComputer implements IJavaCompletionProposalComputer {

    // The feature proposal processor
    private final FeatureProposalProcessor processor = new FeatureProposalProcessor();

    /**
     * Creates a new {@link FeatureProposalComputer}.
     */
    public FeatureProposalComputer() {
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jdt.ui.text.java.IJavaCompletionProposalComputer#sessionStarted
     * ()
     */
    @Override
    public void sessionStarted() {

    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jdt.ui.text.java.IJavaCompletionProposalComputer#
     * computeCompletionProposals
     * (org.eclipse.jdt.ui.text.java.ContentAssistInvocationContext,
     * org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public List<ICompletionProposal> computeCompletionProposals(ContentAssistInvocationContext context, IProgressMonitor monitor) {
        return Arrays.asList(processor.computeCompletionProposals(context.getViewer(), context.getInvocationOffset()));
    }

    /*
     * (non-Javadoc)
     *
     * @see org.eclipse.jdt.ui.text.java.IJavaCompletionProposalComputer#
     * computeContextInformation
     * (org.eclipse.jdt.ui.text.java.ContentAssistInvocationContext,
     * org.eclipse.core.runtime.IProgressMonitor)
     */
    @Override
    public List<IContextInformation> computeContextInformation(ContentAssistInvocationContext context, IProgressMonitor monitor) {
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jdt.ui.text.java.IJavaCompletionProposalComputer#getErrorMessage
     * ()
     */
    @Override
    public String getErrorMessage() {
        return null;
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.jdt.ui.text.java.IJavaCompletionProposalComputer#sessionEnded
     * ()
     */
    @Override
    public void sessionEnded() {
    }

}
