/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/

package at.jku.ssw.ftcm.quickfix;

import at.jku.ssw.ftcm.preferences.Feature;
import at.jku.ssw.ftcm.preferences.ProjectSettingsManager;
import at.jku.ssw.marker.FeatureMarkerFactory;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.ui.IMarkerResolution;
import org.eclipse.ui.IMarkerResolutionGenerator;

public class FeatureQuickFixer implements IMarkerResolutionGenerator {

    @Override
    public IMarkerResolution[] getResolutions(IMarker marker) {
        try {
            // Get the feature from the marker
            Feature feature = ProjectSettingsManager.getInstance(marker.getResource().getProject()).getFeature((int) marker.getAttribute("feature"));

            // Construct the correct prefix
            StringBuilder markerType = new StringBuilder();
            if (marker.isSubtypeOf(FeatureMarkerFactory.FEATURE_CLASS_MARKER)) {
                markerType.append("Class");
            } else if (marker.isSubtypeOf(FeatureMarkerFactory.FEATURE_METHOD_MARKER)) {
                markerType.append("Method");
            } else if (marker.isSubtypeOf(FeatureMarkerFactory.FEATURE_STATEMENT_MARKER)) {
                markerType.append("Statement");
            } else {
                markerType.append("Unknown");
            }

            // Return the result with to quickfix options edit and remove
            return new IMarkerResolution[]{new FeatureRemoveQuickFix("Remove " + markerType.toString() + " Feature " + feature.getDescription() + " (" + feature.getId() + ")"),
                                           new FeatureEditQuickFix("Edit " + markerType.toString() + " Feature " + feature.getDescription() + " (" + feature.getId() + ")")};
        } catch (CoreException e) {
            return new IMarkerResolution[0];
        }
    }
}
