import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.GarbageCollectionCause;
import at.jku.mevss.anttracks.heap.GarbageCollectionType;
import at.jku.mevss.anttracks.heap.HeapListener;
import at.jku.mevss.anttracks.heap.io.MetaDataWriterConfig;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.parser.classdefinitions.ClassDefinitionsFile;
import at.jku.mevss.anttracks.parser.heap.HeapTraceParser;
import at.jku.mevss.anttracks.parser.symbols.SymbolsFile;
import at.jku.mevss.anttracks.parser.symbols.SymbolsParser;
import at.jku.mevss.anttracks.util.Consts;
import at.jku.mevss.anttracks.util.Counter;

import java.io.File;
import java.io.PrintWriter;

public class Main {

    // trace file and meta directory expected in users home folder
    static String basePath = System.getProperty("user.home");
    static String metaDir = basePath + File.separator + Consts.ANT_META_DIRECTORY;
    static File traceFile = new File(basePath + File.separator + "trace");

    private final static File output = new File(System.getProperty("user.home") + File.separator + "labSizes.csv");
    private static PrintWriter pw;

    public static void main(String[] args) throws Exception {

        Counter c = new Counter(1);
        pw = new PrintWriter(output);
        pw.append("gc #;gc phase;avg object count per lab;object counts per lab...;\n");

        SymbolsParser symbolsParser = new SymbolsParser(SymbolsFile.findSymbolsFileToTrace(traceFile),
                                                        ClassDefinitionsFile.findClassDefinitionsFileToTrace(traceFile),
                                                        null,
                                                        Symbols.CALLCONTEXT_STATIC);
        Symbols symbols = symbolsParser.parse();
        HeapTraceParser metaDataParser = new HeapTraceParser(symbols, new MetaDataWriterConfig(metaDir, 100, false));
        metaDataParser.addHeapListener(new HeapListener() {

            @Override
            public void close(DetailedHeap heap) {}

            @Override
            public void phaseChanging(DetailedHeap heap,
                                      GarbageCollectionType from,
                                      GarbageCollectionType to,
                                      GarbageCollectionCause cause,
                                      boolean concurrent,
                                      boolean failed,
                                      long time,
                                      long position,
                                      long traceLength) {}

            @Override
            public void phaseChanged(DetailedHeap heap,
                                     GarbageCollectionType from,
                                     GarbageCollectionType to,
                                     GarbageCollectionCause cause,
                                     boolean concurrent,
                                     boolean failed,
                                     long time,
                                     long position,
                                     long traceLength) {
                pw.append(c.get() + ";");
                pw.append(to == GarbageCollectionType.MUTATOR ? "END;" : "START;");
                pw.append(heap.getLabSizes());
                c.inc();
            }
        });
        metaDataParser.parse();

    }
}
