/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification;

import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.util.AnnotationHelper;

import java.util.logging.Logger;

public abstract class Filter extends Classifier<Boolean> {

    protected Logger LOGGER = Logger.getLogger(this.getClass().getSimpleName());

    @Override
    public String toString() {
        String s = getName();
        return s != null ? s : super.toString();
    }

    /**
     * Gets the filters's name
     *
     * @return The classifier's name
     */
    @Override
    public String getName() {
        if (name == null) {
            at.jku.mevss.anttracks.classification.annotations.Filter annotation = AnnotationHelper.getAnnotation(this,
                                                                                                                 at.jku.mevss.anttracks.classification.annotations
                                                                                                                         .Filter.class);
            name = annotation != null ? annotation.name() : "No name";
        }
        return name;
    }

    /**
     * Gets the classifier's name
     *
     * @return The classifier's name
     */
    @Override
    public String getDesc() {
        if (desc == null) {
            at.jku.mevss.anttracks.classification.annotations.Filter annotation = AnnotationHelper.getAnnotation(this,
                                                                                                                 at.jku.mevss.anttracks.classification.annotations
                                                                                                                         .Filter.class);
            desc = annotation != null ? annotation.desc() : "No description";
        }
        return desc;
    }

    /**
     * Gets the classifier's name
     *
     * @return The classifier's name
     */
    public ClassifierSourceCollection getSourceCollection() {
        if (sourceCollection == null) {
            at.jku.mevss.anttracks.classification.annotations.Filter annotation = AnnotationHelper.getAnnotation(this,
                                                                                                                 at.jku.mevss.anttracks.classification.annotations
                                                                                                                         .Filter.class);
            sourceCollection = annotation != null ? annotation.collection() : ClassifierSourceCollection.ALL;
        }
        return sourceCollection;
    }
}