/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification;

import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.heap.IndexBasedHeap;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.util.CollectionsUtil;

import java.util.function.Supplier;

public class FilterFactory implements CollectionsUtil.Factory<Filter> {
    private final String name;
    private final String desc;
    private final ClassifierSourceCollection sourceCollection;

    private final Supplier<Filter> supplier;

    public FilterFactory(Supplier<Symbols> symbolsSupplier, Supplier<IndexBasedHeap> fastHeapSupplier, Filter filter) {
        this.name = filter.getName();
        this.desc = filter.getDesc();
        this.sourceCollection = filter.getSourceCollection();
        supplier = () -> {
            Filter newFilter = null;
            try {
                newFilter = filter.getClass().newInstance();
            } catch (InstantiationException | IllegalAccessException e) {
                // TODO: Error handling
                e.printStackTrace();
                return null;
            }
            newFilter.setup(symbolsSupplier, fastHeapSupplier);
            newFilter.setIsCustom(true);
            newFilter.setSourceCode(filter.getSourceCode());
            return newFilter;
        };
    }

    public String getName() {
        return name;
    }

    public String getDesc() {
        return desc;
    }

    public ClassifierSourceCollection getSourceCollection() {
        return sourceCollection;
    }

    public boolean isOnTheFlyCompilable() {
        return create().isOnTheFlyCompilable();
    }

    public String getSourceCode() {
        return create().getSourceCode();
    }

    @Override
    public Filter create() {
        return supplier.get();
    }

    @Override
    public String toString() {
        return name;
    }
}
