/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification.annotations;

import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;

import java.lang.annotation.*;

/**
 * This annotation is used to annotate {@link at.jku.mevss.anttracks.classification.Classifier} subclasses.
 *
 * @author Markus Weninger
 */
@Inherited
@Retention(RetentionPolicy.RUNTIME)
@Target({ElementType.TYPE, ElementType.TYPE_USE})
public @interface Classifier {
    /**
     * Returns the classification's name. Examples: "Allocated Type", "Features", ...
     *
     * @return The classification's name
     */
    String name();

    /**
     * Returns the classification's description.
     *
     * @return The classification's description
     */
    String desc();

    /**
     * Returns the classification's type.
     *
     * @return The classification's type
     */
    ClassifierType type();

    /**
     * Returns the classification's source collection.
     *
     * @return The classification's source collection
     */
    ClassifierSourceCollection collection();

    /**
     * Returns the classification's example
     *
     * @return An example classification key for this classifier, e.g. "java.lang.String" for the type classifier
     */
    String example();
}
