/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.diff;

public class AddressIterationHelper {
    private static final long NO_ADDR = -1;
    public final AddressCollection.AddressCollectionIterator iterator;
    private long nextAddrToFind;

    private long lastHeapAddress = -1;

    public AddressIterationHelper(AddressCollection addressCollection) {
        iterator = addressCollection.getIterator();
        setNextAddress();
    }

    public boolean checkNextAddress(long currentHeapAddr) {
        while (nextAddrToFind < currentHeapAddr && nextAddrToFind != NO_ADDR) {
            setNextAddress();
        }

        if (currentHeapAddr < lastHeapAddress) {
            assert false;
            throw new IllegalStateException("How did we end up here? AddressIterationHelper should only be used with sorted heaps");
        }

        if (nextAddrToFind == currentHeapAddr) {
            setNextAddress();
            return true;
        }
        return false;
    }

    public void setNextAddress() {
        if (iterator.hasNext()) {
            nextAddrToFind = iterator.next();
        } else {
            nextAddrToFind = NO_ADDR;
        }
    }
}
