/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.features;

import at.jku.mevss.anttracks.heap.symbols.AllocationSite;

import java.util.ArrayList;
import java.util.List;

public class FeatureMap {

    private final Feature others = new Feature("others", 127, 127, 127);
    private final List<Feature> features = new ArrayList<>();

    {
        features.add(others);
    }

    public Feature[] getFeatures() {
        return features.toArray(new Feature[features.size()]);
    }

    public Feature getFeature(int id) {
        return features.get(id);
    }

    public int getFeatureCount() {
        return features.size();
    }

    public long getPackageMappingCount() {
        return features.stream().flatMap(f -> f.getPackages().stream()).count();
    }

    public long getTypeMappingCount() {
        return features.stream().flatMap(f -> f.getTypes().stream()).count();
    }

    public long getMethodeMappingCount() {
        return features.stream().flatMap(f -> f.getMethods().stream()).count();
    }

    public long getInstructionMappingCount() {
        return features.stream().flatMap(f -> f.getInstructions().stream()).count();
    }

    public void add(Feature feature) {
        features.add(feature);
    }

    public int[] match(AllocationSite site) {
        for (AllocationSite.Location callSite : site.callSites) {
            final String splitter = ".";
            String signature = callSite.signature;
            int index = signature.lastIndexOf(splitter);
            if (index < 0) {
                return new int[]{0};
            }
            String type = signature.substring(0, index);
            String methodAndSignature = signature.substring(index + splitter.length());

            int nMatches = 0;
            boolean[] matches = new boolean[features.size()];

            if (nMatches == 0) {
                for (int id = 0; id < getFeatureCount(); id++) {
                    if (matches[id]) {
                        continue;
                    }
                    Feature feature = getFeature(id);
                    for (InstructionMapping mapping : feature.getInstructions()) {
                        if (mapping.matches(type, methodAndSignature, callSite.bci)) {
                            matches[id] = true;
                            nMatches++;
                            break;
                        }
                    }
                }
            }

            if (nMatches == 0) {
                for (int id = 0; id < getFeatureCount(); id++) {
                    if (matches[id]) {
                        continue;
                    }
                    Feature feature = getFeature(id);
                    for (MethodMapping mapping : feature.getMethods()) {
                        if (mapping.matches(type, methodAndSignature, callSite.bci)) {
                            matches[id] = true;
                            nMatches++;
                            break;
                        }
                    }
                }
            }

            if (nMatches == 0) {
                for (int id = 0; id < getFeatureCount(); id++) {
                    if (matches[id]) {
                        continue;
                    }
                    Feature feature = getFeature(id);
                    for (TypeMapping mapping : feature.getTypes()) {
                        if (mapping.matches(type, methodAndSignature, callSite.bci)) {
                            matches[id] = true;
                            nMatches++;
                            break;
                        }
                    }
                }
            }

            if (nMatches == 0) {
                for (int id = 0; id < getFeatureCount(); id++) {
                    if (matches[id]) {
                        continue;
                    }
                    Feature feature = getFeature(id);
                    for (PackageMapping mapping : feature.getPackages()) {
                        if (mapping.matches(type, methodAndSignature, callSite.bci)) {
                            matches[id] = true;
                            nMatches++;
                            break;
                        }
                    }
                }
            }

            if (nMatches > 0) {
                int[] result = new int[nMatches];
                int cursor = 0;
                for (int id = 0; id < getFeatureCount(); id++) {
                    if (matches[id]) {
                        result[cursor++] = id;
                    }
                }
                return result;
            }
        }

        return new int[]{0};
    }

}
