package at.jku.mevss.anttracks.heap;

import at.jku.mevss.anttracks.heap.objects.ObjectInfo;

import java.util.Arrays;

/**
 * Provides the same information as the original {@link DetailedHeap} datastructure
 * but offers better performance by using index- instead of address-based access
 */
public class FastHeap extends IndexBasedHeap {

    //================================================================================
    // fields
    //================================================================================

    // TODO: Store Object Infos on hard drive
    protected int[] objectInfoIds;
    private long[] addresses;
    private int[][] toPointers;
    private int[][] fromPointers;

    //================================================================================
    // Constructor
    //================================================================================
    public FastHeap(DetailedHeap heap) {
        super(heap);
    }

    @Override
    protected void setup(ObjectInfo[] objectInfos, long[] addressesArray, int[][] toPointersArray, int[][] fromPointersArray) {
        this.objectInfoIds = Arrays.stream(objectInfos).mapToInt(oi -> oi.id).toArray();
        this.addresses = addressesArray;
        this.toPointers = toPointersArray;
        this.fromPointers = fromPointersArray;
    }

    //================================================================================
    // utilities
    //================================================================================
    @Override
    public void clear() {

    }

    //================================================================================
    // getters
    //================================================================================

    @Override
    public long getAddress(int objIndex) {
        if (!valid(objIndex)) {
            return NULL_INDEX;
        }

        return addresses[objIndex];
    }

    @Override
    public int[] getToPointers(int objIndex) {
        if (!valid(objIndex)) {
            return null;
        }

        return toPointers[objIndex];
    }

    @Override
    public int[] getFromPointers(int objIndex) {
        if (!valid(objIndex)) {
            return null;
        }

        return fromPointers[objIndex];
    }

    @Override
    public ObjectInfo getObjectInfo(int objIndex) {
        if (!valid(objIndex)) {
            return null;
        }

        assert objectInfoIds[objIndex] >= 0 : "ObjectInfo index must be positive";

        return objectInfoCache.getById(objectInfoIds[objIndex]);
    }

    @Override
    public int toIndex(long address) {
        int objIndex = Arrays.binarySearch(addresses, address);
        if (!valid(objIndex)) {
            // although it is not a big deal for this method, it shouldn't happen considering current usage of this method
            //assert false;
            return NULL_INDEX;
        }
        return objIndex;
    }
}
