package at.jku.mevss.anttracks.heap.datastructures.cocor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static at.jku.mevss.anttracks.util.DataStructureUtil.starDummyDataStructurePart;

public class PointsToType {
    private Pattern namePattern;
    private String name;
    private boolean follow;
    private List<DataStructurePart> matchingDsParts;

    public PointsToType(String name, boolean follow) {
        this.name = name;
        // create regex from given name, that matches all wildcards ('*') in the name with anything
        String regex = Arrays.stream(name.split(Pattern.quote("*"), -1)).map(Pattern::quote).collect(Collectors.joining(".*"));
        namePattern = Pattern.compile(regex);

        this.follow = follow;
        this.matchingDsParts = new ArrayList<>();
    }

    public void resolve(List<DataStructurePart> definitions) {
        matchingDsParts.clear();
        if (name.equals("*")) {
            matchingDsParts.add(starDummyDataStructurePart);
        } else {
            matchingDsParts.addAll(definitions.stream()
                                              .filter(dsPart -> matches(dsPart.getType()))
                                              .collect(Collectors.toList()));
        }
    }

    public List<DataStructurePart> getMatchingDataStructureParts() {
        return matchingDsParts;
    }

    public String getName() {
        return name;
    }

    public boolean isFollow() {
        return follow;
    }

    /*
    public boolean matches(AllocatedType type) {
        boolean directMatch = matches(type.getFullyQualifiedName(true));
        return directMatch || type.isSubtypeOf(namePattern);
    }
    */

    public boolean mayBeArray() {
        return name.endsWith("[]") || name.endsWith("*");
    }

    private boolean matches(String type) {
        return namePattern.matcher(type).matches();
    }

    public boolean isFlat() {
        return !follow;
    }
}
