/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.io;

import at.jku.mevss.anttracks.heap.GarbageCollectionCause;
import at.jku.mevss.anttracks.heap.GarbageCollectionType;
import at.jku.mevss.anttracks.parser.EventType;
import at.jku.mevss.anttracks.parser.io.BaseFile;
import at.jku.mevss.anttracks.util.Consts;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;

public class HeapIndexWriter implements AutoCloseable {

    private final DataOutputStream out;

    public HeapIndexWriter(String path) throws IOException {
        out = new DataOutputStream(new BufferedOutputStream(BaseFile.openW(path + File.separator + Consts.HEAP_INDEX_META_FILE)));
        out.writeInt(Consts.HEAP_FILES_MAGIC_PREFIX);
    }

    public void write(long position, GarbageCollectionType gcType, EventType gcMeta, GarbageCollectionCause cause, long time, int gcCount, boolean fileExists)
            throws IOException {
        out.writeLong(position);
        out.writeInt(gcType.id);
        out.writeInt(gcMeta.id);
        out.writeInt(cause.id);
        out.writeLong(time);
        out.writeInt(gcCount);
        out.writeBoolean(fileExists);
        // TODO various trace files
    }

    public void flush() throws IOException {
        out.flush();
    }

    @Override
    public void close() throws IOException {
        out.close();
    }

}
