/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.io;

import at.jku.mevss.anttracks.heap.statistics.*;
import at.jku.mevss.anttracks.parser.io.BaseFile;
import at.jku.mevss.anttracks.util.Consts;

import java.io.BufferedOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;

public class StatisticsWriter implements AutoCloseable {

    private final DataOutputStream out;

    public StatisticsWriter(String path) throws IOException {
        this.out = new DataOutputStream(new BufferedOutputStream(BaseFile.openW(path + File.separator + Consts.STATISTICS_META_FILE)));
        out.writeInt(Consts.HEAP_FILES_MAGIC_PREFIX);
    }

    public void flush() throws IOException {
        out.flush();
    }

    @Override
    public void close() throws IOException {
        out.close();
    }

    public boolean write(Statistics heapStatistics) throws IOException {
        out.writeLong(heapStatistics.info.time);
        out.writeInt(heapStatistics.info.meta.id);
        out.writeInt(heapStatistics.info.type.id);
        out.writeInt(heapStatistics.info.cause.id);
        out.writeBoolean(heapStatistics.info.concurrent);
        out.writeBoolean(heapStatistics.info.failed);

        writeSpaceStatistics(heapStatistics.eden);
        writeSpaceStatistics(heapStatistics.survivor);
        writeSpaceStatistics(heapStatistics.old);

        return true;
    }

    private void writeSpaceStatistics(SpaceStatistics space) throws IOException {
        writeAllocators(space.allocators);
        writeMemoryConsumption(space.memoryConsumption);
        writeObjectTypes(space.objectTypes);
        writeFeatureAllocations(space.featureConsumptions);
    }

    private void writeAllocators(Allocators allocator) throws IOException {
        out.writeLong(allocator.getVm());
        out.writeLong(allocator.getIr());
        out.writeLong(allocator.getC1());
        out.writeLong(allocator.getC2());
    }

    private void writeMemoryConsumption(MemoryConsumption memoryConsumption) throws IOException {
        out.writeLong(memoryConsumption.getObjects());
        out.writeLong(memoryConsumption.getBytes());
    }

    private void writeObjectTypes(ObjectTypes objectTypes) throws IOException {
        out.writeLong(objectTypes.getInstances());
        out.writeLong(objectTypes.getSmallArrays());
        out.writeLong(objectTypes.getBigArrays());
    }

    private void writeFeatureAllocations(MemoryConsumption[] featureAllocation) throws IOException {
        if (featureAllocation == null) {
            out.writeBoolean(false);
        } else {
            out.writeBoolean(true);
            out.writeInt(featureAllocation.length);
            for (MemoryConsumption memoryConsumption : featureAllocation) {
                out.writeLong(memoryConsumption.getObjects());
                out.writeLong(memoryConsumption.getBytes());
            }
        }
    }

}
