/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.objects;

import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.parser.EventType;
import at.jku.mevss.anttracks.util.Consts;

public class ArrayObjectInfo extends ObjectInfo implements ArrayInfo {

    public static ArrayObjectInfo get(ObjectInfoCache cache, EventType eventType, AllocationSite allocationSite, AllocatedType type, int length, Symbols symbols) {
        ArrayObjectInfo info = new ArrayObjectInfo(eventType, allocationSite, type, length, symbols);

        return (ArrayObjectInfo) cache.get(info);
    }

    public static ArrayObjectInfo get(ObjectInfoCache cache,
                                      ArrayObjectInfo info,
                                      EventType eventType,
                                      AllocationSite allocationSite,
                                      AllocatedType type,
                                      int length,
                                      Symbols symbols) {

        int size = getObjectSize(type, length);
        info.eventType = eventType;
        info.type = type;
        info.allocationSite = allocationSite;
        info.size = padObjectSize(size, symbols.heapWordSize);
        info.length = length;

        return (ArrayObjectInfo) cache.get(info);
    }

    private int length;

    public ArrayObjectInfo() {
        super();
        length = 0;
    }

    private ArrayObjectInfo(EventType eventType, AllocationSite allocationSite, AllocatedType type, int length, Symbols symbols) {
        super(eventType, allocationSite, type, getObjectSize(type, length), symbols.heapWordSize);
        assert length >= 0;
        this.length = length;
    }

    public ArrayObjectInfo(ArrayObjectInfo info) {
        super(info.eventType, info.allocationSite, info.type, info.size);
        length = info.length;
    }

    private static int getObjectSize(AllocatedType type, int length) {
        int sizeInfo = type.size;
        int headerSize = sizeInfo >>> 8;
        int elemSize = sizeInfo & Consts.HEX_ELEM_SIZE;
        int size = headerSize + elemSize * length;
        return size;
    }

    @Override
    public int hashCode() {
        return super.hashCode() * 39 + length;
    }

    @Override
    public boolean equals(Object other) {
        return super.equals(other) && length == ((ArrayObjectInfo) other).length;
    }

    @Override
    public String toString() {
        return "Array " + super.toString();
    }

    @Override
    public boolean isArray() {
        return true;
    }

    @Override
    public int getLength() {
        return length;
    }

    @Override
    public boolean isSmallArray() {
        return length < Consts.ARRAY_SIZE_MAX_SMALL;
    }
}
