/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.objects;

import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.parser.EventType;

public class InstanceObjectInfo extends ObjectInfo {

    private static InstanceObjectInfo nullInfo = new InstanceObjectInfo();

    public static InstanceObjectInfo get(ObjectInfoCache cache, EventType eventType, AllocationSite allocationSite, AllocatedType type, Symbols symbols) {
        InstanceObjectInfo info = new InstanceObjectInfo(eventType, allocationSite, type, symbols);
        return (InstanceObjectInfo) cache.get(info);
    }

    public static InstanceObjectInfo get(ObjectInfoCache cache,
                                         InstanceObjectInfo info,
                                         EventType eventType,
                                         AllocationSite allocationSite,
                                         AllocatedType type,
                                         Symbols symbols) {
        int size = type == null ? 0 : type.size;
        info.eventType = eventType;
        info.type = type;
        info.allocationSite = allocationSite;
        info.size = padObjectSize(size, symbols.heapWordSize);

        return (InstanceObjectInfo) cache.get(info);
    }

    public static ObjectInfo NULL(ObjectInfoCache cache, Symbols symbols) {
        return get(cache, nullInfo, EventType.NOP, null, null, symbols);
    }

    public InstanceObjectInfo() {
        super();
    }

    private InstanceObjectInfo(EventType eventType, AllocationSite allocationSite, AllocatedType type, Symbols symbols) {
        this(eventType, allocationSite, type, type.size, symbols);
    }

    protected InstanceObjectInfo(EventType eventType, AllocationSite allocationSite, AllocatedType type, int size, Symbols symbols) {
        super(eventType, allocationSite, type, size, symbols.heapWordSize);
    }

    public InstanceObjectInfo(InstanceObjectInfo info) {
        super(info.eventType, info.allocationSite, info.type, info.size);
    }

    @Override
    public String toString() {
        return "Instance " + super.toString();
    }

    @Override
    public boolean isArray() {
        return false;
    }
}
