package at.jku.mevss.anttracks.heap.roots;

import at.jku.mevss.anttracks.heap.DetailedHeap;

import java.nio.ByteBuffer;

public class ClassLoaderRoot extends RootPtr {

    public final String loaderName;

    public ClassLoaderRoot(long addr, String loaderName) {
        super(addr);
        this.loaderName = loaderName;
    }

    public String toString() {
        StringBuilder b = new StringBuilder(getClass().getSimpleName()).append("\n");
        b.append("- ").append(loaderName != null ? loaderName : "???").append("\n");

        return b.toString();
    }

    @Override
    public String toShortString() {
        return getClass().getSimpleName() + " " + (loaderName != null ? loaderName : "???");
    }

    @Override
    public byte[] getMetadata() {
        return ByteBuffer.allocate(1 + 1 * Long.BYTES + 1 * Integer.BYTES + loaderName.getBytes().length * Byte.BYTES)
                         .put(RootType.CLASS_LOADER_ROOT.byteVal)
                         .putLong(addr)
                         .putInt(loaderName.length())
                         .put(loaderName.getBytes())
                         .array();
    }

    @Override
    public RootPtr.RootType getRootType() {
        return RootType.CLASS_LOADER_ROOT;
    }

    @Override
    public String getTypeString() { return "Class loader"; }

    @Override
    public void resolve(DetailedHeap heap) throws Exception {
        resolved = true;
    }

    @Override
    public String[] hierarchy() {
        String[] ret = {"Root: Class loader", "Name: " + (loaderName != null ? loaderName : "???")};
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (!ClassLoaderRoot.class.isAssignableFrom(obj.getClass())) {
            return false;
        }

        final ClassLoaderRoot other = (ClassLoaderRoot) obj;

        return this.addr == other.addr && this.loaderName.equals(other.loaderName);
    }

}
