package at.jku.mevss.anttracks.heap.roots;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;

import java.nio.ByteBuffer;
import java.util.Arrays;

public class CodeBlobRoot extends RootPtr {

    public final int classId;
    public final int methodId;
    private AllocatedType klass;
    private AllocatedType.MethodInfo method;

    public CodeBlobRoot(long addr, int classId, int methodId) {
        super(addr);
        this.classId = classId;
        this.methodId = methodId;
    }

    public String toString() {
        StringBuilder b = new StringBuilder("Code blob:\n");
        b.append("- ").append(klass != null ? klass.name : "???").append(" (").append(classId).append(")\n");
        b.append("-- ").append(method != null ? method.name : "???").append(" (").append(methodId).append(")\n");

        return b.toString();
    }

    @Override
    public String toShortString() {
        return getClass().getSimpleName() + " " + (method != null ? method.name : "???") + " in " + (klass != null ? klass.getExternalName(true, false) : "???");
    }

    @Override
    public byte[] getMetadata() {
        return ByteBuffer.allocate(1 + Long.BYTES + 2 * Integer.BYTES).put(RootType.CODE_BLOB_ROOT.byteVal).putLong(addr).putInt(classId).putInt(methodId).array();
    }

    @Override
    public RootType getRootType() {
        return RootType.CODE_BLOB_ROOT;
    }

    @Override
    public void resolve(DetailedHeap heap) throws Exception {
        klass = heap.getSymbols().types.stream().filter(c -> c.id == this.classId).findFirst().orElse(null);
        if (klass != null) {
            method = Arrays.stream(klass.methodInfos).filter(methodInfo -> methodInfo.idnum == methodId).findFirst().orElse(null);
            resolved = true;
        }

        //throw new Exception("Could not resolve code blob root: " + toString());
    }

    @Override
    public String[] hierarchy() {
        String[] ret = {"Root: Code blob",
                        "Class: " + (klass != null ? klass.getExternalName(false, false) : "???"),
                        "Method: " + (method != null ? method.name : "???")};
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (!CodeBlobRoot.class.isAssignableFrom(obj.getClass())) {
            return false;
        }

        final CodeBlobRoot other = (CodeBlobRoot) obj;

        return this.addr == other.addr && this.classId == other.classId && this.methodId == other.methodId;
    }
}
