package at.jku.mevss.anttracks.heap.roots;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.parser.heap.ThreadInfo;

import java.nio.ByteBuffer;

public class JNILocalRoot extends RootPtr {
    public final long threadId;
    private ThreadInfo thread;

    public JNILocalRoot(long addr, long threadId) {
        super(addr);
        this.threadId = threadId;
    }

    public String toString() {
        StringBuilder b = new StringBuilder("JNI local:\n");
        b.append("- ").append(thread != null ? thread.threadName : "???").append(" (").append(threadId).append(")\n");

        return b.toString();
    }

    @Override
    public String toShortString() {
        return getClass().getSimpleName() + " in " + (thread != null ? thread.threadName : "???");
    }

    @Override
    public byte[] getMetadata() {
        return ByteBuffer.allocate(1 + 2 * Long.BYTES).put(RootType.JNI_LOCAL_ROOT.byteVal).putLong(addr).putLong(threadId).array();
    }

    @Override
    public RootType getRootType() {
        return RootType.JNI_LOCAL_ROOT;
    }

    @Override
    public void resolve(DetailedHeap heap) throws Exception {
        thread = heap.threads.get(threadId);
        if (thread == null) {
            //            throw new Exception("Could not resolve JNI local root: " + toString());
        } else {
            resolved = true;
        }
    }

    @Override
    public String[] hierarchy() {
        String[] ret = {"Root: JNI local", "Thread: " + (thread != null ? thread.threadName : "???")};
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (!JNILocalRoot.class.isAssignableFrom(obj.getClass())) {
            return false;
        }

        final JNILocalRoot other = (JNILocalRoot) obj;

        return this.addr == other.addr && this.threadId == other.threadId;
    }
}
