package at.jku.mevss.anttracks.heap.roots;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.parser.heap.ThreadInfo;

import java.nio.ByteBuffer;

public class VMInternalThreadDataRoot extends RootPtr {

    public final long threadId;
    private ThreadInfo thread;

    public VMInternalThreadDataRoot(long addr, long threadId) {
        super(addr);
        this.threadId = threadId;
    }

    public String toString() {
        StringBuilder b = new StringBuilder(getClass().getSimpleName()).append("\n");
        b.append(" - ").append(thread != null ? thread.threadName : "???").append("\n");

        return b.toString();
    }

    @Override
    public String toShortString() {
        return getClass().getSimpleName() + " in " + (thread != null ? thread.threadName : "???");
    }

    @Override
    public byte[] getMetadata() {
        return ByteBuffer.allocate(1 + Long.BYTES + Long.BYTES).put(RootType.VM_INTERNAL_THREAD_DATA_ROOT.byteVal).putLong(addr).putLong(threadId).array();
    }

    @Override
    public RootPtr.RootType getRootType() {
        return RootType.VM_INTERNAL_THREAD_DATA_ROOT;
    }

    @Override
    public void resolve(DetailedHeap heap) throws Exception {
        thread = heap.threads.get(threadId);
        if (thread == null) {
            //            throw new Exception("Could not resolve Thread root: " + toString());
        } else {
            resolved = true;
        }
    }

    @Override
    public String[] hierarchy() {
        String[] ret = {"Root: VM internal thread data", "Thread: " + (thread != null ? thread.threadName : "???")};
        return ret;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }

        if (!VMInternalThreadDataRoot.class.isAssignableFrom(obj.getClass())) {
            return false;
        }

        final VMInternalThreadDataRoot other = (VMInternalThreadDataRoot) obj;

        return this.addr == other.addr;
    }

}
