/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.heap.symbols;

import at.jku.mevss.anttracks.util.Consts;

import java.util.Arrays;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Objects;

public class ClassDefinitions implements Iterable<ClassDefinition> {

    ClassDefinition[] definitions;
    boolean isEmpty;

    public ClassDefinitions() {
        definitions = new ClassDefinition[Consts.START_ARRAY_LEN];
        isEmpty = true;
    }

    public void add(ClassDefinition definition) {
        if (definition.id >= definitions.length) {
            int newSize = definitions.length * 2;
            while (definition.id >= newSize) {
                newSize *= 2;
            }
            definitions = Arrays.copyOf(definitions, newSize);
        }
        definitions[definition.id] = definition;
        isEmpty = false;
    }

    public ClassDefinition get(int id) {
        return definitions[id];
    }

    /**
     * Get the current length of the array (that is, the maximum capacity before the array is expanded).
     *
     * @return The current capacity.
     */
    public int getLength() {
        return definitions.length;
    }

    /**
     * Get the current count of class definitions.
     *
     * @return The number of class definitions stored.
     */
    public int getCount() {
        return (int) Arrays.stream(definitions).filter(Objects::nonNull).count();
    }

    /**
     * Get whether there are any class definitions.
     *
     * @return True if {@link #getCount()} would return a non-zero value.
     */
    public boolean isEmpty() {
        return isEmpty;
    }

    @Override
    public Iterator<ClassDefinition> iterator() {
        return new Iterator<ClassDefinition>() {
            int idx = 0;

            private boolean advance() {
                while (idx < definitions.length && definitions[idx] == null) {
                    idx++;
                }
                return idx < definitions.length;
            }

            @Override
            public ClassDefinition next() {
                if (!advance()) {
                    throw new NoSuchElementException();
                }
                return definitions[idx++];
            }

            @Override
            public boolean hasNext() {
                return advance();
            }
        };
    }
}
