/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser;

import at.jku.mevss.anttracks.heap.space.SpaceType;
import at.jku.mevss.anttracks.util.TraceException;

public class RelAddrFactory {

    private final long alignment;
    private long base;
    // stuff carried over from discarded RelAddr class
    private static final SpaceType[] spaces;
    private final static int SPACE_MASK = 0x3;
    private final static int ADDR_MASK = 0xFFFFFFFC;

    static {
        spaces = SpaceType.values();
    }

    public RelAddrFactory(long alignment) {
        this.alignment = alignment;
        setBase(0);
    }

    public void setBase(long base) {
        this.base = base;
    }

    public long create(int rel_addr) throws TraceException {
        long offset = rel_addr;
        offset &= (1L << 32) - 1; // necessary because of sign extension
        offset = offset * alignment;
        long addr = base + offset;
        return addr;
    }

    /*
     * public static long getRelAddr(Heap heap, long absAddr) throws TraceException { return absAddr - getSpaceFromAbsolute(heap,
     * absAddr).getAddress(); }
     */

    public static SpaceType getDefinedSpaceOnly(int spaceId) {
        return spaces[spaceId & SPACE_MASK];
    }

    public static long getDefinedAddrOnly(int addr) {
        return addr & ADDR_MASK;
    }
}
