package at.jku.mevss.anttracks.parser;

import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.roots.RootPtr;
import at.jku.mevss.anttracks.heap.space.Space;
import at.jku.mevss.anttracks.heap.space.SpaceMode;
import at.jku.mevss.anttracks.heap.space.SpaceType;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.util.TraceException;

import java.util.List;

public interface TraceParsingEventHandler {

    public abstract void doCleanUp(ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParseSpaceRedefine(int index, long startAddr, long size, boolean postponed, int source);

    public abstract void doParseSpaceDestroy(int firstIndex, long nRegions);

    public abstract void doParseSpaceRelease(int index);

    public abstract void doParseSpaceAlloc(int index, SpaceMode spaceMode, SpaceType spaceType);

    public abstract void doParseSpaceCreate(int index, long startAddr, long size) throws TraceException;

    public abstract void doParseGCInfo(int index, int id) throws TraceException;

    public abstract void doParseGCFailed(int index) throws TraceException;

    public abstract void doParseGCStart(ParserGCInfo gcInfo, long start, long end, boolean check) throws TraceException;

    public abstract void doParseGCEnd(ParserGCInfo info, ThreadLocalHeap threadLocalheap, long start, long end, boolean failed, boolean check) throws TraceException;

    public abstract void doParseGCInterrupt(int id, long address) throws TraceException;

    public abstract void doParseGCContinue(int id, long address) throws TraceException;

    public abstract long doGCMove(EventType type, ThreadLocalHeap threadLocalHeap, ObjectInfo obj, long fromAddr, long toAddr, SpaceType toSpaceType)
            throws TraceException;

    public abstract ObjectInfo getObjectIsFiller(Space space, long addr) throws TraceException;

    public abstract void doParseGCClassLoaderRootPtr(long ptr, String loaderName) throws TraceException;

    public abstract void doParseGCClassRootPtr(long ptr, int classId) throws TraceException;

    public abstract void doParseGCStaticFieldRootPtr(long ptr, int classId, int offset) throws TraceException;

    public abstract void doParseGCJNILocalRootPtr(long ptr, long threadId) throws TraceException;

    public abstract void doParseGCLocalVariableRootPtr(long ptr, long threadId, int classId, int methodId, int slot) throws TraceException;

    public abstract void doParseGCVMInternalThreadDataRootPtr(long ptr, long threadId) throws TraceException;

    public abstract void doParseGCCodeBlobRootPtr(long ptr, int classId, int methodId) throws TraceException;

    public abstract void doParseGCJNIGlobalRootPtr(long ptr, boolean weak) throws TraceException;

    public abstract void doParseGCOtherRootPtr(long ptr, RootPtr.RootType rootType) throws TraceException;

    public abstract void doParseGCDebugRootPtr(long ptr, String vmCall) throws TraceException;

    public abstract List<ObjectInfo> doParseGCMoveRegion(EventType type,
                                                         long fromAddr,
                                                         long toAddr,
                                                         SpaceType toSpaceType,
                                                         int numOfObjects,
                                                         ThreadLocalHeap threadLocalHeap)
            throws TraceException;

    public abstract Space getSpace(long addr);

    public abstract void doParseThreadAlive(int header, long id, String name);

    public abstract void doParseThreadDeath(long id);

    public abstract void doParseSyncObj(EventType type,
                                        int allocationSiteId,
                                        int allocatedTypeId,
                                        AllocatedType allocatedType,
                                        long fromAddr,
                                        long toAddr,
                                        SpaceType toSpaceType,
                                        int length,
                                        int size,
                                        ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParseObjAllocFastCi(EventType type,
                                               int allocationSiteId,
                                               AllocationSite allocationSite,
                                               AllocatedType allocatedType,
                                               boolean isArray,
                                               int arrayLength,
                                               ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParseObjAllocFastC2DeviantType(EventType type,
                                                          int header,
                                                          int allocationSiteId,
                                                          AllocatedType allocatedType,
                                                          boolean isArray,
                                                          int arrayLength,
                                                          ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParseObjAllocFastIr(EventType type, int allocationSiteId, AllocationSite allocationSite, ThreadLocalHeap threadLocalHeap)
            throws TraceException;

    public abstract void doParseObjAllocNormalCi(EventType type,
                                                 int allocationSiteId,
                                                 AllocationSite allocationSite,
                                                 AllocatedType allocatedType,
                                                 long addr,
                                                 boolean isArray,
                                                 int arrayLength,
                                                 ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParseObjAllocNormalIr(EventType type, int allocationSiteId, AllocationSite allocationSite, long addr, ThreadLocalHeap threadLocalHeap)
            throws TraceException;

    public abstract void doParseObjAllocSlowCiIr_Deviant(EventType type,
                                                         int allocationSiteId,
                                                         AllocationSite allocationSite,
                                                         long addr,
                                                         int arrayLength,
                                                         int realAllocatedTypeId,
                                                         ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParseObjAllocSlow(EventType type,
                                             int allocationSiteId,
                                             AllocationSite allocationSite,
                                             long addr,
                                             boolean isArray,
                                             int arrayLength,
                                             int size,
                                             boolean mayBeFiller,
                                             ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParseTlabAlloc(int size, long addr, ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doParsePlabAlloc(int size, long addr, ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract long getMoveTarget(long fromAddr, long toAddr, SpaceType toSpaceType, ThreadLocalHeap threadLocalHeap) throws TraceException;

    public abstract void doPtrEvent(EventType type, ThreadLocalHeap threadLocalHeap, long fromAddr, long toAddr, long[] ptrs, boolean isFiller) throws TraceException;

    public abstract void doParseGCTag(String tagText);
}
