/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;

public class TraceScannerFactory {

    private final TraceFile.Access[] files;
    private final long from, to;
    private int index;

    public TraceScannerFactory(TraceFile.Access[] files) {
        this(files, 0, getLength(files));
    }

    public TraceScannerFactory(TraceFile.Access[] files, long from, long to) {
        this.files = Arrays.copyOf(files, files.length);
        this.from = from;
        this.to = to;
        this.index = 0;

        skip();
    }

    private void skip() {
        long cursor = 0;
        while (index < files.length && cursor + files[index].length() <= from) {
            cursor += files[index].length();
            index++;
        }
    }

    public long getFrom() {
        return from;
    }

    public long getTo() {
        return to;
    }

    public long getLength() {
        return to - from;
    }

    public Scanner getNext() throws IOException {
        Scanner result;
        if (index < files.length) {
            TraceFile.Access file = files[index];
            long cursor = getCursor();
            long from = cursor < this.from ? (this.from - cursor) : 0;
            long to = cursor + file.length() < this.to ? file.length() : (this.to - cursor);
            result = to - from > 0 ? file.open(cursor, from, to) : null;
            index++;
        } else {
            result = null;
        }
        return result;
    }

    private long getCursor() {
        return getLength(files, index);
    }

    private static long getLength(TraceFile.Access[] files) {
        return getLength(files, files.length);
    }

    private static long getLength(TraceFile.Access[] files, int length) {
        return Arrays.asList(files).stream().limit(length).mapToLong(f -> f.length()).sum();
    }

    public static TraceScannerFactory create(File container, int[] header, File example) throws IOException {
        return create(container, header, example, null, null);
    }

    public static TraceScannerFactory create(File container, int[] header, File example, Long from, Long to) throws IOException {
        TraceFile.Access[] files = TraceFile.open(container, header, example);
        return from != null && to != null ? new TraceScannerFactory(files, from, to) : new TraceScannerFactory(files);
    }

}
