/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.heap;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.roots.RootPtr;
import at.jku.mevss.anttracks.heap.space.Space;
import at.jku.mevss.anttracks.heap.space.SpaceMode;
import at.jku.mevss.anttracks.heap.space.SpaceType;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.parser.*;
import at.jku.mevss.anttracks.util.TraceException;

import java.util.List;

public class HeapEventHandler implements TraceParsingEventHandler {

    // --------------------------------------------------------
    // ---------------------- Fields --------------------------
    // --------------------------------------------------------

    public final ParsingInfo parsingInfo;
    public final HeapBuilder heapConstructor;
    public final DetailedHeap heap;

    // --------------------------------------------------------
    // ------------------- Constructor ------------------------
    // --------------------------------------------------------

    public HeapEventHandler(DetailedHeap heap, Symbols symbols, ParsingInfo parsingInfo) {
        this.heap = heap;
        this.parsingInfo = parsingInfo;
        this.heapConstructor = new HeapBuilder(this.heap, symbols, parsingInfo);
    }

    // --------------------------------------------------------
    // --------------------- Methods --------------------------
    // --------------------------------------------------------

    // --------------------------------------------------------
    // --------------------- General --------------------------
    // --------------------------------------------------------

    @Override
    public void doCleanUp(ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doCleanUp(threadLocalHeap);
    }

    @Override
    public void doParseGCStart(ParserGCInfo info, long start, long end, boolean check) throws TraceException {
        heapConstructor.doParseGCStart(info, start, end, check);
    }

    @Override
    public void doParseGCEnd(ParserGCInfo info, ThreadLocalHeap threadlocalHeap, long start, long end, boolean failed, boolean check) throws TraceException {
        heapConstructor.doParseGCEnd(info, threadlocalHeap, start, end, failed, check);
    }

    @Override
    public void doParseThreadAlive(int header, long id, String name) {
        heapConstructor.doParseThreadAlive(header, id, name);
    }

    @Override
    public void doParseThreadDeath(long id) {
        heapConstructor.doParseThreadDead(id);
    }

    // --------------------------------------------------------
    // ------------------- Move events ------------------------
    // --------------------------------------------------------

    @Override
    public List<ObjectInfo> doParseGCMoveRegion(EventType type, long fromAddr, long toAddr, SpaceType toSpaceType, int numOfObjects, ThreadLocalHeap threadLocalHeap)
            throws TraceException {
        return heapConstructor.doParseGCMoveRegion(type, fromAddr, toAddr, toSpaceType, numOfObjects, threadLocalHeap);
    }

    @Override
    public void doParseSyncObj(EventType type,
                               int allocationSiteId,
                               int allocatedTypeId,
                               AllocatedType allocatedType,
                               long fromAddr,
                               long toAddr,
                               SpaceType toSpaceType,
                               int length,
                               int size,
                               ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseSyncObj(type, allocationSiteId, allocatedTypeId, allocatedType, fromAddr, toAddr, toSpaceType, length, size, threadLocalHeap);
    }

    // --------------------------------------------------------
    // ------------------- Allocations ------------------------
    // --------------------------------------------------------

    @Override
    public void doParseObjAllocFastCi(EventType type,
                                      int allocationSiteId,
                                      AllocationSite allocationSite,
                                      AllocatedType allocatedType,
                                      boolean isArray,
                                      int arrayLength,
                                      ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseObjAllocFastCi(type, allocationSiteId, allocationSite, allocatedType, isArray, arrayLength, threadLocalHeap);
    }

    @Override
    public void doParseObjAllocFastC2DeviantType(EventType type,
                                                 int header,
                                                 int allocationSiteId,
                                                 AllocatedType allocatedType,
                                                 boolean isArray,
                                                 int arrayLength,
                                                 ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseObjAllocFastC2DeviantType(type, header, allocationSiteId, allocatedType, isArray, arrayLength, threadLocalHeap);
    }

    @Override
    public void doParseObjAllocFastIr(EventType type, int allocationSiteId, AllocationSite allocationSite, ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseObjAllocFastIr(type, allocationSiteId, allocationSite, threadLocalHeap);
    }

    @Override
    public void doParseObjAllocNormalCi(EventType type,
                                        int allocationSiteId,
                                        AllocationSite allocationSite,
                                        AllocatedType allocatedType,
                                        long addr,
                                        boolean isArray,
                                        int arrayLength,
                                        ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseObjAllocNormalCi(type, allocationSiteId, allocationSite, allocatedType, addr, isArray, arrayLength, threadLocalHeap);
    }

    @Override
    public void doParseObjAllocNormalIr(EventType type, int allocationSiteId, AllocationSite allocationSite, long addr, ThreadLocalHeap threadLocalHeap)
            throws TraceException {
        heapConstructor.doParseObjAllocNormalIr(type, allocationSiteId, allocationSite, addr, threadLocalHeap);
    }

    @Override
    public void doParseObjAllocSlowCiIr_Deviant(EventType type,
                                                int allocationSiteId,
                                                AllocationSite allocationSite,
                                                long addr,
                                                int arrayLength,
                                                int realAllocatedTypeId,
                                                ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseObjAllocSlowCiIr_Deviant(type, allocationSiteId, allocationSite, addr, arrayLength, realAllocatedTypeId, threadLocalHeap);
    }

    @Override
    public void doParseObjAllocSlow(EventType type,
                                    int allocationSiteId,
                                    AllocationSite allocationSite,
                                    long addr,
                                    boolean isArray,
                                    int arrayLength,
                                    int size,
                                    boolean mayBeFiller,
                                    ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseObjAllocSlow(type, allocationSiteId, allocationSite, addr, isArray, arrayLength, size, mayBeFiller, threadLocalHeap);
    }

    @Override
    public void doParseTlabAlloc(int size, long addr, ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParseTlabAlloc(size, addr, threadLocalHeap);
    }

    @Override
    public void doParsePlabAlloc(int size, long addr, ThreadLocalHeap threadLocalHeap) throws TraceException {
        heapConstructor.doParsePlabAlloc(size, addr, threadLocalHeap);
    }

    // --------------------------------------------------------
    // ----------------------- Spaces -------------------------
    // --------------------------------------------------------

    @Override
    public void doParseSpaceRedefine(int index, long startAddr, long size, boolean postponed, int source) {
        heapConstructor.doParseSpaceRedefine(index, startAddr, size, postponed, source);
    }

    @Override
    public void doParseSpaceDestroy(int firstIndex, long nRegions) {
        heapConstructor.doParseSpaceDestroy(firstIndex, nRegions);
    }

    @Override
    public void doParseSpaceRelease(int index) {
        heapConstructor.doParseSpaceRelease(index);
    }

    @Override
    public void doParseSpaceAlloc(int index, SpaceMode mode, SpaceType type) {
        heapConstructor.doParseSpaceAlloc(index, mode, type);
    }

    @Override
    public void doParseSpaceCreate(int index, long startAddr, long size) throws TraceException {
        heapConstructor.doParseSpaceCreate(index, startAddr, size);
    }

    // --------------------------------------------------------
    // ----------------------- GC -----------------------------
    // --------------------------------------------------------

    @Override
    public void doParseGCInfo(int index, int gcId) throws TraceException {
        heapConstructor.doParseGCInfo(index, gcId);
    }

    @Override
    public void doParseGCFailed(int index) throws TraceException {
        heapConstructor.doParseGCFailed(index);
    }

    @Override
    public void doParseGCInterrupt(int id, long address) throws TraceException {
        heapConstructor.doParseGCInterrupt(id, address);
    }

    @Override
    public void doParseGCContinue(int id, long address) throws TraceException {
        heapConstructor.doParseGCContinue(id, address);
    }

    // --------------------------------------------------------
    // ----------------------- Moves --------------------------
    // --------------------------------------------------------

    @Override
    public long getMoveTarget(long fromAddr, long toAddr, SpaceType toSpaceType, ThreadLocalHeap threadLocalHeap) throws TraceException {
        return heapConstructor.getMoveTarget(fromAddr, toAddr, toSpaceType, threadLocalHeap);
    }

    @Override
    public ObjectInfo getObjectIsFiller(Space space, long addr) throws TraceException {
        return heapConstructor.getObjectIfFiller(space, addr);
    }

    @Override
    public long doGCMove(EventType type, ThreadLocalHeap threadLocalHeap, ObjectInfo obj, long fromAddr, long toAddr, SpaceType toSpaceType) throws TraceException {
        return heapConstructor.doGCMove(type, threadLocalHeap, obj, fromAddr, toAddr, toSpaceType);

    }

	/*
    @Override
	public long doGCMove(EventType type, ThreadLocalHeap threadLocalHeap, ObjectInfoAge objAge, long fromAddr, long toAddr,
			SpaceType toSpaceType) throws TraceException {
		return heapConstructor.doGCMove(type, threadLocalHeap, objAge, fromAddr, toAddr, toSpaceType);
	}
	*/

    @Override
    public void doPtrEvent(EventType eventType, ThreadLocalHeap threadLocalHeap, long fromAddr, long toAddr, long[] ptrs, boolean isFiller) throws TraceException {
        heapConstructor.doPtrEvent(eventType, threadLocalHeap, fromAddr, toAddr, ptrs, isFiller);
    }

    @Override
    public void doParseGCTag(String tagText) {
        heapConstructor.addTag(tagText);
    }

    // --------------------------------------------------------
    // ----------------------- Ptr ----------------------------
    // --------------------------------------------------------

    @Override
    public void doParseGCClassLoaderRootPtr(long ptr, String loaderName) throws TraceException {
        heapConstructor.doParseGCClassLoaderRootPtr(ptr, loaderName);
    }

    @Override
    public void doParseGCClassRootPtr(long ptr, int classId) throws TraceException {
        heapConstructor.doParseGCClassRootPtr(ptr, classId);
    }

    @Override
    public void doParseGCStaticFieldRootPtr(long ptr, int classId, int offset) throws TraceException {
        heapConstructor.doParseGCStaticFieldRootPtr(ptr, classId, offset);
    }

    @Override
    public void doParseGCLocalVariableRootPtr(long ptr, long threadId, int classId, int methodId, int slot) throws TraceException {
        heapConstructor.doParseGCLocalVariableRootPtr(ptr, threadId, classId, methodId, slot);
    }

    @Override
    public void doParseGCVMInternalThreadDataRootPtr(long ptr, long threadId) throws TraceException {
        heapConstructor.doParseGCVMInternalThreadDataRootPtr(ptr, threadId);
    }

    @Override
    public void doParseGCCodeBlobRootPtr(long ptr, int classId, int methodId) throws TraceException {
        heapConstructor.doParseGCCodeBlobRootPtr(ptr, classId, methodId);
    }

    @Override
    public void doParseGCJNILocalRootPtr(long ptr, long threadId) throws TraceException {
        heapConstructor.doParseGCJNILocalRootPtr(ptr, threadId);
    }

    @Override
    public void doParseGCJNIGlobalRootPtr(long ptr, boolean weak) throws TraceException {
        heapConstructor.doParseGCJNIGlobalRootPtr(ptr, weak);
    }

    @Override
    public void doParseGCOtherRootPtr(long ptr, RootPtr.RootType rootType) throws TraceException {
        heapConstructor.doParseGCOtherRootPtr(ptr, rootType);
    }

    @Override
    public void doParseGCDebugRootPtr(long ptr, String vmCall) throws TraceException {
        heapConstructor.doParseGCDebugRootPtr(ptr, vmCall);
    }

    // --------------------------------------------------------
    // ----------------------- Helper -------------------------
    // --------------------------------------------------------

    @Override
    public Space getSpace(long addr) {
        return heapConstructor.getSpace(addr);
    }

    public DetailedHeap getHeap() {
        return this.heap;
    }
}
