/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.heap.pointer;

import at.jku.mevss.anttracks.parser.EventType;

import java.util.Arrays;

public class PtrEvent {
    public final EventType eventType;
    public final long toAddr;
    public final long fromAddr;
    public long[] ptrs;
    public static final int MAX_PTRS_PER_EVENT = 12;
    public static final int ENCODING_END = 0;
    public static final int ENCODING_RELATIVE_PTR = 1;
    public static final int ENCODING_ABSOLUTE_PTR = 2;
    public static final int ENCODING_NULL_PTR = 3;

    public PtrEvent(EventType eventType, long fromAddr, long toAddr, long[] ptrs) {
        this.eventType = eventType;
        this.fromAddr = fromAddr;
        this.toAddr = toAddr;
        this.ptrs = ptrs;
    }

    public boolean isDedicatedPtrEvent() {
        return EventType.isDedicatedPtrEvent(eventType);
    }

    @Override
    public String toString() {
        return String.format("PtrEvent (%s) from %,d to %,d, containing following pointers: %s",
                             eventType.toString(),
                             fromAddr,
                             toAddr,
                             ptrs == null ? "NULL" : Arrays.toString(ptrs));
    }
}
