/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.util;

import at.jku.mevss.anttracks.heap.symbols.AllocatedType;

public class SignatureConverter {

    private final static char FULLY_QUALIFIED_CLASS = 'L';
    private final static char ARRAY = '[';
    private final static char VOID = 'V';

    private final static char BOOLEAN = 'Z';
    private final static char BYTE = 'B';
    private final static char CHAR = 'C';
    private final static char SHORT = 'S';
    private final static char INT = 'I';
    private final static char LONG = 'J';
    private final static char FLOAT = 'F';
    private final static char DOUBLE = 'D';
    private final static char UNKNOWN = AllocatedType.ALLOCATED_TYPE_IDENTIFIER_UNKNOWN_SIG;

    public static String convertToJavaMethodSignature(String original) {
        return convertToJavaMethodSignature(original, false);
    }

    public static String convertToJavaMethodSignature(String original, boolean omitPackage) {
        int paramStart = original.indexOf('(');
        int paramEnd = original.indexOf(')');

        if (paramStart >= 0 && paramEnd >= 0) {
            String signature = original.substring(0, paramStart);
            String declaringType = signature.substring(0, signature.lastIndexOf('.'));
            String method = signature.substring(declaringType.length() + 1);
            String args = convertToJavaType(original.substring(paramStart + 1, paramEnd), omitPackage);
            String returnType = convertToJavaType(original.substring(paramEnd + 1, original.length()), omitPackage);

            // switch(method) {
            // case "<init>": method = declaringType.contains(".") ? declaringType.substring(declaringType.lastIndexOf('.') + 1) :
            // declaringType; break;
            // case "<clinit>": method = "static"; break;
            // }

            if (omitPackage) {
                declaringType = declaringType.substring(declaringType.lastIndexOf('.') + 1);
            }

            return declaringType + '.' + method + '(' + args + ')' + " : " + returnType;
        } else {
            return original; // VM_INTERNAL;
        }

    }

    public static String convertToJavaType(String typeSignature) {
        return convertToJavaType(typeSignature, false);
    }

    public static String convertToJavaType(String typeSignatures, boolean omitPackage) {
        StringBuilder result = new StringBuilder();

        while (typeSignatures.length() > 0) {
            if (result.length() > 0) {
                result.append(", ");
            }

            int arrayRank = 0;
            for (int i = 0; typeSignatures.charAt(i) == ARRAY; i++) {
                arrayRank++;
            }
            typeSignatures = typeSignatures.substring(arrayRank);

            String type;
            if (typeSignatures.charAt(0) == FULLY_QUALIFIED_CLASS) {
                type = typeSignatures.substring(1, typeSignatures.indexOf(';'));
                typeSignatures = typeSignatures.substring(type.length() + 2);
                type = type.replace('/', '.');
                if (omitPackage && type.contains(".")) {
                    type = type.substring(type.lastIndexOf('.') + 1);
                }
            } else {
                switch (typeSignatures.charAt(0)) {
                    case VOID:
                        type = "void";
                        break;
                    case BOOLEAN:
                        type = "boolean";
                        break;
                    case CHAR:
                        type = "char";
                        break;
                    case BYTE:
                        type = "byte";
                        break;
                    case SHORT:
                        type = "short";
                        break;
                    case INT:
                        type = "int";
                        break;
                    case LONG:
                        type = "long";
                        break;
                    case FLOAT:
                        type = "float";
                        break;
                    case DOUBLE:
                        type = "double";
                        break;
                    case UNKNOWN:
                        type = "unknown";
                        break;
                    default:
                        assert false;
                        type = null;
                        break;
                }
                typeSignatures = typeSignatures.substring(1);
            }

            result.append(type);
            for (int i = 0; i < arrayRank; i++) {
                result.append("[]");
            }
        }

        return result.toString();
    }
}
