/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.util;

import java.util.Objects;

public class TraceException extends Exception {

    /**
     * A {@link RuntimeException} that wraps a {@link TraceException} so it can be thrown from inside a lambda. Exceptions of this type
     * should be caught and {@linkplain #rethrow() rethrown} as soon as possible.
     *
     * @author Peter Feichtinger
     */
    public static class TraceExceptionLambdaWrapper extends RuntimeException {
        private static final long serialVersionUID = 1L;

        /**
         * Create a {@link TraceExceptionLambdaWrapper} from the specified exception and throw it.
         *
         * @param ex The {@link TraceException} to wrap.
         * @throws TraceExceptionLambdaWrapper Always.
         */
        public static void boom(TraceException ex) {
            throw new TraceExceptionLambdaWrapper(ex);
        }

        /**
         * Create a {@link TraceExceptionLambdaWrapper} from the specified {@link TraceException}.
         *
         * @param inner The exception to wrap.
         */
        public TraceExceptionLambdaWrapper(TraceException inner) {
            super(Objects.requireNonNull(inner));
        }

        /**
         * Rethrow the wrapped {@link TraceException}.
         *
         * @throws TraceException Always.
         */
        public void rethrow() throws TraceException {
            throw (TraceException) getCause();
        }
    }

    private static final long serialVersionUID = 1L;

    public TraceException() {
        super();
    }

    public TraceException(String message) {
        super(message);
    }

    public TraceException(Throwable cause) {
        super(cause);
    }

}
