/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.classification.test;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierFactoryList;
import at.jku.mevss.anttracks.classification.nodes.MapGroupingNode;
import at.jku.mevss.anttracks.classification.nodes.MapGroupingNode.HeapObjectGroupingKey;
import at.jku.mevss.anttracks.util.TraceException;
import org.junit.Before;
import org.junit.Test;

import java.io.*;
import java.util.HashMap;

import static org.junit.Assert.assertEquals;

public class MapGroupingNode_Test {

    MapGroupingNode flat_g1 = new MapGroupingNode();
    MapGroupingNode flat_g2 = new MapGroupingNode();
    MapGroupingNode flat_g3 = new MapGroupingNode();

    MapGroupingNode deep_g1 = new MapGroupingNode();
    MapGroupingNode deep_g2 = new MapGroupingNode();
    MapGroupingNode deep_g3 = new MapGroupingNode();

    SampleClassifier sampleClassifier = new SampleClassifier();

    ClassifierFactoryList availableClassifier;

    @Before
    public void initClass() throws IOException, TraceException, Exception {
        availableClassifier = new ClassifierFactoryList(null, null, null);
        availableClassifier.add(sampleClassifier);

        flat_g1 = new MapGroupingNode();
        flat_g1.addLeaf(new HeapObjectGroupingKey(1), 10);
        flat_g1.addLeaf(new HeapObjectGroupingKey(2), 20);
        flat_g1.addLeaf(new HeapObjectGroupingKey(3), 30);

        flat_g2 = new MapGroupingNode();
        flat_g2.addLeaf(new HeapObjectGroupingKey(1), 10);
        flat_g2.addLeaf(new HeapObjectGroupingKey(2), 20);
        flat_g2.addLeaf(new HeapObjectGroupingKey(3), 30);

        flat_g3 = new MapGroupingNode();
        flat_g3.addLeaf(new HeapObjectGroupingKey(1), 10);
        flat_g3.addLeaf(new HeapObjectGroupingKey(2), 20);
        flat_g3.addLeaf(new HeapObjectGroupingKey(3), 30);

        deep_g1 = new MapGroupingNode();
        deep_g1.addLeaf(new HeapObjectGroupingKey(1), 10);
        deep_g1.addLeaf(new HeapObjectGroupingKey(2), 20);
        deep_g1.addLeaf(new HeapObjectGroupingKey(3), 30);
        MapGroupingNode child = null;
        child = (MapGroupingNode) deep_g1.addChildIfNeeded(sampleClassifier, "c1");
        child.addLeaf(new HeapObjectGroupingKey(1), 10);
        child.addLeaf(new HeapObjectGroupingKey(2), 10);
        child.addLeaf(new HeapObjectGroupingKey(3), 10);
        child = null;
        child = (MapGroupingNode) deep_g1.addChildIfNeeded(sampleClassifier, "c2");
        child.addLeaf(new HeapObjectGroupingKey(2), 10);
        child.addLeaf(new HeapObjectGroupingKey(3), 10);
        child.addLeaf(new HeapObjectGroupingKey(4), 10);
        child = (MapGroupingNode) deep_g1.addChildIfNeeded(sampleClassifier, "c3");
        child.addLeaf(new HeapObjectGroupingKey(5), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1");
        child.addLeaf(new HeapObjectGroupingKey(5), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1");
        child.addLeaf(new HeapObjectGroupingKey(5), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1");
        child.addLeaf(new HeapObjectGroupingKey(5), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1-1");
        child.addLeaf(new HeapObjectGroupingKey(5), 10);
        child = (MapGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-2");
        child.addLeaf(new HeapObjectGroupingKey(5), 10);
        child = (MapGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-3");
        child.addLeaf(new HeapObjectGroupingKey(1000), 10);

        deep_g2 = new MapGroupingNode();
        deep_g2.addLeaf(new HeapObjectGroupingKey(1), 10);
        deep_g2.addLeaf(new HeapObjectGroupingKey(2), 20);
        deep_g2.addLeaf(new HeapObjectGroupingKey(3), 30);
        child = (MapGroupingNode) deep_g2.addChildIfNeeded(sampleClassifier, "c1");
        child.addLeaf(new HeapObjectGroupingKey(1), 10);
        child.addLeaf(new HeapObjectGroupingKey(2), 10);
        child.addLeaf(new HeapObjectGroupingKey(3), 10);
        child = null;
        child = (MapGroupingNode) deep_g2.addChildIfNeeded(sampleClassifier, "c2");
        child.addLeaf(new HeapObjectGroupingKey(2), 10);
        child.addLeaf(new HeapObjectGroupingKey(3), 10);
        child.addLeaf(new HeapObjectGroupingKey(4), 10);
        child = (MapGroupingNode) deep_g2.addChildIfNeeded(sampleClassifier, "c3");
        child.addLeaf(new HeapObjectGroupingKey(6), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1");
        child.addLeaf(new HeapObjectGroupingKey(6), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1");
        child.addLeaf(new HeapObjectGroupingKey(6), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1");
        child.addLeaf(new HeapObjectGroupingKey(6), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1-1");
        child.addLeaf(new HeapObjectGroupingKey(6), 10);
        child = (MapGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-2");
        child.addLeaf(new HeapObjectGroupingKey(6), 10);
        child = (MapGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-3");
        child.addLeaf(new HeapObjectGroupingKey(1000), 10);

        deep_g3 = new MapGroupingNode();
        deep_g3.addLeaf(new HeapObjectGroupingKey(1), 10);
        deep_g3.addLeaf(new HeapObjectGroupingKey(2), 20);
        deep_g3.addLeaf(new HeapObjectGroupingKey(3), 30);
        child = (MapGroupingNode) deep_g3.addChildIfNeeded(sampleClassifier, "c1");
        child.addLeaf(new HeapObjectGroupingKey(1), 10);
        child.addLeaf(new HeapObjectGroupingKey(2), 10);
        child.addLeaf(new HeapObjectGroupingKey(3), 10);
        child = null;
        child = (MapGroupingNode) deep_g3.addChildIfNeeded(sampleClassifier, "c2");
        child.addLeaf(new HeapObjectGroupingKey(2), 10);
        child.addLeaf(new HeapObjectGroupingKey(3), 10);
        child.addLeaf(new HeapObjectGroupingKey(4), 10);
        child = (MapGroupingNode) deep_g3.addChildIfNeeded(sampleClassifier, "c3");
        child.addLeaf(new HeapObjectGroupingKey(7), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1");
        child.addLeaf(new HeapObjectGroupingKey(7), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1");
        child.addLeaf(new HeapObjectGroupingKey(7), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1");
        child.addLeaf(new HeapObjectGroupingKey(7), 10);
        child = (MapGroupingNode) child.addChildIfNeeded(sampleClassifier, "c3-1-1-1-1");
        child.addLeaf(new HeapObjectGroupingKey(7), 10);
        child = (MapGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-2");
        child.addLeaf(new HeapObjectGroupingKey(7), 10);
        child = (MapGroupingNode) child.getParent().addChildIfNeeded(sampleClassifier, "c3-1-1-1-3");
        child.addLeaf(new HeapObjectGroupingKey(1000), 10);
    }

    @Test
    public void test_Merge_Flat() {
        MapGroupingNode g = (MapGroupingNode) flat_g1.merge(flat_g2).merge(flat_g3);
        assertEquals(180, g.getObjectCount());
    }

    @Test
    public void test_Merge_Deep() {
        MapGroupingNode g = (MapGroupingNode) deep_g1.merge(deep_g2).merge(deep_g3);
        assertEquals(570, g.getObjectCount());
    }

    @Test
    public void test_Merge_SampleDown_Flat() {
        MapGroupingNode g = (MapGroupingNode) flat_g1.merge(flat_g2).merge(flat_g3).sampleTopDown();
        assertEquals(180, g.getObjectCount());
    }

    @Test
    public void test_Merge_SampleDown_Deep() {
        MapGroupingNode g = (MapGroupingNode) deep_g1.merge(deep_g2).merge(deep_g3).sampleTopDown();
        assertEquals(570, g.getObjectCount());
    }

    @Test
    public void test_writeAndRead() throws FileNotFoundException {
        HashMap<String, Classifier<?>> classifier = new HashMap<>();
        availableClassifier.forEach(fac -> classifier.put(fac.getName(), fac.create()));

        MapGroupingNode writeTree = (MapGroupingNode) deep_g1.merge(deep_g2).merge(deep_g3).sampleTopDown();
        MapGroupingNode.writeTree(new DataOutputStream(new FileOutputStream("./testoutput", false)), writeTree);
        MapGroupingNode readTree = MapGroupingNode.readTree(new DataInputStream(new FileInputStream("./testoutput")), null, classifier);
        assertEquals(writeTree.toString(), readTree.toString());
    }
}
