package at.jku.mevss.anttracks.dominators.test;

import at.jku.mevss.anttracks.heap.*;
import at.jku.mevss.anttracks.parser.EventType;
import at.jku.mevss.anttracks.util.AntTracksDominators;
import at.jku.mevss.anttracks.util.TestUtility;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;

public class DominatorTreeTest {

    private static PreprocessedHeap preprocessedHeap;

    @BeforeClass
    public static void initClass() throws Exception {
        String traceFolder = TestUtility.getExampleTrace("PointerTraces/DominatorTree", "./downloadedTrace");
        preprocessedHeap = TestUtility.preprocess(traceFolder, true);
    }

    @Test
    public void testEx1DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(hIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(iIndex));
        assertEquals(gIndex, fastHeap.getImmediateDominator(jIndex));
        assertEquals(fIndex, fastHeap.getImmediateDominator(kIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(lIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(mIndex));
    }

    @Test
    public void testEx1DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(hIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(iIndex));
        assertEquals(gIndex, dominators.getImmediateDominator(jIndex));
        assertEquals(fIndex, dominators.getImmediateDominator(kIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(lIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(mIndex));
    }

    @Test
    public void testEx2DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 1));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int rIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$R")).findAny().orElse(-1);
        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);

        assertNotEquals(-1, rIndex);
        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(rIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(aIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(hIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(iIndex));
        assertEquals(gIndex, fastHeap.getImmediateDominator(jIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(kIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(lIndex));
    }

    @Test
    public void testEx2DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 1));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int rIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$R")).findAny().orElse(-1);
        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);

        assertNotEquals(-1, rIndex);
        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);

        assertEquals(-2, dominators.getImmediateDominator(rIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(aIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(hIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(iIndex));
        assertEquals(gIndex, dominators.getImmediateDominator(jIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(kIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(lIndex));
    }

    @Test
    public void testEx3DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 2));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(hIndex));
    }

    @Test
    public void testEx3DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 2));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(hIndex));
    }

    @Test
    public void testEx4DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 3));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int rIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$R")).findAny().orElse(-1);
        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);

        assertNotEquals(-1, rIndex);
        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(rIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(hIndex));
        assertEquals(hIndex, fastHeap.getImmediateDominator(iIndex));
        assertEquals(iIndex, fastHeap.getImmediateDominator(jIndex));
        assertEquals(iIndex, fastHeap.getImmediateDominator(kIndex));
        assertEquals(kIndex, fastHeap.getImmediateDominator(lIndex));
        assertEquals(rIndex, fastHeap.getImmediateDominator(mIndex));
    }

    @Test
    public void testEx4DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 3));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int rIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$R")).findAny().orElse(-1);
        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);

        assertNotEquals(-1, rIndex);
        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);

        assertEquals(-2, dominators.getImmediateDominator(rIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(hIndex));
        assertEquals(hIndex, dominators.getImmediateDominator(iIndex));
        assertEquals(iIndex, dominators.getImmediateDominator(jIndex));
        assertEquals(iIndex, dominators.getImmediateDominator(kIndex));
        assertEquals(kIndex, dominators.getImmediateDominator(lIndex));
        assertEquals(rIndex, dominators.getImmediateDominator(mIndex));
    }

    @Test
    public void testEx5DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 4));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(-2, fastHeap.getImmediateDominator(bIndex));
        assertEquals(-2, fastHeap.getImmediateDominator(cIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(hIndex));
    }

    @Test
    public void testEx5DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 4));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(-2, dominators.getImmediateDominator(bIndex));
        assertEquals(-2, dominators.getImmediateDominator(cIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(hIndex));
    }

    @Test
    public void testEx6DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 5));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(fIndex));
    }

    @Test
    public void testEx6DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 5));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(fIndex));
    }

    @Test
    public void testEx7DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 6));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(dIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(gIndex, fastHeap.getImmediateDominator(hIndex));
        assertEquals(hIndex, fastHeap.getImmediateDominator(iIndex));
        assertEquals(hIndex, fastHeap.getImmediateDominator(jIndex));
    }

    @Test
    public void testEx7DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 6));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(dIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(gIndex, dominators.getImmediateDominator(hIndex));
        assertEquals(hIndex, dominators.getImmediateDominator(iIndex));
        assertEquals(hIndex, dominators.getImmediateDominator(jIndex));
    }

    @Test
    public void testEx8DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 7));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(gIndex));
    }

    @Test
    public void testEx8DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 7));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(gIndex));
    }

    @Test
    public void testEx9DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 8));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);
        int nIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$N")).findAny().orElse(-1);
        int oIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$O")).findAny().orElse(-1);
        int pIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$P")).findAny().orElse(-1);
        int qIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$Q")).findAny().orElse(-1);
        int rIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$R")).findAny().orElse(-1);
        int sIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$S")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);
        assertNotEquals(-1, nIndex);
        assertNotEquals(-1, oIndex);
        assertNotEquals(-1, pIndex);
        assertNotEquals(-1, qIndex);
        assertNotEquals(-1, rIndex);
        assertNotEquals(-1, sIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(cIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(gIndex, fastHeap.getImmediateDominator(hIndex));
        assertEquals(hIndex, fastHeap.getImmediateDominator(iIndex));
        assertEquals(iIndex, fastHeap.getImmediateDominator(jIndex));
        assertEquals(iIndex, fastHeap.getImmediateDominator(kIndex));
        assertEquals(kIndex, fastHeap.getImmediateDominator(lIndex));
        assertEquals(kIndex, fastHeap.getImmediateDominator(mIndex));
        assertEquals(iIndex, fastHeap.getImmediateDominator(nIndex));
        assertEquals(gIndex, fastHeap.getImmediateDominator(oIndex));
        assertEquals(oIndex, fastHeap.getImmediateDominator(pIndex));
        assertEquals(oIndex, fastHeap.getImmediateDominator(qIndex));
        assertEquals(gIndex, fastHeap.getImmediateDominator(rIndex));   // there is a mistake in the original example...R has G as idom and not O
        assertEquals(rIndex, fastHeap.getImmediateDominator(sIndex));
    }

    @Test
    public void testEx9DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 8));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);
        int nIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$N")).findAny().orElse(-1);
        int oIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$O")).findAny().orElse(-1);
        int pIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$P")).findAny().orElse(-1);
        int qIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$Q")).findAny().orElse(-1);
        int rIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$R")).findAny().orElse(-1);
        int sIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$S")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);
        assertNotEquals(-1, nIndex);
        assertNotEquals(-1, oIndex);
        assertNotEquals(-1, pIndex);
        assertNotEquals(-1, qIndex);
        assertNotEquals(-1, rIndex);
        assertNotEquals(-1, sIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(cIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(gIndex, dominators.getImmediateDominator(hIndex));
        assertEquals(hIndex, dominators.getImmediateDominator(iIndex));
        assertEquals(iIndex, dominators.getImmediateDominator(jIndex));
        assertEquals(iIndex, dominators.getImmediateDominator(kIndex));
        assertEquals(kIndex, dominators.getImmediateDominator(lIndex));
        assertEquals(kIndex, dominators.getImmediateDominator(mIndex));
        assertEquals(iIndex, dominators.getImmediateDominator(nIndex));
        assertEquals(gIndex, dominators.getImmediateDominator(oIndex));
        assertEquals(oIndex, dominators.getImmediateDominator(pIndex));
        assertEquals(oIndex, dominators.getImmediateDominator(qIndex));
        assertEquals(gIndex, dominators.getImmediateDominator(rIndex));   // there is a mistake in the original example...R has G as idom and not O
        assertEquals(rIndex, dominators.getImmediateDominator(sIndex));
    }

    @Test
    public void testEx10DomTreeWithSlowAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 9));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);

        assertEquals(-2, fastHeap.getImmediateDominator(aIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(bIndex));
        assertEquals(bIndex, fastHeap.getImmediateDominator(cIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(dIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(eIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(fIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(gIndex));
        assertEquals(eIndex, fastHeap.getImmediateDominator(hIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(iIndex));
        assertEquals(iIndex, fastHeap.getImmediateDominator(jIndex));
        assertEquals(iIndex, fastHeap.getImmediateDominator(kIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(lIndex));
        assertEquals(aIndex, fastHeap.getImmediateDominator(mIndex));
    }

    @Test
    public void testEx10DomTreeWithFastAlgorithm() throws Exception {
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 9));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();

        AntTracksDominators dominators = new AntTracksDominators(fastHeap);

        int aIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$A")).findAny().orElse(-1);
        int bIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$B")).findAny().orElse(-1);
        int cIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$C")).findAny().orElse(-1);
        int dIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$D")).findAny().orElse(-1);
        int eIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$E")).findAny().orElse(-1);
        int fIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$F")).findAny().orElse(-1);
        int gIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$G")).findAny().orElse(-1);
        int hIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$H")).findAny().orElse(-1);
        int iIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$I")).findAny().orElse(-1);
        int jIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$J")).findAny().orElse(-1);
        int kIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$K")).findAny().orElse(-1);
        int lIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$L")).findAny().orElse(-1);
        int mIndex = fastHeap.stream().filter(idx -> fastHeap.getType(idx).getFullyQualifiedName().endsWith("DominatorTreeExample$M")).findAny().orElse(-1);

        assertNotEquals(-1, aIndex);
        assertNotEquals(-1, bIndex);
        assertNotEquals(-1, cIndex);
        assertNotEquals(-1, dIndex);
        assertNotEquals(-1, eIndex);
        assertNotEquals(-1, fIndex);
        assertNotEquals(-1, gIndex);
        assertNotEquals(-1, hIndex);
        assertNotEquals(-1, iIndex);
        assertNotEquals(-1, jIndex);
        assertNotEquals(-1, kIndex);
        assertNotEquals(-1, lIndex);
        assertNotEquals(-1, mIndex);

        assertEquals(-2, dominators.getImmediateDominator(aIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(bIndex));
        assertEquals(bIndex, dominators.getImmediateDominator(cIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(dIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(eIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(fIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(gIndex));
        assertEquals(eIndex, dominators.getImmediateDominator(hIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(iIndex));
        assertEquals(iIndex, dominators.getImmediateDominator(jIndex));
        assertEquals(iIndex, dominators.getImmediateDominator(kIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(lIndex));
        assertEquals(aIndex, dominators.getImmediateDominator(mIndex));
    }
}
