/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.test;

import at.jku.mevss.anttracks.util.ArraysUtil;
import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class ArraysUtil_Test {

    private static final String[] ARRAY = new String[]{"A", "B", "C", "D", "E", "F"};
    private static final String[] INSERTEE = new String[]{"X", "Y", "Z"};

    @Test
    public void testInsert() {
        String[] testee = ArraysUtil.insert(String.class, ARRAY, INSERTEE, 3);
        assertEquals(9, testee.length);
        assertEquals("A", testee[0]);
        assertEquals("B", testee[1]);
        assertEquals("C", testee[2]);
        assertEquals("X", testee[3]);
        assertEquals("Y", testee[4]);
        assertEquals("Z", testee[5]);
        assertEquals("D", testee[6]);
        assertEquals("E", testee[7]);
        assertEquals("F", testee[8]);
    }

    @Test
    public void testInsertBeginning() {
        String[] testee = ArraysUtil.insert(String.class, ARRAY, INSERTEE, 0);
        assertEquals(9, testee.length);
        assertEquals("X", testee[0]);
        assertEquals("Y", testee[1]);
        assertEquals("Z", testee[2]);
        assertEquals("A", testee[3]);
        assertEquals("B", testee[4]);
        assertEquals("C", testee[5]);
        assertEquals("D", testee[6]);
        assertEquals("E", testee[7]);
        assertEquals("F", testee[8]);
    }

    @Test
    public void testInsertEnd() {
        String[] testee = ArraysUtil.insert(String.class, ARRAY, INSERTEE, 6);
        assertEquals(9, testee.length);
        assertEquals("A", testee[0]);
        assertEquals("B", testee[1]);
        assertEquals("C", testee[2]);
        assertEquals("D", testee[3]);
        assertEquals("E", testee[4]);
        assertEquals("F", testee[5]);
        assertEquals("X", testee[6]);
        assertEquals("Y", testee[7]);
        assertEquals("Z", testee[8]);
    }

    @Test
    public void testRemove() {
        String[] testee = ArraysUtil.remove(String.class, ARRAY, 2, 2);
        assertEquals(4, testee.length);
        assertEquals("A", testee[0]);
        assertEquals("B", testee[1]);
        assertEquals("E", testee[2]);
        assertEquals("F", testee[3]);
    }

    @Test
    public void testRemoveBeginning() {
        String[] testee = ArraysUtil.remove(String.class, ARRAY, 0, 2);
        assertEquals(4, testee.length);
        assertEquals("C", testee[0]);
        assertEquals("D", testee[1]);
        assertEquals("E", testee[2]);
        assertEquals("F", testee[3]);
    }

    @Test
    public void testRemoveEnd() {
        String[] testee = ArraysUtil.remove(String.class, ARRAY, 4, 2);
        assertEquals(4, testee.length);
        assertEquals("A", testee[0]);
        assertEquals("B", testee[1]);
        assertEquals("C", testee[2]);
        assertEquals("D", testee[3]);
    }

    @Test
    public void testConcatEmpty() {
        String[] testee = ArraysUtil.concat(String.class);
        assertEquals(0, testee.length);
    }

    @Test
    public void testConcatEmpties() {
        String[] testee = ArraysUtil.concat(String.class, new String[0], new String[0]);
        assertEquals(0, testee.length);
    }

    @Test
    public void testConcatOne() {
        String[] testee = ArraysUtil.concat(String.class, new String[]{"A", "B"});
        assertEquals(2, testee.length);
        assertEquals("A", testee[0]);
        assertEquals("B", testee[1]);
    }

    @Test
    public void testConcatTwo() {
        String[] testee = ArraysUtil.concat(String.class, new String[]{"A", "B"}, new String[]{"C", "D"});
        assertEquals(4, testee.length);
        assertEquals("A", testee[0]);
        assertEquals("B", testee[1]);
        assertEquals("C", testee[2]);
        assertEquals("D", testee[3]);
    }

    @Test
    public void testConcat() {
        String[] testee = ArraysUtil.concat(String.class, new String[]{"A", "B"}, new String[]{"C"}, new String[]{}, new String[]{"D", "E", "F"});
        assertEquals(6, testee.length);
        assertEquals("A", testee[0]);
        assertEquals("B", testee[1]);
        assertEquals("C", testee[2]);
        assertEquals("D", testee[3]);
        assertEquals("E", testee[4]);
        assertEquals("F", testee[5]);
    }
}
