/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.test;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.diff.ComplexAddressCollection;
import at.jku.mevss.anttracks.diff.Interval;
import at.jku.mevss.anttracks.diff.MultipleObjectCollection;
import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.parser.EventType;
import org.junit.Assert;

import java.util.Set;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static org.junit.Assert.assertTrue;

public class Interval_Test {

    private class Move {
        ObjectID obj;
        long to;

        @SuppressWarnings("unused")
        public Move(ObjectID obj, long addr) {
            this.obj = obj;
            to = addr;
        }
    }

    final static AllocationSite site1 = new AllocationSite(1, new AllocationSite.Location[0], 1);
    final static AllocationSite site2 = new AllocationSite(2, new AllocationSite.Location[0], 2);
    final static AllocationSite site3 = new AllocationSite(3, new AllocationSite.Location[0], 3);
    final static AllocationSite site4 = new AllocationSite(4, new AllocationSite.Location[0], 4);

    final static AllocatedType type1 = new AllocatedType(1, -1, "Test", 8);
    final static AllocatedType type2 = new AllocatedType(2, -1, "Test", 8);
    final static AllocatedType type3 = new AllocatedType(3, -1, "Test", 8);
    final static AllocatedType type4 = new AllocatedType(4, -1, "Test", 8);

    private static final ObjectInfo[] OBJECT_INFOS = new ObjectInfo[]{new TestObjectInfo(EventType.OBJ_ALLOC_SLOW, site1, type1, 8, 8),
                                                                      new TestObjectInfo(EventType.OBJ_ALLOC_SLOW, site2, type2, 8, 8),
                                                                      new TestObjectInfo(EventType.OBJ_ALLOC_SLOW, site3, type3, 8, 8),
                                                                      new TestObjectInfo(EventType.OBJ_ALLOC_SLOW, site4, type4, 8, 8),};

    private static final TestObjectFactory OBJECT_FACTORY = new TestObjectFactory();

    private static final ObjectID[] OBJECTS = new ObjectID[]{OBJECT_FACTORY.generateKey(OBJECT_INFOS[0]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[1]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[2]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[3]),

                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[0]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[1]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[2]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[3]),

                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[0]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[1]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[2]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[3]),

                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[0]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[1]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[2]),
                                                             OBJECT_FACTORY.generateKey(OBJECT_INFOS[3]),};

    @SuppressWarnings("unused")
    private static final ClassifierChain oc = new ClassifierChain(new Classifier<AllocationSite>() {

        @Override
        public AllocationSite classify() {
            return allocationSite();
        }
    });

    @SuppressWarnings("unused")
    private static ObjectID[] getObjects(int from, int to) {
        ObjectID[] result = new ObjectID[to - from];
        System.arraycopy(OBJECTS, from, result, 0, to - from);
        return result;
    }

    @SuppressWarnings("unused")
    private static void assertEquals(ObjectID[] expected, ComplexAddressCollection actual) {
        Stream.of(expected).map(x -> x.addr).forEach(exp -> assertTrue(actual.contains(exp)));
    }

    @SuppressWarnings("unused")
    private static void assertEquals(ObjectID[] expected, MultipleObjectCollection actual) {
        Set<ObjectInfo> addresses = Stream.of(expected).map(x -> x.info).collect(Collectors.toSet());
        assertEquals(addresses, actual.getObjectInfos());
    }

    private static <T> void assertEquals(Set<T> expected, Set<T> actual) {
        if (expected.size() != actual.size()) {
            Assert.fail();
        } else {
            if (expected.stream().anyMatch(obj -> !actual.contains(obj))) {
                Assert.fail();
            }
        }
    }

    /*
     * @Test public <T> void test_Perm() { TestHeap heap1 = new TestHeap(null, OBJECTS); TestHeap heap2 = new TestHeap(heap1, OBJECTS);
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(OBJECTS, interval.info.perm); assertEquals(new ObjectID[0], interval.info.born); assertEquals(new ObjectID[0],
     * interval.info.died); assertEquals(new ObjectID[0], interval.info.temp);
     *
     * Assert.assertEquals(Arrays.stream(OBJECTS).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(OBJECTS).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(0, interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals(0, interval.info.born.getObjectCountClassified(2)); Assert.assertEquals(0,
     * interval.info.died.getObjectCountClassified(1)); Assert.assertEquals(0, interval.info.died.getObjectCountClassified(2));
     * Assert.assertEquals(0, interval.info.temp.getObjectCountClassified(1)); Assert.assertEquals(0,
     * interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void test_Born() { TestHeap heap1 = new TestHeap(null); TestHeap heap2 = new TestHeap(heap1, OBJECTS); TestHeap[] heaps
     * = new TestHeap[] { heap1, heap2 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(new ObjectID[0], interval.info.perm); assertEquals(OBJECTS, interval.info.born); assertEquals(new ObjectID[0],
     * interval.info.died); assertEquals(new ObjectID[0], interval.info.temp);
     *
     * Assert.assertEquals(0, interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(0,
     * interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(Arrays.stream(OBJECTS).filter(o -> o.info.allocationSiteId ==
     * 1).count(), interval.info.born.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(OBJECTS).filter(o ->
     * o.info.allocationSiteId == 2).count(), interval.info.born.getObjectCountClassified(2)); Assert.assertEquals(0,
     * interval.info.died.getObjectCountClassified(1)); Assert.assertEquals(0, interval.info.died.getObjectCountClassified(2));
     * Assert.assertEquals(0, interval.info.temp.getObjectCountClassified(1)); Assert.assertEquals(0,
     * interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void test_Died() { TestHeap heap1 = new TestHeap(null, OBJECTS); TestHeap heap2 = new TestHeap(heap1); TestHeap[] heaps
     * = new TestHeap[] { heap1, heap2 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(new ObjectID[0], interval.info.perm); assertEquals(new ObjectID[0], interval.info.born); assertEquals(OBJECTS,
     * interval.info.died); assertEquals(new ObjectID[0], interval.info.temp);
     *
     * Assert.assertEquals(0, interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(0,
     * interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(0, interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals(0, interval.info.born.getObjectCountClassified(2)); Assert.assertEquals(Arrays.stream(OBJECTS).filter(o ->
     * o.info.allocationSiteId == 1).count(), interval.info.died.getObjectCountClassified(1));
     * Assert.assertEquals(Arrays.stream(OBJECTS).filter(o -> o.info.allocationSiteId == 2).count(),
     * interval.info.died.getObjectCountClassified(2)); Assert.assertEquals(0, interval.info.temp.getObjectCountClassified(1));
     * Assert.assertEquals(0, interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void test_Temporary() { TestHeap heap1 = new TestHeap(null); TestHeap heap2 = new TestHeap(heap1, OBJECTS); TestHeap
     * heap3 = new TestHeap(heap2); TestHeap[] heaps = new TestHeap[] { heap1, heap2, heap3 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(new ObjectID[0], interval.info.perm); assertEquals(new ObjectID[0], interval.info.born); assertEquals(new ObjectID[0],
     * interval.info.died); assertEquals(OBJECTS, interval.info.temp);
     *
     * Assert.assertEquals(0, interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(0,
     * interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(0, interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals(0, interval.info.born.getObjectCountClassified(2)); Assert.assertEquals(0,
     * interval.info.died.getObjectCountClassified(1)); Assert.assertEquals(0, interval.info.died.getObjectCountClassified(2));
     * Assert.assertEquals(Arrays.stream(OBJECTS).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.temp.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(OBJECTS).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void testPyramid() { ObjectID[] objects1 = getObjects(0, 4); ObjectID[] objects2 = getObjects(4, 8); ObjectID[]
     * objects3
     * = getObjects(8, 12); ObjectID[] objects4 = getObjects(12, 16);
     *
     * TestHeap heap1 = new TestHeap(null, objects1); TestHeap heap2 = new TestHeap(heap1, objects1, objects2); TestHeap heap3 = new
     * TestHeap(heap2, objects1, objects2, objects3); TestHeap heap4 = new TestHeap(heap3, objects1, objects2, objects3, objects4);
     * TestHeap
     * heap5 = new TestHeap(heap4, objects1, objects2, objects3); TestHeap heap6 = new TestHeap(heap5, objects1, objects2); TestHeap
     * heap7 =
     * new TestHeap(heap6, objects1);
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2, heap3, heap4, heap5, heap6, heap7 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(objects1, interval.info.perm); assertEquals(new ObjectID[0], interval.info.born); assertEquals(new ObjectID[0],
     * interval.info.died); assertEquals(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4), interval.info.temp);
     *
     * Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(0, interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals(0, interval.info.born.getObjectCountClassified(2)); Assert.assertEquals(0,
     * interval.info.died.getObjectCountClassified(1)); Assert.assertEquals(0, interval.info.died.getObjectCountClassified(2));
     * Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4)) .filter(o ->
     * o.info.allocationSiteId == 1).count(), interval.info.temp.getObjectCountClassified(1));
     * Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4)) .filter(o ->
     * o.info.allocationSiteId == 2).count(), interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void testPyramidLeft() { ObjectID[] objects1 = getObjects(0, 4); ObjectID[] objects2 = getObjects(4, 8); ObjectID[]
     * objects3 = getObjects(8, 12); ObjectID[] objects4 = getObjects(12, 16);
     *
     * TestHeap heap1 = new TestHeap(null, objects1); TestHeap heap2 = new TestHeap(heap1, objects1, objects2); TestHeap heap3 = new
     * TestHeap(heap2, objects1, objects2, objects3); TestHeap heap4 = new TestHeap(heap3, objects1, objects2, objects3, objects4);
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2, heap3, heap4 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(objects1, interval.info.perm); assertEquals(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4),
     * interval.info.born); assertEquals(new ObjectID[0], interval.info.died); assertEquals(new ObjectID[0], interval.info.temp);
     *
     * Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class,
     * objects2, objects3, objects4)) .filter(o -> o.info.allocationSiteId == 1).count(), interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4)) .filter(o ->
     * o.info.allocationSiteId == 2).count(), interval.info.born.getObjectCountClassified(2)); Assert.assertEquals(0,
     * interval.info.died.getObjectCountClassified(1)); Assert.assertEquals(0, interval.info.died.getObjectCountClassified(2));
     * Assert.assertEquals(0, interval.info.temp.getObjectCountClassified(1)); Assert.assertEquals(0,
     * interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void testPyramidRight() { ObjectID[] objects1 = getObjects(0, 4); ObjectID[] objects2 = getObjects(4, 8); ObjectID[]
     * objects3 = getObjects(8, 12); ObjectID[] objects4 = getObjects(12, 16);
     *
     * TestHeap heap1 = new TestHeap(null, objects1, objects2, objects3, objects4); TestHeap heap2 = new TestHeap(heap1, objects1,
     * objects2,
     * objects3); TestHeap heap3 = new TestHeap(heap2, objects1, objects2); TestHeap heap4 = new TestHeap(heap3, objects1);
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2, heap3, heap4 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(objects1, interval.info.perm); assertEquals(new ObjectID[0], interval.info.born);
     * assertEquals(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4), interval.info.died); assertEquals(new ObjectID[0],
     * interval.info.temp);
     *
     * Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(0, interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals(0, interval.info.born.getObjectCountClassified(2));
     * Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4)) .filter(o ->
     * o.info.allocationSiteId == 1).count(), interval.info.died.getObjectCountClassified(1));
     * Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class, objects2, objects3, objects4)) .filter(o ->
     * o.info.allocationSiteId == 2).count(), interval.info.died.getObjectCountClassified(2)); Assert.assertEquals(0,
     * interval.info.temp.getObjectCountClassified(1)); Assert.assertEquals(0, interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void testOneAndTwoHalfPyramids() { ObjectID[] objects1 = getObjects(0, 4); ObjectID[] objects2 = getObjects(4, 8);
     * ObjectID[] objects3 = getObjects(8, 12); ObjectID[] objects4 = getObjects(12, 16);
     *
     * TestHeap heap1 = new TestHeap(null, objects1, objects2); TestHeap heap2 = new TestHeap(heap1, objects1); TestHeap heap3 = new
     * TestHeap(heap2, objects1, objects3); TestHeap heap4 = new TestHeap(heap3, objects1); TestHeap heap5 = new TestHeap(heap4, objects1,
     * objects4);
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2, heap3, heap4, heap5 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(objects1, interval.info.perm); assertEquals(objects4, interval.info.born); assertEquals(objects2, interval.info.died);
     * assertEquals(objects3, interval.info.temp);
     *
     * Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(objects1).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals(Arrays.stream(objects4).filter(o ->
     * o.info.allocationSiteId == 1).count(), interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals(Arrays.stream(objects4).filter(o -> o.info.allocationSiteId == 2).count(),
     * interval.info.born.getObjectCountClassified(2)); Assert.assertEquals(Arrays.stream(objects2).filter(o -> o.info.allocationSiteId ==
     * 1).count(), interval.info.died.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(objects2).filter(o ->
     * o.info.allocationSiteId == 2).count(), interval.info.died.getObjectCountClassified(2));
     * Assert.assertEquals(Arrays.stream(objects3).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.temp.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(objects3).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void testLine() { ObjectID[] objects1 = getObjects(0, 2); ObjectID[] objects2 = getObjects(2, 4); ObjectID[] objects3 =
     * getObjects(4, 6); ObjectID[] objects4 = getObjects(6, 7); ObjectID[] objects5 = getObjects(7, 10); ObjectID[] objects6 =
     * getObjects(10, 13); ObjectID[] objects7 = getObjects(13, 15); ObjectID[] objects8 = getObjects(16, 16);
     *
     * TestHeap heap1 = new TestHeap(null, objects1, objects2, objects3); TestHeap heap2 = new TestHeap(heap1, objects1, objects2,
     * objects4); TestHeap heap3 = new TestHeap(heap2, objects5, objects2, objects4); TestHeap heap4 = new TestHeap(heap3, objects5,
     * objects2, objects4); TestHeap heap5 = new TestHeap(heap4, objects5, objects2, objects4); TestHeap heap6 = new TestHeap(heap5,
     * objects6, objects2, objects7); TestHeap heap7 = new TestHeap(heap6, objects6, objects2, objects8);
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2, heap3, heap4, heap5, heap6, heap7 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(objects2, interval.info.perm); assertEquals(ArraysUtil.concat(ObjectID.class, objects6, objects8), interval.info.born);
     * assertEquals(ArraysUtil.concat(ObjectID.class, objects1, objects3), interval.info.died);
     * assertEquals(ArraysUtil.concat(ObjectID.class, objects4, objects5, objects7), interval.info.temp);
     *
     * Assert.assertEquals(Arrays.stream(objects2).filter(o -> o.info.allocationSiteId == 1).count(),
     * interval.info.perm.getObjectCountClassified(1)); Assert.assertEquals(Arrays.stream(objects2).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.perm.getObjectCountClassified(2)); Assert.assertEquals( Arrays.stream(ArraysUtil.concat(ObjectID.class,
     * objects6, objects8)).filter(o -> o.info.allocationSiteId == 1).count(), interval.info.born.getObjectCountClassified(1));
     * Assert.assertEquals( Arrays.stream(ArraysUtil.concat(ObjectID.class, objects6, objects8)).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.born.getObjectCountClassified(2)); Assert.assertEquals( Arrays.stream(ArraysUtil.concat(ObjectID.class,
     * objects1, objects3)).filter(o -> o.info.allocationSiteId == 1).count(), interval.info.died.getObjectCountClassified(1));
     * Assert.assertEquals( Arrays.stream(ArraysUtil.concat(ObjectID.class, objects1, objects3)).filter(o -> o.info.allocationSiteId ==
     * 2).count(), interval.info.died.getObjectCountClassified(2)); Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class,
     * objects4, objects5, objects7)) .filter(o -> o.info.allocationSiteId == 1).count(), interval.info.temp.getObjectCountClassified(1));
     * Assert.assertEquals(Arrays.stream(ArraysUtil.concat(ObjectID.class, objects4, objects5, objects7)) .filter(o ->
     * o.info.allocationSiteId == 2).count(), interval.info.temp.getObjectCountClassified(2)); }
     *
     * @Test public void testEmpty() { TestHeap heap1 = new TestHeap(null); TestHeap heap2 = new TestHeap(heap1); TestHeap heap3 = new
     * TestHeap(heap2);
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2, heap3 };
     *
     * Interval interval = new Interval(heaps[0]); interval.info.setClassifier(oc); for (int i = 1; i < heaps.length; i++) {
     * interval.add(heaps[i]); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(new ObjectID[0], interval.info.perm); assertEquals(new ObjectID[0], interval.info.born); assertEquals(new ObjectID[0],
     * interval.info.died); assertEquals(new ObjectID[0], interval.info.temp); }
     *
     * @Test public void test_SingleMove() throws CloneNotSupportedException { ObjectID objectBeforeMove = OBJECTS[0]; ObjectID
     * objectAfterMove = OBJECTS[0].clone();
     *
     * TestHeap heap1 = new TestHeap(null, new ObjectID[] { objectBeforeMove }); TestHeap heap2 = new TestHeap(heap1, new ObjectID[] {
     * objectAfterMove });
     *
     * Move[][] moves = new Move[][] { new Move[] { new Move(objectAfterMove, 1) } };
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2 };
     *
     * boolean start = true; Interval interval = new Interval(heaps[0], 0, start); interval.info.setClassifier(oc); for (int i = 1; i <
     * heaps.length; i++) { start = !start; TestHeap curHeap = heaps[i]; if (!start) { Stream.of(moves[i - 1]).forEach(move ->
     * exectuteMove(interval, curHeap, move)); } interval.add(curHeap, 0, start); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(new ObjectID[] { objectAfterMove }, interval.info.perm); assertEquals(new ObjectID[0], interval.info.born);
     * assertEquals(new ObjectID[0], interval.info.died); assertEquals(new ObjectID[0], interval.info.temp); }
     *
     * @Test public void test_OverwriteMove() throws CloneNotSupportedException { ObjectID objectBeforeMove =
     * OBJECT_FACTORY.generateKey(OBJECT_INFOS[0]); ObjectID overwrittenObject = OBJECT_FACTORY.generateKey(OBJECT_INFOS[0]); ObjectID
     * objectAfterMove = objectBeforeMove.clone();
     *
     * ObjectID[] objects1 = new ObjectID[] { objectBeforeMove, overwrittenObject }; ObjectID[] objects2 = new ObjectID[] { objectAfterMove
     * }; TestHeap heap1 = new TestHeap(null, objects1); TestHeap heap2 = new TestHeap(heap1, objects2);
     *
     * Move[][] moves = new Move[][] { new Move[] { new Move(objectAfterMove, overwrittenObject.addr) } };
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2 };
     *
     * boolean start = true; Interval interval = new Interval(heaps[0], 0, start); interval.info.setClassifier(oc); for (int i = 1; i <
     * heaps.length; i++) { start = !start; TestHeap curHeap = heaps[i]; if (!start) { Stream.of(moves[i - 1]).forEach(move ->
     * exectuteMove(interval, curHeap, move)); } interval.add(curHeap, 0, start); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(new ObjectID[] { objectAfterMove }, interval.info.perm); assertEquals(new ObjectID[0], interval.info.born);
     * assertEquals(new ObjectID[] { overwrittenObject }, interval.info.died); assertEquals(new ObjectID[0], interval.info.temp); }
     *
     * @Test public void test_SingleMoveAndBorn() throws CloneNotSupportedException { ObjectID objectBeforeMove = OBJECTS[0]; ObjectID
     * objectAfterMove = OBJECTS[0].clone(); ObjectID objectAssigned = OBJECTS[1];
     *
     * TestHeap heap1 = new TestHeap(null, new ObjectID[] { objectBeforeMove }); TestHeap heap2 = new TestHeap(heap1, new ObjectID[] {
     * objectAfterMove, objectAssigned });
     *
     * Move[][] moves = new Move[][] { new Move[] { new Move(objectAfterMove, 1000) } };
     *
     * TestHeap[] heaps = new TestHeap[] { heap1, heap2 };
     *
     * boolean start = true; Interval interval = new Interval(heaps[0], 0, start); interval.info.setClassifier(oc); for (int i = 1; i <
     * heaps.length; i++) { start = !start; TestHeap curHeap = heaps[i]; if (!start) { Stream.of(moves[i - 1]).forEach(move ->
     * exectuteMove(interval, curHeap, move)); } interval.add(curHeap, 0, start); } interval.setLastHeap(heaps[heaps.length - 1]);
     *
     * assertEquals(new ObjectID[] { objectAfterMove }, interval.info.perm); assertEquals(new ObjectID[] { objectAssigned },
     * interval.info.born); assertEquals(new ObjectID[0], interval.info.died); assertEquals(new ObjectID[0], interval.info.temp); }
     */

    @SuppressWarnings("unused")
    private void exectuteMove(Interval interval, TestHeap heap, Move move) {
        interval.updateAddress(move.obj.addr, move.to);
        move.obj.addr = move.to;
    }
}
