/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.anttracks.parser.test;

import at.jku.mevss.anttracks.heap.objects.ObjectInfo;
import at.jku.mevss.anttracks.heap.roots.RootPtr;

import java.util.Arrays;
import java.util.List;

public final class ObjectID {

    public long addr;
    public long[] pointedFrom;
    public long[] pointsTo;
    public List<RootPtr> rootPtrs;
    public final ObjectInfo info;

    public ObjectID(long addr, long[] pointedFrom, long[] pointsTo, ObjectInfo info) {
        this.addr = addr;
        this.pointedFrom = pointedFrom;
        this.pointsTo = pointsTo;
        this.info = info;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        ObjectID other = (ObjectID) obj;
        if (addr != other.addr) {
            return false;
        }
        if (info == null) {
            if (other.info != null) {
                return false;
            }
        } else if (!info.equals(other.info)) {
            return false;
        }
        if (!Arrays.equals(pointsTo, other.pointsTo)) {
            return false;
        }
        if (!Arrays.equals(pointedFrom, other.pointedFrom)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        // return Long.hashCode(hash) ^ info.hashCode();
        return addr < 0 ? info.hashCode() : Long.hashCode(addr);
        // return Long.hashCode(hash);
    }

    @Override
    public String toString() {
        return Long.toHexString(addr) + " (" + info.hashCode() + ")";
    }

    @Override
    protected ObjectID clone() throws CloneNotSupportedException {
        ObjectID cloned = new ObjectID(addr, pointedFrom.clone(), pointsTo.clone(), info);
        return cloned;
    }

}
