package at.jku.mevss.scala.trace.parser.test

import java.io.{File, FileNotFoundException}

class Grouping_Test extends JUnitSuite {

    // Global test data
    val resourcePath = "test/resources"
    val typeClassifier = new Classifier with Single[String] {
        val name = "Type"

        def classify(address: Long, space: SpaceInfo, objType: AllocatedType, size: Integer, isArray: Boolean, arrayLength: Integer,
                     allocationSite: AllocationSite, pointedFrom: Array[Long], pointsTo: Array[Long], eventType: EventType) = objType.name
    }

    // Setup
    val symbolsFile = new File(resourcePath)
    System.err.println("Looking for symbols file in;" + symbolsFile.getAbsolutePath)
    if (!symbolsFile.exists()) {
        throw new FileNotFoundException("Symbols file could not be found at;" + symbolsFile.getAbsolutePath())
    }

    // ---- Create index file
    val symbols = new SymbolsParser(symbolsFile, null).parse()
    new HeapTraceParser(symbols,
        new MetaDataWriterConfig(resourcePath + File.separator + Consts.ANT_META_DIRECTORY, 100, false)).parse()

    @Test
    def test_PermDiedCreation() = {
        val grouping = diff(EventType.GC_END, 0, EventType.GC_END, 2)
        System.err.println(grouping)
        assert(100000 == grouping.born.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.temp.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_PermDiedCreation successful")
    }

    @Test
    def test_Gc1EndToGc2End() = {
        val grouping = diff(EventType.GC_END, 1, EventType.GC_END, 2)
        System.out.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1EndToGc2End successful")
    }

    @Test
    def test_Gc1EndToGc3End() = {
        val grouping = diff(EventType.GC_END, 1, EventType.GC_END, 3)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1EndToGc3End successful")
    }

    @Test
    def test_Gc1EndToGc4End() = {
        val grouping = diff(EventType.GC_END, 1, EventType.GC_END, 4)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.temp.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1EndToGc4End successful")
    }

    @Test
    def test_Gc1EndToGc5End() = {
        val grouping = diff(EventType.GC_END, 1, EventType.GC_END, 5)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.temp.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1EndToGc5End successful")
    }

    @Test
    def test_Gc1StartToGc2End() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_END, 2)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc2End successful")
    }

    @Test
    def test_Gc1StartToGc3End() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_END, 3)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc3End successful")
    }

    @Test
    def test_Gc1StartToGc4End() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_END, 4)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.temp.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc4End successful")
    }

    @Test
    def test_Gc1StartToGc5End() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_END, 5)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.temp.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc5End successful")
    }

    @Test
    def test_Gc1StartToGc2Start() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_START, 2)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.perm.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc2Start successful")
    }

    @Test
    def test_Gc1StartToGc3Start() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_START, 3)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc3Start successful")
    }

    @Test
    def test_Gc1StartToGc4Start() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_START, 4)
        System.out.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc4Start successful")
    }

    @Test
    def test_Gc1StartToGc5Start() = {
        val grouping = diff(EventType.GC_START, 1, EventType.GC_START, 5)
        // System.err.println(grouping)
        assert(100000 == grouping.perm.get.->("Lexample02/perm/Perm;").get.summedAmount)
        assert(100000 == grouping.born.get.->("Lexample02/born/Born;").get.summedAmount)
        assert(100000 == grouping.temp.get.->("Lexample02/temp/Temp;").get.summedAmount)
        assert(100000 == grouping.died.get.->("Lexample02/died/Died;").get.summedAmount)
        System.err.println("Test test_Gc1StartToGc5Start successful")
    }

    private def diff(startEvent: EventType, startGcNr: Int, endEvent: EventType, endGcNr: Int): DiffGrouping = {
        val diffParser = new HeapDiffTraceParser(symbols,
            new MetaDataConfig(resourcePath + File.separator + Consts.ANT_META_DIRECTORY),
            new GarbageCollectionLookup(startEvent, GarbageCollectionType.MAJOR, symbols.causes.getByName("System.gc()"), startGcNr),
            new GarbageCollectionLookup(endEvent, GarbageCollectionType.MAJOR, symbols.causes.getByName("System.gc()"), endGcNr))
        return Interop.classify(diffParser.parse().getIntervals().get(0).info, List(typeClassifier))
    }
}