package at.jku.mevss.trace.gui.chart.base;

import at.jku.mevss.anttracks.util.ApplicationStatistics;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.PlatformUtil;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.CheckBox;
import javafx.scene.layout.BorderPane;

import java.awt.*;
import java.util.logging.Logger;

public abstract class AntChartPane<DS, DATA> extends BorderPane {

    private enum ClickAction {
        ZOOMING,
        DRAGGING,
        SELECTING
    }

    protected Logger LOGGER = Logger.getLogger(this.getClass().getSimpleName());

    @FXML
    protected CheckBox zoomSynchronizedCheckbox;
    @FXML
    protected Button configureButton;
    private static int nextId = 0;

    protected ChartSynchronizer chartSynchronizer;
    protected boolean chartSynchronizerAvailable;

    private BooleanProperty isZoomSynchronizedProperty = new SimpleBooleanProperty(true);

    public BooleanProperty isZoomSynchronizedProperty() {
        return isZoomSynchronizedProperty;
    }

    public boolean isZoomSynchronized() {
        return isZoomSynchronizedProperty.get();
    }

    public void setIsZoomSynchronized(boolean b) {
        isZoomSynchronizedProperty.set(b);
    }

    public AntChartPane() {
        FXMLUtil.load(this, AntChartPane.class);
        /*
        String color = "";
        // create random object - reuse this as often as possible
        Random random = new Random();

        // create a big random number - maximum is ffffff (hex) = 16777215 (dez)
        int nextInt = random.nextInt(256*256*256);

        // format it as hexadecimal string (with hashtag and leading zeros)
        String colorCode = String.format("#%06x", nextInt);
        setStyle("-fx-background-color: " + colorCode);
        */
    }

    public void init(ChartSynchronizer chartSynchronizer) {
        chartSynchronizerAvailable = true;
        this.chartSynchronizer = chartSynchronizer;
        if (this.chartSynchronizer == null) {
            this.chartSynchronizer = new ChartSynchronizer();
            chartSynchronizerAvailable = false;
        }

        initializeChart();
        initializeZoomSynchronizedCheckBox(chartSynchronizerAvailable);
        initializeConfigureButton();
        initializeSynchronizerListeners();
    }

    private void initializeSynchronizerListeners() {
        chartSynchronizer.addSelectionListener(new ChartSynchronizer.SelectionListener() {

            @Override
            public void inCompareModeChanged(ChartSynchronizer sender) {
                updateMarker();
            }

            @Override
            public void selectionChanged(ChartSynchronizer sender) {
                updateMarker();
            }

        });
        chartSynchronizer.addZoomListener(new ChartSynchronizer.ZoomListener() {

            @Override
            public void zoomChanged(ChartSynchronizer sender) {
                //System.out.println(String.format("Zoom: %10.3f to %10.3f", sender.getZoomXLowerBound(), sender.getZoomXUpperBound()));
                zoomX(sender.getZoomXLowerBound(), sender.getZoomXUpperBound());
            }

            @Override
            public void domainChanged(ChartSynchronizer sender) {}

        });
    }

    public void update(DATA data) {
        ApplicationStatistics.Measurement measureUpdate = ApplicationStatistics.getInstance().createMeasurement("Charts: Update - " + getClass().getSimpleName());
        DS dataset;
        if (createsNewDatasetOnUpdate()) {
            dataset = createDataSet(data);
            dataset = getScaler().postProcessDataset(dataset);
            DS finalDataset = dataset;
            PlatformUtil.runAndWait(() -> {
                ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("Charts: Update Chart On UI - " + getClass().getSimpleName());
                updateChartOnUIThread(finalDataset);
                m.end();
            });

        } else {
            dataset = updateDataSet(data);
        }

        postDataSetUpdate();
        measureUpdate.end();
    }

    public Color[] getSeriesColors() {
        return null;
    }

    protected abstract void updateChartOnUIThread(DS dataset);

    protected abstract boolean createsNewDatasetOnUpdate();

    protected abstract DS createDataSet(DATA data);

    protected abstract DS updateDataSet(DATA data);

    protected abstract void postDataSetUpdate();

    public abstract void zoomX(double lowerBound, double upperBound);

    protected abstract void initializeChart();

    protected abstract EventHandler<ActionEvent> getOnConfigureAction();

    public abstract double getMinYValue(DS dataset);

    public abstract double getMaxYValue(DS dataset);

    public abstract void setXLabel(String label);

    public abstract void setYLabel(String label);

    protected void initializeConfigureButton() {
        configureButton.setOnAction(getOnConfigureAction());
        configureButton.setVisible(configureButton.getOnAction() != null);
    }

    protected void initializeZoomSynchronizedCheckBox(boolean available) {
        if (available) {
            isZoomSynchronizedProperty.bind(zoomSynchronizedCheckbox.selectedProperty());
            isZoomSynchronizedProperty.addListener((observableValue, oldVal, newVal) -> {
                if (!oldVal && newVal && chartSynchronizerAvailable) {
                    // checked synchronize box -> zoom immediately to current synchronized level
                    zoomX(chartSynchronizer.getZoomXLowerBound(), chartSynchronizer.getZoomXUpperBound());
                }
            });
        } else {
            zoomSynchronizedCheckbox.setVisible(false);
        }
    }

    // -----------------------------------------------
    // ---------- ScaleAdjustable --------------------
    // -----------------------------------------------

    public AxisScaler<DS> getScaler() {
        return new AxisScaler.NoAxisScaler<>();
    }

    // -----------------------------------------------
    // -----------------------------------------------
    // -----------------------------------------------

    /**
     * Update current selection, ... (called via syncer)
     */
    public abstract void updateMarker();

    public String getPaneId() {
        return String.valueOf(hashCode());
    }
}
