/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.chart.base;

import at.jku.mevss.trace.gui.chart.jfreechart.xy.line.gc.GCJFreeChart;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.alivedead.AliveDeadStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.allocatingsubsystem.AllocatingSubsystemStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.feature.FeatureStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.memory.MemoryStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.objectkind.ObjectKindStackedAreaJFreeChart;
import at.jku.mevss.trace.gui.model.AppInfo;

public class ApplicationChartFactory {

    public static enum ApplicationChartType {
        OBJECTS,
        BYTES,
        GC_TYPES,
        ALIVE_DEAD_BYTES,
        ALIVE_DEAD_OBJECTS,
        ALIVE_DEAD_RELATIVE,
        FEATURE_OBJECTS,
        FEATURE_BYTES,
        ALLOCATING_SUBSYSTEM,
        OBJECT_KINDS,
    }

    public static enum MemoryConsumptionUnit {
        BYTES("Memory", "# Bytes"),
        OBJECTS("Objects", "# Objects");

        private final String label;
        private final String unit;

        private MemoryConsumptionUnit(String label, String unit) {
            this.label = label;
            this.unit = unit;
        }

        public String getLabel() {
            return label;
        }

        public String getUnit() {
            return unit;
        }
    }

    public static enum FeatureMemoryConsumptionUnit {
        BYTES("Memory", "# Bytes"),
        OBJECTS("Objects", "# Objects");

        private final String label;
        private final String unit;

        private FeatureMemoryConsumptionUnit(String label, String unit) {
            this.label = label;
            this.unit = unit;
        }

        public String getLabel() {
            return label;
        }

        public String getUnit() {
            return unit;
        }
    }

    public static enum AliveDeadMemoryConsumptionUnit {
        BYTES("# Bytes"),
        OBJECTS("# Objects"),
        RELATIVE("% Objects");

        private final String unit;

        private AliveDeadMemoryConsumptionUnit(String unit) {
            this.unit = unit;
        }

        public String getUnit() {
            return unit;
        }
    }

    private static AntChartPane<?, AppInfo> createBytesChart(ChartSynchronizer chartSynchronizer) {
        MemoryStackedAreaJFreeChart chart = new MemoryStackedAreaJFreeChart();
        chart.init(chartSynchronizer, MemoryConsumptionUnit.BYTES);
        return chart;
    }

    private static AntChartPane<?, AppInfo> createObjectsChart(ChartSynchronizer chartSynchronizer) {
        MemoryStackedAreaJFreeChart chart = new MemoryStackedAreaJFreeChart();
        chart.init(chartSynchronizer, MemoryConsumptionUnit.OBJECTS);
        return chart;
    }

    private static AntChartPane<?, AppInfo> createGCTypesChart(ChartSynchronizer chartSynchronizer) {
        GCJFreeChart chart = new GCJFreeChart();
        chart.init(chartSynchronizer);
        return chart;
    }

    private static AntChartPane<?, AppInfo> createAliveDeadBytesChart(ChartSynchronizer chartSynchronizer) {
        AliveDeadStackedAreaJFreeChart chartPanel = new AliveDeadStackedAreaJFreeChart();
        chartPanel.init(chartSynchronizer, AliveDeadMemoryConsumptionUnit.BYTES);
        return chartPanel;
    }

    private static AntChartPane<?, AppInfo> createAliveDeadObjectsChart(ChartSynchronizer chartSynchronizer) {
        AliveDeadStackedAreaJFreeChart chartPanel = new AliveDeadStackedAreaJFreeChart();
        chartPanel.init(chartSynchronizer, AliveDeadMemoryConsumptionUnit.OBJECTS);
        return chartPanel;
    }

    private static AntChartPane<?, AppInfo> createAliveDeadRelativeChart(ChartSynchronizer chartSynchronizer) {
        AliveDeadStackedAreaJFreeChart chartPanel = new AliveDeadStackedAreaJFreeChart();
        chartPanel.init(chartSynchronizer, AliveDeadMemoryConsumptionUnit.RELATIVE);
        return chartPanel;
    }

    private static AntChartPane<?, AppInfo> createFeatureObjectsChart(ChartSynchronizer chartSynchronizer) {
        FeatureStackedAreaJFreeChart chartPanel = new FeatureStackedAreaJFreeChart();
        chartPanel.init(chartSynchronizer, FeatureMemoryConsumptionUnit.OBJECTS);
        return chartPanel;
    }

    private static AntChartPane<?, AppInfo> createFeatureBytesChart(ChartSynchronizer chartSynchronizer) {
        FeatureStackedAreaJFreeChart chartPanel = new FeatureStackedAreaJFreeChart();
        chartPanel.init(chartSynchronizer, FeatureMemoryConsumptionUnit.BYTES);
        return chartPanel;
    }

    private static AntChartPane<?, AppInfo> createAllocatingSubsystemChart(ChartSynchronizer chartSynchronizer) {
        AllocatingSubsystemStackedAreaJFreeChart allocatingSubsystemApplicationChart = new AllocatingSubsystemStackedAreaJFreeChart();
        allocatingSubsystemApplicationChart.init(chartSynchronizer);
        return allocatingSubsystemApplicationChart;
    }

    private static AntChartPane<?, AppInfo> createObjectKindsChart(ChartSynchronizer chartSynchronizer) {
        ObjectKindStackedAreaJFreeChart objectKindsApplicationChart = new ObjectKindStackedAreaJFreeChart();
        objectKindsApplicationChart.init(chartSynchronizer);
        return objectKindsApplicationChart;
    }

    public static AntChartPane<?, AppInfo> createChart(ApplicationChartType type, ChartSynchronizer chartSynchronizer) {
        switch (type) {
            case OBJECTS:
                return createObjectsChart(chartSynchronizer);
            case BYTES:
                return createBytesChart(chartSynchronizer);
            case GC_TYPES:
                return createGCTypesChart(chartSynchronizer);
            case ALIVE_DEAD_BYTES:
                return createAliveDeadBytesChart(chartSynchronizer);
            case ALIVE_DEAD_OBJECTS:
                return createAliveDeadObjectsChart(chartSynchronizer);
            case ALIVE_DEAD_RELATIVE:
                return createAliveDeadRelativeChart(chartSynchronizer);
            case FEATURE_OBJECTS:
                return createFeatureObjectsChart(chartSynchronizer);
            case FEATURE_BYTES:
                return createFeatureBytesChart(chartSynchronizer);
            case ALLOCATING_SUBSYSTEM:
                return createAllocatingSubsystemChart(chartSynchronizer);
            case OBJECT_KINDS:
                return createObjectKindsChart(chartSynchronizer);
            default:
                throw new IllegalArgumentException("Chart Type not supported: " + type.toString());
        }
    }
}
