package at.jku.mevss.trace.gui.chart.fxchart.xy.stackedareapane;

import at.jku.mevss.anttracks.heap.statistics.Statistics;
import at.jku.mevss.trace.gui.chart.base.ApplicationChartFactory;
import at.jku.mevss.trace.gui.chart.base.AxisScaler;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.fxchart.xy.stackedareapane.base.StackedAreaJavaFXChartPane;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.application.Platform;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.scene.chart.XYChart;

public class MemoryApplicationStackedAreaChart extends StackedAreaJavaFXChartPane<AppInfo> {

    XYChart.Series<Number, Number> edenSeries;
    XYChart.Series<Number, Number> survivorSeries;
    XYChart.Series<Number, Number> oldSeries;

    private ApplicationChartFactory.MemoryConsumptionUnit memoryUnit;

    public MemoryApplicationStackedAreaChart() {
        FXMLUtil.load(this, MemoryApplicationStackedAreaChart.class);
    }

    public void init(ChartSynchronizer chartSynchronizer, ApplicationChartFactory.MemoryConsumptionUnit memoryUnit) {
        super.init(chartSynchronizer);
        this.memoryUnit = memoryUnit;
        chart.setTitle(memoryUnit.getLabel());
    }

    @Override
    protected ObservableList<XYChart.Series<Number, Number>> updateDataSet(AppInfo appInfo) {
        double maxX = getMaxXValue(chart.getData());
        for (int i = 0; i < appInfo.getStatistics().size(); i++) {
            Statistics stat = appInfo.getStatistics().get(i);
            double time = stat.info.time;

            if (time > maxX) {
                XYChart.Data<Number, Number> edenData;
                XYChart.Data<Number, Number> survivorData;
                XYChart.Data<Number, Number> oldData;

                switch (memoryUnit) {
                    case BYTES:
                        edenData = new XYChart.Data<>(time, stat.eden.memoryConsumption.getBytes());
                        survivorData = new XYChart.Data<>(time, stat.survivor.memoryConsumption.getBytes());
                        oldData = new XYChart.Data<>(time, stat.old.memoryConsumption.getBytes());
                        break;
                    case OBJECTS:
                        edenData = new XYChart.Data<>(time, stat.eden.memoryConsumption.getObjects());
                        survivorData = new XYChart.Data<>(time, stat.survivor.memoryConsumption.getObjects());
                        oldData = new XYChart.Data<>(time, stat.old.memoryConsumption.getObjects());
                        break;
                    default:
                        throw new IllegalStateException();
                }

                Platform.runLater(() -> {
                    edenSeries.getData().add(edenData);
                    survivorSeries.getData().add(survivorData);
                    oldSeries.getData().add(oldData);
                });
            }
        }

        return chart.getData();
    }

    @Override
    public AxisScaler<ObservableList<XYChart.Series<Number, Number>>> getScaler() {
        switch (memoryUnit) {
            case OBJECTS:
                return new AxisScaler.JavaFXAxisScaler.ObjectsScaler(this);
            case BYTES:
                return new AxisScaler.JavaFXAxisScaler.BytesScaler(this);
            default:
                LOGGER.warning("No scaler defined for " + memoryUnit);
                return new AxisScaler.JavaFXAxisScaler.NoAxisScaler(this);
        }
    }

    @Override
    protected void initializeChartSeries() {
        edenSeries = new XYChart.Series<>();
        edenSeries.setName("Eden");
        survivorSeries = new XYChart.Series<>();
        survivorSeries.setName("Survivor");
        oldSeries = new XYChart.Series<>();
        oldSeries.setName("Old");
        chart.getData().add(edenSeries);
        chart.getData().add(survivorSeries);
        chart.getData().add(oldSeries);
    }

    @Override
    protected void updateDataLabelContent() {

    }

    @Override
    protected EventHandler<ActionEvent> getOnConfigureAction() {
        return null;
    }
}
