package at.jku.mevss.trace.gui.chart.jfreechart.base;

import at.jku.mevss.trace.gui.model.ChartSelection;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.EntityCollection;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.fx.ChartCanvas;
import org.jfree.chart.fx.ChartViewer;
import org.jfree.chart.fx.interaction.AbstractMouseHandlerFX;
import org.jfree.chart.fx.interaction.PanHandlerFX;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer2;

public class AntChartViewer extends ChartViewer {

    private boolean alreadyAddedListeners = false;

    private class LineChartMouseListener extends AbstractMouseHandlerFX {
        public LineChartMouseListener() {
            super("ClickListener", false, false, false, false);
        }

        @Override
        public void handleMouseClicked(ChartCanvas canvas, MouseEvent e) {
            if (e.getButton().equals(MouseButton.PRIMARY) && getChart() instanceof AntJFreeChart && ((AntJFreeChart) getChart()).getSynchronizer() != null) {
                ChartRenderingInfo info = canvas.getRenderingInfo();
                if (info == null) {
                    return;
                }
                EntityCollection entities = info.getEntityCollection();
                if (entities == null) {
                    return;
                }
                ChartEntity entity = entities.getEntity(e.getX(), e.getY());
                if (entity == null) {
                    return;
                }

                if (entity != null && (entity instanceof XYItemEntity)) {
                    XYPlot plot = getChart().getXYPlot();

                    XYItemEntity item = (XYItemEntity) entity;
                    int series = item.getSeriesIndex();
                    int itemId = item.getItem();
                    double time = item.getDataset().getXValue(series, itemId);
                    double height = item.getDataset().getYValue(series, itemId);
                    // Check if we have stacked renderer. If so, we must
                    // calculate the summed height of all series below the selected
                    // one.
                    StackedXYAreaRenderer2 stackedRenderer = null;
                    for (int i = 0; i < plot.getRendererCount(); i++) {
                        if (plot.getRenderer(i) instanceof StackedXYAreaRenderer2) {
                            stackedRenderer = (StackedXYAreaRenderer2) plot.getRenderer(i);
                        }
                    }
                    if (stackedRenderer != null) {
                        for (int seriesIndex = 0; seriesIndex < series; seriesIndex++) {
                            height += item.getDataset().getYValue(seriesIndex, itemId);
                        }
                    }

                    ChartSelection selection = new ChartSelection(series, itemId, time, height, ((AntJFreeChart) getChart()).getChartID());

                    ((AntJFreeChart) getChart()).getSynchronizer().sync(selection);
                }
            }
        }

        @Override
        public void handleMouseMoved(ChartCanvas canvas, MouseEvent e) {
            if (getChart() instanceof AntJFreeChart) {
                ChartRenderingInfo info = canvas.getRenderingInfo();
                if (info == null) {
                    return;
                }
                EntityCollection entities = info.getEntityCollection();
                if (entities == null) {
                    return;
                }
                ChartEntity entity = entities.getEntity(e.getX(), e.getY());
                if (entity == null) {
                    return;
                }

                if (entity != null && (entity instanceof XYItemEntity)) {
                    XYPlot plot = getChart().getXYPlot();

                    XYItemEntity item = (XYItemEntity) entity;
                    int series = item.getSeriesIndex();
                    int itemId = item.getItem();

                    ((AntJFreeChart) getChart()).setCrosshairSeries(series);
                    ((AntJFreeChart) getChart()).setCrosshairItem(itemId);
                }
            }
        }
    }

    public void setChart(AntJFreeChart chart) {
        super.setChart(chart);

        if (!alreadyAddedListeners) {
            getCanvas().addAuxiliaryMouseHandler(new LineChartMouseListener());
            getCanvas().addMouseHandler(new PanHandlerFX("Pan Mouse Listener", false, true, false, false));
            alreadyAddedListeners = true;
        }
        if (chart.getCrosshairOverlay() != null) {
            getCanvas().addOverlay(chart.getCrosshairOverlay());
        }

    }
}