package at.jku.mevss.trace.gui.chart.jfreechart.base;

import at.jku.mevss.anttracks.util.ApplicationStatistics;
import at.jku.mevss.anttracks.util.Tuple;
import at.jku.mevss.trace.gui.chart.base.AntChartPane;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.model.ChartSelection;
import at.jku.mevss.trace.gui.utils.PlatformUtil;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StackedXYAreaRenderer;
import org.jfree.chart.renderer.xy.XYAreaRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.XYDataset;

import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class AntJFreeChartXYPlot extends XYPlot {
    public class AntShapeRendererForStackedCharts extends StackedXYAreaRenderer {
        private static final long serialVersionUID = -6167343503243232421L;
        private final Rectangle2D nullRectangle = new Rectangle2D.Float(0, 0, 0, 0);

        private final ChartSynchronizer synchronizer;
        private final AntJFreeChart chart;

        private static final float shapeWidth = 7.5f;
        private final Shape shape = new Ellipse2D.Float(-shapeWidth / 2, -shapeWidth / 2, shapeWidth, shapeWidth);

        private final HashMap<Integer, List<Integer>> showShapes = new HashMap<>();

        public AntShapeRendererForStackedCharts(AntJFreeChart chart, AntChartPane<AbstractIntervalXYDataset, ?> chartPane, ChartSynchronizer synchronizer) {
            super(XYAreaRenderer.SHAPES);

            this.chart = chart;
            this.synchronizer = synchronizer;

            setShapePaint(Color.BLACK);
            setShapeStroke(new BasicStroke(shapeWidth));

            for (int series = 0; series < 10; series++) {
                setSeriesShape(series, shape);
            }

            if (this.synchronizer != null) {
                synchronizer.addSelectionListener(new ChartSynchronizer.SelectionListener() {
                    @Override
                    public void inCompareModeChanged(ChartSynchronizer sender) {

                    }

                    @Override
                    public void selectionChanged(ChartSynchronizer sender) {
                        ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("JFreeChart: Sync Selection");
                        HashMap<Double, Tuple<Double, List<Tuple<Integer, Integer>>>> nearestPoints = new HashMap<>();

                        for (int series = 0; series < chart.getXYPlot().getDataset().getSeriesCount(); series++) {
                            for (int item = 0; item < chart.getXYPlot().getDataset().getItemCount(series); item++) {
                                for (ChartSelection cs : sender.getSelection().get()) {
                                    Tuple<Double, List<Tuple<Integer, Integer>>> currentBestSolution = nearestPoints.get(cs.x);
                                    if (currentBestSolution == null) {
                                        nearestPoints.put(cs.x, new Tuple<>(chart.getXYPlot().getDataset().getXValue(series, item), new ArrayList<>()));
                                        currentBestSolution = nearestPoints.get(cs.x);
                                    } else if (Math.abs(currentBestSolution.a - cs.x) > Math.abs(chart.getXYPlot().getDataset().getXValue(series, item) - cs.x)) {
                                        nearestPoints.put(cs.x, new Tuple<>(chart.getXYPlot().getDataset().getXValue(series, item), new ArrayList<>()));
                                        currentBestSolution = nearestPoints.get(cs.x);
                                    }

                                    if (Math.abs(currentBestSolution.a - cs.x) == Math.abs(chart.getXYPlot().getDataset().getXValue(series, item) - cs.x)) {
                                        nearestPoints.get(cs.x).b.add(new Tuple<>(series, item));
                                    }
                                }
                            }
                        }

                        showShapes.clear();
                        nearestPoints.values().stream().flatMap(x -> x.b.stream()).forEach(seriesItemTuple -> {
                            if (!showShapes.containsKey(seriesItemTuple.a)) {
                                showShapes.put(seriesItemTuple.a, new ArrayList<>());
                            }
                            showShapes.get(seriesItemTuple.a).add(seriesItemTuple.b);
                        });
                        m.end();
                    }
                });
            }
        }

        @Override
        public Shape getItemShape(int series, int item) {
            return ((showShapes.containsKey(series) && showShapes.get(series).contains(item))) ? shape : nullRectangle;
        }
    }

    public class AntShapeRendererForLineCharts extends XYLineAndShapeRenderer {
        private static final long serialVersionUID = -6167343503243232421L;
        private final Rectangle2D nullRectangle = new Rectangle2D.Float(0, 0, 0, 0);

        private final ChartSynchronizer synchronizer;
        private final AntJFreeChart chart;

        private static final float shapeWidth = 7.5f;
        private final Shape shape = new Ellipse2D.Float(-shapeWidth / 2, -shapeWidth / 2, shapeWidth, shapeWidth);

        private final HashMap<Integer, List<Integer>> showShapes = new HashMap<>();

        public AntShapeRendererForLineCharts(AntJFreeChart chart, AntChartPane<AbstractIntervalXYDataset, ?> chartPane, ChartSynchronizer synchronizer) {
            super(false, true);

            this.chart = chart;
            this.synchronizer = synchronizer;

            setDefaultStroke(new BasicStroke(shapeWidth));
            setDefaultOutlineStroke(new BasicStroke(shapeWidth));
            for (int series = 0; series < 10; series++) {
                setSeriesShape(series, shape);
                setSeriesPaint(series, Color.BLACK);
                setSeriesStroke(series, new BasicStroke(shapeWidth));
                setSeriesOutlineStroke(series, new BasicStroke(shapeWidth));
            }

            if (this.synchronizer != null) {
                synchronizer.addSelectionListener(new ChartSynchronizer.SelectionListener() {
                    @Override
                    public void inCompareModeChanged(ChartSynchronizer sender) {

                    }

                    @Override
                    public void selectionChanged(ChartSynchronizer sender) {
                        ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("JFreeChart: Sync Selection");
                        HashMap<Double, Tuple<Double, List<Tuple<Integer, Integer>>>> nearestPoints = new HashMap<>();

                        for (int series = 0; series < chart.getXYPlot().getDataset().getSeriesCount(); series++) {
                            for (int item = 0; item < chart.getXYPlot().getDataset().getItemCount(series); item++) {
                                for (ChartSelection cs : sender.getSelection().get()) {
                                    Tuple<Double, List<Tuple<Integer, Integer>>> currentBestSolution = nearestPoints.get(cs.x);
                                    if (currentBestSolution == null) {
                                        nearestPoints.put(cs.x, new Tuple<>(chart.getXYPlot().getDataset().getXValue(series, item), new ArrayList<>()));
                                        currentBestSolution = nearestPoints.get(cs.x);
                                    } else if (Math.abs(currentBestSolution.a - cs.x) > Math.abs(chart.getXYPlot().getDataset().getXValue(series, item) - cs.x)) {
                                        nearestPoints.put(cs.x, new Tuple<>(chart.getXYPlot().getDataset().getXValue(series, item), new ArrayList<>()));
                                        currentBestSolution = nearestPoints.get(cs.x);
                                    }

                                    if (Math.abs(currentBestSolution.a - cs.x) == Math.abs(chart.getXYPlot().getDataset().getXValue(series, item) - cs.x)) {
                                        nearestPoints.get(cs.x).b.add(new Tuple<>(series, item));
                                    }
                                }
                            }
                        }

                        showShapes.clear();
                        nearestPoints.values().stream().flatMap(x -> x.b.stream()).forEach(seriesItemTuple -> {
                            if (!showShapes.containsKey(seriesItemTuple.a)) {
                                showShapes.put(seriesItemTuple.a, new ArrayList<>());
                            }
                            showShapes.get(seriesItemTuple.a).add(seriesItemTuple.b);
                        });
                        m.end();
                    }
                });
            }
        }

        @Override
        public boolean getItemShapeVisible(int series, int item) {
            return ((showShapes.containsKey(series) && showShapes.get(series).contains(item)));
        }
    }

    private static final long serialVersionUID = -8124501729860976121L;
    private final ChartSynchronizer synchronizer;
    private final AntChartPane<AbstractIntervalXYDataset, ?> chartPane;

    public AntJFreeChartXYPlot(AntChartPane<AbstractIntervalXYDataset, ?> chartPane,
                               XYDataset dataset,
                               ValueAxis xAxis,
                               ValueAxis yAxis,
                               XYItemRenderer renderer,
                               ChartSynchronizer synchronizer) {
        super(dataset, xAxis, yAxis, renderer);
        this.chartPane = chartPane;
        this.synchronizer = synchronizer;
    }

    @Override
    public void setDataset(XYDataset dataset) {
        PlatformUtil.runAndWait(() -> {
            for (int i = 0; i < getDatasetCount(); i++) {
                setDataset(i, dataset);
            }
        });
    }

    @Override
    public void panDomainAxes(double percent, PlotRenderingInfo info, Point2D source) {
        super.panDomainAxes(percent, info, source);
        if (synchronizer != null && chartPane.isZoomSynchronized()) {
            synchronizer.setZoomXBounds(getChart().getXYPlot().getDomainAxis().getRange().getLowerBound(),
                                        getChart().getXYPlot().getDomainAxis().getRange().getUpperBound());
        }
    }

    @Override
    public void zoom(double percent) {
        super.zoom(percent);
        if (synchronizer != null && chartPane.isZoomSynchronized()) {
            synchronizer.setZoomXBounds(getChart().getXYPlot().getDomainAxis().getRange().getLowerBound(),
                                        getChart().getXYPlot().getDomainAxis().getRange().getUpperBound());
        }
    }

    @Override
    public void zoomDomainAxes(double factor, PlotRenderingInfo state, Point2D source) {
        super.zoomDomainAxes(factor, state, source);
        if (synchronizer != null && chartPane.isZoomSynchronized()) {
            synchronizer.setZoomXBounds(getChart().getXYPlot().getDomainAxis().getRange().getLowerBound(),
                                        getChart().getXYPlot().getDomainAxis().getRange().getUpperBound());
        }
    }

    @Override
    public void zoomDomainAxes(double factor, PlotRenderingInfo state, Point2D source, boolean useAnchor) {
        super.zoomDomainAxes(factor, state, source, useAnchor);
        if (synchronizer != null && chartPane.isZoomSynchronized()) {
            synchronizer.setZoomXBounds(getChart().getXYPlot().getDomainAxis().getRange().getLowerBound(),
                                        getChart().getXYPlot().getDomainAxis().getRange().getUpperBound());
        }
    }

    @Override
    public void zoomDomainAxes(double lowerPercent, double upperPercent, PlotRenderingInfo state, Point2D source) {
        super.zoomDomainAxes(lowerPercent, upperPercent, state, source);
        if (synchronizer != null && chartPane.isZoomSynchronized()) {
            synchronizer.setZoomXBounds(getChart().getXYPlot().getDomainAxis().getRange().getLowerBound(),
                                        getChart().getXYPlot().getDomainAxis().getRange().getUpperBound());
        }
    }
}