package at.jku.mevss.trace.gui.chart.jfreechart.xy.base;

import at.jku.mevss.trace.gui.chart.base.AntChartPane;
import at.jku.mevss.trace.gui.chart.jfreechart.base.AntChartViewer;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.PlatformUtil;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.DefaultTableXYDataset;

public abstract class XYBaseJFreeChartPane<DATA> extends AntChartPane<AbstractIntervalXYDataset, DATA> {
    /**
     * chartViewer represents the JavaFX container for a JFreeChart.
     * it contains the following parts:
     * chartViewer.getCanvas()                         ... painting canvas, used to add overlays
     * chartViewer.getChart()                          ... the JFreeChart
     * chartViewer.getChart().getXYPlot()              ... the chart's plot
     * chartViewer.getChart().getXYPlot().getDataset() ... the chart's dataset
     */
    @FXML
    protected AntChartViewer chartViewer;

    public XYBaseJFreeChartPane() {
        FXMLUtil.load(this, XYBaseJFreeChartPane.class);
    }

    @Override
    protected abstract AbstractIntervalXYDataset createDataSet(DATA appInfo);

    @Override
    protected abstract void initializeChart();

    @Override
    protected abstract EventHandler<ActionEvent> getOnConfigureAction();

    @Override
    protected void updateChartOnUIThread(AbstractIntervalXYDataset dataset) {
        chartViewer.getChart().getXYPlot().setDataset(dataset);
    }

    @Override
    protected boolean createsNewDatasetOnUpdate() {
        return true;
    }

    @Override
    protected void postDataSetUpdate() {

    }

    @Override
    public void zoomX(double lowerBound, double upperBound) {
        if (lowerBound == Double.MIN_VALUE || upperBound == Double.MAX_VALUE) {
            chartViewer.getChart().getXYPlot().getDomainAxis().setAutoRange(true);
        } else {
            chartViewer.getChart().getXYPlot().getDomainAxis().setRange(lowerBound, upperBound);
            // updateCrosshair();
            // updateDataLabelLayout();
        }
        chartViewer.getChart().getXYPlot().getRangeAxis().setAutoRange(true);
        chartViewer.getChart().getXYPlot().getRangeAxis().setUpperMargin(0.05);
    }

    // Not needed to implement
    @Override
    protected DefaultTableXYDataset updateDataSet(DATA appInfo) {
        return null;
    }

    @Override
    public void setXLabel(String label) {
        PlatformUtil.runAndWait(() -> {
            chartViewer.getChart().getXYPlot().getDomainAxis().setLabel(label);
        });
    }

    @Override
    public void setYLabel(String label) {
        PlatformUtil.runAndWait(() -> {
            chartViewer.getChart().getXYPlot().getRangeAxis().setLabel(label);
        });
    }

    @Override
    public void updateMarker() {
        chartViewer.getChart().fireChartChanged();
    }
}
