package at.jku.mevss.trace.gui.chart.jfreechart.xy.line.gc;

import at.jku.mevss.anttracks.heap.GarbageCollectionCause;
import at.jku.mevss.anttracks.heap.StatisticGCInfo;
import at.jku.mevss.trace.gui.chart.base.AxisScaler;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.fxchart.xy.linepane.GCTypesChartConfigurationDialog;
import at.jku.mevss.trace.gui.chart.fxchart.xy.linepane.ShowGCCause;
import at.jku.mevss.trace.gui.chart.jfreechart.base.JFreeChartFactory;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.line.base.LineJFreeChartPane;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.util.AddALotXYSeries;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.WindowUtil;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import org.jfree.chart.labels.XYItemLabelGenerator;
import org.jfree.data.xy.*;

import java.util.*;
import java.util.stream.Collectors;

public class GCJFreeChart extends LineJFreeChartPane<AppInfo> implements ShowGCCause {
    private static final class Info {
        public final GarbageCollectionCause cause;
        public final boolean failed;

        public Info(GarbageCollectionCause cause, boolean failed) {
            this.cause = cause;
            this.failed = failed;
        }
    }

    private static boolean useFirstDerivation = false;

    final Set<Integer> displayedCauses = new HashSet<>();
    private Map<Number, Info> infos;

    public GCJFreeChart() {
        FXMLUtil.load(this, GCJFreeChart.class);
    }

    public void init(ChartSynchronizer chartSynchronizer) {
        super.init(chartSynchronizer);
    }

    public AbstractIntervalXYDataset createDataSet(AppInfo appInfo) {
        AddALotXYSeries minorSeries = new AddALotXYSeries("Minor", true);
        List<XYDataItem> minorData = new ArrayList<>();
        AddALotXYSeries majorSeries = new AddALotXYSeries("Major", true);
        List<XYDataItem> majorData = new ArrayList<>();
        AddALotXYSeries majorConcurrentSeries = new AddALotXYSeries("Major (concurrent)", true);
        List<XYDataItem> majorConcurrentData = new ArrayList<>();
        AddALotXYSeries minorConcurrentSeries = new AddALotXYSeries("Minor (concurrent)", true);
        List<XYDataItem> minorConcurrentData = new ArrayList<>();

        infos = new HashMap<>();

        if (useFirstDerivation) {
            double minor = 0;
            double major = 0;

            for (int i = 0; i + 1 < appInfo.getStatistics().size(); i += 2) {
                StatisticGCInfo startInfo = appInfo.getStatistics().get(i).info;
                StatisticGCInfo endInfo = appInfo.getStatistics().get(i + 1).info;

                double time = endInfo.time;
                double start = startInfo.time;
                double end = endInfo.time;
                double duration = end - start;

                if (startInfo.type.isFull()) {
                    major += duration;
                } else {
                    minor += duration;
                }

                minorData.add(new XYDataItem(time, minor));
                majorData.add(new XYDataItem(time, major));
                infos.put(time, new Info(startInfo.cause, endInfo.failed));
            }
        } else {
            for (int i = 0; i + 1 < appInfo.getStatistics().size(); i += 2) {
                StatisticGCInfo startInfo = appInfo.getStatistics().get(i).info;
                StatisticGCInfo endInfo = appInfo.getStatistics().get(i + 1).info;
                long start = startInfo.time;
                long end = endInfo.time;
                long duration = end - start;

                List<XYDataItem> series = startInfo.type.isFull() ?
                                          (startInfo.concurrent ? majorConcurrentData : majorData) :
                                          (startInfo.concurrent ? minorConcurrentData : minorData);

                XYDataItem firstPoint = new XYDataItem(1.0 * start, 0);
                XYDataItem secondPoint = new XYDataItem(1.0 * start, duration);
                XYDataItem thirdPoint = new XYDataItem(1.0 * end, duration);
                XYDataItem fourthPoint = new XYDataItem(1.0 * end, 0);
                series.add(firstPoint);
                series.add(secondPoint);
                series.add(thirdPoint);
                series.add(fourthPoint);
                infos.put(1.0 * start, new Info(startInfo.cause, endInfo.failed));
            }
        }

        minorSeries.setData(minorData, false);
        majorSeries.setData(majorData, false);
        majorConcurrentSeries.setData(majorConcurrentData, false);
        minorConcurrentSeries.setData(minorConcurrentData, false);

        XYSeriesCollection dataset = new XYSeriesCollection();
        dataset.addSeries(majorConcurrentSeries);
        dataset.addSeries(minorConcurrentSeries);
        dataset.addSeries(majorSeries);
        dataset.addSeries(minorSeries);

        displayedCauses.addAll(Arrays.stream(appInfo.getSymbols().causes.getAll()).filter(c -> !c.common).map(c -> c.id).collect(Collectors.toList()));

        return dataset;
    }

    @Override
    protected void initializeChart() {
        chartViewer.setChart(JFreeChartFactory.createLineXYChart(this,
                                                                 "Garbage Collection Pauses",
                                                                 null,
                                                                 "Time [ms]",
                                                                 "Duration [ms]",
                                                                 new DefaultTableXYDataset(),
                                                                 new XYItemLabelGenerator() {
                                                                     @Override
                                                                     public String generateLabel(XYDataset dataset, int series, int item) {
                                                                         Info info = infos.get(dataset.getX(series, item));
                                                                         // info may be null for GC end
                                                                         if (info != null) {
                                                                             return displayedCauses.contains(info.cause.id) ? info.cause.name : "";
                                                                         }
                                                                         return "";
                                                                     }
                                                                 },
                                                                 chartSynchronizer));
    }

    @Override
    protected EventHandler<ActionEvent> getOnConfigureAction() {
        return ae -> {
            GCTypesChartConfigurationDialog dialog = new GCTypesChartConfigurationDialog();
            dialog.init(this);
            WindowUtil.centerInMainFrame(dialog);
            dialog.showAndWait();
        };
    }

    @Override
    public AxisScaler<AbstractIntervalXYDataset> getScaler() {
        return new AxisScaler.JFreeChartAxisScaler.GCScaler(this);
    }

    @Override
    public Set<Integer> getDisplayedCauses() {
        return displayedCauses;
    }

    @Override
    public void updateDisplayCause(boolean show, GarbageCollectionCause... causes) {
        if (show) {
            displayedCauses.addAll(Arrays.stream(causes).map(c -> c.id).collect(Collectors.toList()));
        } else {
            displayedCauses.removeAll(Arrays.stream(causes).map(c -> c.id).collect(Collectors.toList()));
        }
        chartViewer.getChart().fireChartChanged();
        //updateDataLabelContent();
    }
}
