package at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.alivedead;

import at.jku.mevss.anttracks.heap.statistics.Statistics;
import at.jku.mevss.trace.gui.chart.base.ApplicationChartFactory;
import at.jku.mevss.trace.gui.chart.base.AxisScaler;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.jfreechart.base.JFreeChartFactory;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.base.StackedAreaJFreeChartPane;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.util.AddALotXYSeries;
import at.jku.mevss.trace.gui.model.AppInfo;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.XYDataItem;

import java.util.ArrayList;
import java.util.List;

public class AliveDeadStackedAreaJFreeChart extends StackedAreaJFreeChartPane<AppInfo> {

    ApplicationChartFactory.AliveDeadMemoryConsumptionUnit aliveDeadMemoryConsumptionUnit;

    public AliveDeadStackedAreaJFreeChart() {

    }

    public void init(ChartSynchronizer chartSynchronizer, ApplicationChartFactory.AliveDeadMemoryConsumptionUnit aliveDeadMemoryConsumptionUnit) {
        this.aliveDeadMemoryConsumptionUnit = aliveDeadMemoryConsumptionUnit;
        super.init(chartSynchronizer);
    }

    @Override
    protected AbstractIntervalXYDataset createDataSet(AppInfo appInfo) {
        AddALotXYSeries survivedSeries = new AddALotXYSeries("Survived", false);
        List<XYDataItem> survivedData = new ArrayList<>();
        AddALotXYSeries diedSeries = new AddALotXYSeries("Died", false);
        List<XYDataItem> diedData = new ArrayList<>();

        switch (aliveDeadMemoryConsumptionUnit) {
            case BYTES:
                for (int i = 0; i + 1 < appInfo.getStatistics().size(); i += 2) {
                    Statistics start = appInfo.getStatistics().get(i);
                    Statistics end = appInfo.getStatistics().get(i + 1);

                    double time = start.info.time;

                    long before = start.eden.memoryConsumption.getBytes() + start.survivor.memoryConsumption.getBytes() + start.old.memoryConsumption.getBytes();
                    long after = end.eden.memoryConsumption.getBytes() + end.survivor.memoryConsumption.getBytes() + end.old.memoryConsumption.getBytes();

                    survivedData.add(new XYDataItem(time, after));
                    diedData.add(new XYDataItem(time, before - after));
                }
                break;
            case OBJECTS:
                for (int i = 0; i + 1 < appInfo.getStatistics().size(); i += 2) {
                    Statistics start = appInfo.getStatistics().get(i);
                    Statistics end = appInfo.getStatistics().get(i + 1);

                    double time = start.info.time;

                    long before = start.eden.memoryConsumption.getObjects() + start.survivor.memoryConsumption.getObjects() + start.old.memoryConsumption.getObjects();
                    long after = end.eden.memoryConsumption.getObjects() + end.survivor.memoryConsumption.getObjects() + end.old.memoryConsumption.getObjects();

                    survivedData.add(new XYDataItem(time, after));
                    diedData.add(new XYDataItem(time, before - after));
                }
                break;
            case RELATIVE:
                boolean consecutiveGC = false;
                Statistics start = null;
                Statistics end = null;

                for (int i = 0; i + 1 < appInfo.getStatistics().size(); i += 2) {
                    if (!consecutiveGC) {
                        start = appInfo.getStatistics().get(i);
                    }
                    end = appInfo.getStatistics().get(i + 1);
                    if (i + 2 < appInfo.getStatistics().size()) {
                        long timeDiff = appInfo.getStatistics().get(i + 2).info.time - end.info.time;
                        consecutiveGC = timeDiff == 0;
                        if (consecutiveGC) {
                            LOGGER.finest("Consecutive GCs found at " + end.info.time);
                            continue;
                        }
                    }

                    double time = start.info.time;
                    double endTime = end.info.time;

                    long before = start.eden.memoryConsumption.getObjects() + start.survivor.memoryConsumption.getObjects() + start.old.memoryConsumption.getObjects();
                    long after = end.eden.memoryConsumption.getObjects() + end.survivor.memoryConsumption.getObjects() + end.old.memoryConsumption.getObjects();
                    double survivedRatio = 1.0 * after / before > 1 ? 1 : 1.0 * after / before; // survived ratio > 1 may happen if during a (concurrent) GC more
                    // objects are allocated than collected
                    double diedRatio = 1.0 - survivedRatio;

                    XYDataItem survivedDataItem = new XYDataItem(time, survivedRatio * 100.0);
                    XYDataItem diedDataItem = new XYDataItem(time, diedRatio * 100.0);

                    survivedData.add(survivedDataItem);
                    diedData.add(diedDataItem);
                }
                break;
            default:
                throw new IllegalStateException();
        }

        survivedSeries.setData(survivedData, false);
        diedSeries.setData(diedData, false);

        DefaultTableXYDataset dataset = new DefaultTableXYDataset();
        dataset.addSeries(diedSeries);
        dataset.addSeries(survivedSeries);

        return dataset;
    }

    @Override
    protected void initializeChart() {
        chartViewer.setChart(JFreeChartFactory.createStackedXYAreaChart(this,
                                                                        "Survived / Died: " + aliveDeadMemoryConsumptionUnit.getUnit(),
                                                                        null,
                                                                        "set by scaler",
                                                                        "set by scaler",
                                                                        new DefaultTableXYDataset(),
                                                                        null,
                                                                        new StandardXYToolTipGenerator(),
                                                                        null,
                                                                        chartSynchronizer));
    }

    @Override
    protected EventHandler<ActionEvent> getOnConfigureAction() {
        return null;
    }

    @Override
    public AxisScaler<AbstractIntervalXYDataset> getScaler() {
        return new AxisScaler.JFreeChartAxisScaler.AliveDeadScaler(this, aliveDeadMemoryConsumptionUnit);
    }

}
