package at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.allocatingsubsystem;

import at.jku.mevss.anttracks.heap.statistics.Statistics;
import at.jku.mevss.trace.gui.chart.base.AxisScaler;
import at.jku.mevss.trace.gui.chart.jfreechart.base.JFreeChartFactory;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.base.StackedAreaJFreeChartPane;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.util.AddALotXYSeries;
import at.jku.mevss.trace.gui.classification.classifier.AllocatingSubsystemClassifier;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.XYDataItem;

import java.util.ArrayList;
import java.util.List;

public class AllocatingSubsystemStackedAreaJFreeChart extends StackedAreaJFreeChartPane<AppInfo> {

    public AllocatingSubsystemStackedAreaJFreeChart() {
        FXMLUtil.load(this, AllocatingSubsystemStackedAreaJFreeChart.class);
    }

    public DefaultTableXYDataset createDataSet(AppInfo appInfo) {
        AddALotXYSeries vmSeries = new AddALotXYSeries(AllocatingSubsystemClassifier.VM, false);
        List<XYDataItem> vmData = new ArrayList<>();
        AddALotXYSeries irSeries = new AddALotXYSeries(AllocatingSubsystemClassifier.IR, false);
        List<XYDataItem> irData = new ArrayList<>();
        AddALotXYSeries c1Series = new AddALotXYSeries(AllocatingSubsystemClassifier.C1, false);
        List<XYDataItem> c1Data = new ArrayList<>();
        AddALotXYSeries c2Series = new AddALotXYSeries(AllocatingSubsystemClassifier.C1, false);
        List<XYDataItem> c2Data = new ArrayList<>();

        for (int i = 0; i < appInfo.getStatistics().size(); i++) {
            Statistics stat = appInfo.getStatistics().get(i);
            double time = stat.info.time;

            XYDataItem vmDataItem = new XYDataItem(time, stat.eden.allocators.getVm() + stat.survivor.allocators.getVm() + stat.old.allocators.getVm());
            XYDataItem irDataItem = new XYDataItem(time, stat.eden.allocators.getIr() + stat.survivor.allocators.getIr() + stat.old.allocators.getIr());
            XYDataItem c1DataItem = new XYDataItem(time, stat.eden.allocators.getC1() + stat.survivor.allocators.getC1() + stat.old.allocators.getC1());
            XYDataItem c2DataItem = new XYDataItem(time, stat.eden.allocators.getC2() + stat.survivor.allocators.getC2() + stat.old.allocators.getC2());
            vmData.add(vmDataItem);
            irData.add(irDataItem);
            c1Data.add(c1DataItem);
            c2Data.add(c2DataItem);
        }

        vmSeries.setData(vmData, false);
        irSeries.setData(irData, false);
        c1Series.setData(c1Data, false);
        c2Series.setData(c2Data, false);

        DefaultTableXYDataset dataset = new DefaultTableXYDataset();
        dataset.addSeries(vmSeries);
        dataset.addSeries(irSeries);
        dataset.addSeries(c1Series);
        dataset.addSeries(c2Series);

        return dataset;
    }

    @Override
    protected void initializeChart() {
        chartViewer.setChart(JFreeChartFactory.createStackedXYAreaChart(this,
                                                                        "Allocating Subsystem",
                                                                        null,
                                                                        "set by scaler",
                                                                        "set by scaler",
                                                                        new DefaultTableXYDataset(),
                                                                        null,
                                                                        new StandardXYToolTipGenerator(),
                                                                        null,
                                                                        chartSynchronizer));
    }

    @Override
    protected EventHandler<ActionEvent> getOnConfigureAction() {
        return null;
    }

    @Override
    public AxisScaler<AbstractIntervalXYDataset> getScaler() {
        return new AxisScaler.JFreeChartAxisScaler.ObjectsScaler(this);
    }
}
