package at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.diffing;

import at.jku.mevss.trace.gui.chart.base.ApplicationChartFactory;
import at.jku.mevss.trace.gui.chart.base.AxisScaler;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.jfreechart.base.JFreeChartFactory;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.base.StackedAreaJFreeChartPane;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.util.AddALotXYSeries;
import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.model.DiffingData;
import at.jku.mevss.trace.gui.model.Size;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.XYDataItem;

import java.awt.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.function.Function;

public class DiffingStackedAreaJFreeChart extends StackedAreaJFreeChartPane<List<DiffingData>> {
    public static final Color[] DIFFING_COLORS = new Color[]{new Color(0x66, 0xb2, 0xff),
                                                             new Color(0xff, 0x66, 0x66),
                                                             new Color(0xb2, 0xff, 0x66),
                                                             new Color(0x6a, 0x99, 0x3d),
                                                             new Color(0xc0, 0xc0, 0xc0)};

    private ApplicationChartFactory.MemoryConsumptionUnit unit;
    private Function<Size, Long> accessor;
    private List<DiffingData> data;

    public DiffingStackedAreaJFreeChart() {
        FXMLUtil.load(this, DiffingStackedAreaJFreeChart.class);
    }

    public void init(ChartSynchronizer synchronizer, ApplicationChartFactory.MemoryConsumptionUnit unit) {
        this.unit = unit;
        switch (unit) {
            case OBJECTS:
                accessor = size -> (long) size.objects;
                break;
            case BYTES:
                accessor = size -> (long) size.bytes;
                break;
        }
        super.init(synchronizer);
    }

    public DefaultTableXYDataset createDataSet(List<DiffingData> dataList) {
        AddALotXYSeries permSeries = new AddALotXYSeries("Perm", false);
        List<XYDataItem> permData = new ArrayList<>();
        AddALotXYSeries diedSeries = new AddALotXYSeries("Died", false);
        List<XYDataItem> diedData = new ArrayList<>();
        AddALotXYSeries bornMinSeries = new AddALotXYSeries("Born (min)", false);
        List<XYDataItem> bornMinData = new ArrayList<>();
        AddALotXYSeries bornMaxSeries = new AddALotXYSeries("Born (max)", false);
        List<XYDataItem> bornMaxData = new ArrayList<>();
        AddALotXYSeries tempSeries = new AddALotXYSeries("Temp", false);
        List<XYDataItem> tempData = new ArrayList<>();

        boolean ea = false;
        assert ea = true;

        DiffingData[] data = dataList.stream().toArray(DiffingData[]::new);
        // if (data.length >= 2 && accessor.apply(data[0].perm) == 0) {
        // data[0] = new DiffingData(data[0].id, data[0].time);
        // data[0].add(DiffingDataInfo.PermObjects, data[1].perm.objects + data[1].died.objects);
        // data[0].add(DiffingDataInfo.PermBytes, data[1].perm.bytes + data[1].died.bytes);
        // }
        DiffingData last = data[data.length - 1];

        // perm is always falling (going to died)
        for (int i = 1; i < data.length; i++) {
            assert accessor.apply(data[i - 1].perm) >= accessor.apply(data[i].perm) : "Perm must not grow";
        }
        // died is always rising (coming from perm)
        for (int i = 1; i < data.length; i++) {
            assert accessor.apply(data[i - 1].died) <= accessor.apply(data[i].died) : "Died must not shrink";
        }
        // difference between perm and died must match
        for (int i = 1; i < data.length; i++) {
            long deltaDied = accessor.apply(data[i].died) - accessor.apply(data[i - 1].died);
            long deltaPerm = accessor.apply(data[i - 1].perm) - accessor.apply(data[i].perm);

            assert deltaDied == deltaPerm : String.format(
                    "Changes in perm and died must match. Perm Before: %d; Died Before: %d; Perm After: %d; Died After: %d; DeltaPerm: " +
                            "%d, DeltaDied: %d;",
                    accessor.apply(data[i - 1].perm),
                    accessor.apply(data[i - 1].died),
                    accessor.apply(data[i].perm),
                    accessor.apply(data[i].died),
                    deltaPerm,
                    deltaDied);
        }

        // born is rising or falling (coming from <allocations>, going to temp)
        // temp is always rising (coming from born)
        for (int i = 1; i < data.length; i++) {
            assert accessor.apply(data[i - 1].temp) <= accessor.apply(data[i].temp) : "Temp must not shrink";
        }

        long[] allocations = new long[data.length];
        long[] deallocations = new long[data.length];
        {
            long last_born = 0, last_temp = 0;
            for (int i = 0; i < allocations.length; i++) {
                DiffingData current = data[i];
                deallocations[i] = accessor.apply(current.temp) - last_temp;
                allocations[i] = (accessor.apply(current.born) + deallocations[i]) - last_born;
                last_born = accessor.apply(current.born);
                last_temp = accessor.apply(current.temp);
            }
        }

        long[] perms = new long[data.length];
        long[] dieds = new long[data.length];
        long[] borns = new long[data.length];
        long[] temps = new long[data.length];
        for (int i = 0; i < data.length; i++) {
            DiffingData d = data[i];
            dieds[i] = accessor.apply(last.died) - accessor.apply(d.died);
            perms[i] = accessor.apply(d.perm) - dieds[i];
            temps[i] = (i == 0) ? 0 : (temps[i - 1] + allocations[i] /* not all are really temp, some are born */ - deallocations[i]);
            borns[i] = accessor.apply(d.perm) + accessor.apply(d.born) - perms[i] - dieds[i] - temps[i]; /* borns will always be zero here */
        }

        /* fix current issure that we do not know when temp objects become born */
        {
            long born_leftovers = accessor.apply(last.born) - borns[borns.length - 1];
            long max = born_leftovers;
            for (int i = data.length - 1; i >= 0 && born_leftovers > 0; i--) {
                max = Math.min(max, temps[i]);
                long correction = Math.min(max, (i + 1 < data.length) ? Math.min(allocations[i + 1], born_leftovers) : 0);
                borns[i] = Math.min(max, (i + 1 < data.length) ? borns[i + 1] - correction : born_leftovers);
                temps[i] -= borns[i];
                born_leftovers -= (i + 1 < data.length ? (borns[i + 1] - borns[i]) : 0);
            }
        }

        if (ea) {
            assert dieds[dieds.length - 1] == 0;
            assert borns[0] == 0;
            assert temps[0] == 0 && temps[temps.length - 1] == 0;
            for (int i = 0; i < data.length; i++) {
                if (i > 0) {
                    assert perms[i - 1] == perms[i] : String.format("perm inconsistent: %d -> %d", perms[i - 1], perms[i]);
                    assert dieds[i - 1] >= dieds[i] : String.format("died inconsistent: %d -> %d", dieds[i - 1], dieds[i]);
                    assert borns[i - 1] <= borns[i] : String.format("born inconsistent: %d -> %d", borns[i - 1], borns[i]);
                    // dunno anything about temps
                }
                assert perms[i] >= 0 && dieds[i] >= 0 && borns[i] >= 0 && temps[i] >= 0 : String.format("negative values: %d, %d, %d, %d",
                                                                                                        perms[i],
                                                                                                        dieds[i],
                                                                                                        borns[i],
                                                                                                        temps[i]);
                assert accessor.apply(data[i].perm) + accessor.apply(data[i].born) == perms[i] + dieds[i] + borns[i] + temps[i] : String.format(
                        "live objects do not add up: %d + %d != %d + %d + %d + %d",
                        accessor.apply(data[i].perm),
                        accessor.apply(data[i].born),
                        perms[i],
                        dieds[i],
                        borns[i],
                        temps[i]);
            }
        }

        long[] borns_max = new long[data.length];
        {
            long max = accessor.apply(last.born);
            for (int i = data.length - 1; i >= 0; i--) {
                max = Math.min(max, temps[i] + borns[i]);
                borns_max[i] = max;
                assert borns_max[i] >= borns[i] : String.format("born max inconsistent: %d, %d, %d, %d", borns[i], borns_max[i], temps[i], max);
            }
            for (int i = 0; i < data.length; i++) {
                borns_max[i] -= borns[i];
                temps[i] = temps[i] - borns_max[i];
            }
        }

        if (ea) {
            assert dieds[dieds.length - 1] == 0;
            assert borns[0] == 0;
            assert borns_max[0] == 0;
            assert temps[0] == 0 && temps[temps.length - 1] == 0;
            for (int i = 0; i < data.length; i++) {
                if (i > 0) {
                    assert perms[i - 1] == perms[i] : String.format("perm inconsistent: %d -> %d", perms[i - 1], perms[i]);
                    assert dieds[i - 1] >= dieds[i] : String.format("died inconsistent: %d -> %d", dieds[i - 1], dieds[i]);
                    assert borns[i - 1] <= borns[i] : String.format("born inconsistent: %d -> %d", borns[i - 1], borns[i]);
                    assert borns[i - 1] + borns_max[i - 1] <= borns[i] + borns_max[i] : String.format("born (max) inconsistent: %d + %d -> %d + %d",
                                                                                                      borns[i - 1],
                                                                                                      borns_max[i - 1],
                                                                                                      borns[i],
                                                                                                      borns_max[i]);
                    // dunno anything about temps
                }
                assert perms[i] >= 0 && dieds[i] >= 0 && borns[i] >= 0 && borns_max[i] >= 0 && temps[i] >= 0;
                assert accessor.apply(data[i].perm) + accessor.apply(data[i].born) == perms[i] + dieds[i] + borns[i] + borns_max[i] + temps[i] : String.format(
                        "%d + %d != %d + %d + %d + %d + %d",
                        accessor.apply(data[i].perm),
                        accessor.apply(data[i].born),
                        perms[i],
                        dieds[i],
                        borns[i],
                        borns_max[i],
                        temps[i]);
            }
        }

        double[] times = Arrays.stream(data).mapToDouble(d -> (double) d.time).toArray();
        for (int i = 1; i < times.length; i++) {
            if (times[i - 1] == times[i]) {
                int start = i - 1;
                int end = i + 1;
                while (end < times.length && times[start] == times[end]) {
                    end++;
                }
                double step = 1.0 / (end - start);
                for (int j = 0; j < end - start; j++) {
                    times[start + j] += j * step;
                }
            } else {
                assert times[i - 1] < times[i];
            }
        }

        for (int i = 0; i < data.length; i++) {
            permData.add(new XYDataItem(times[i], perms[i]));
            diedData.add(new XYDataItem(times[i], dieds[i]));
            bornMinData.add(new XYDataItem(times[i], borns[i]));
            bornMaxData.add(new XYDataItem(times[i], borns_max[i]));
            tempData.add(new XYDataItem(times[i], temps[i]));
        }

        permSeries.setData(permData, false);
        diedSeries.setData(diedData, false);
        bornMinSeries.setData(bornMinData, false);
        bornMaxSeries.setData(bornMaxData, false);
        tempSeries.setData(tempData, false);

        DefaultTableXYDataset dataset = new DefaultTableXYDataset();
        dataset.addSeries(permSeries);
        dataset.addSeries(diedSeries);
        dataset.addSeries(bornMinSeries);
        dataset.addSeries(bornMaxSeries);
        dataset.addSeries(tempSeries);

        return dataset;
    }

    @Override
    public Color[] getSeriesColors() {
        return DIFFING_COLORS;
    }

    @Override
    protected void initializeChart() {
        chartViewer.setChart(JFreeChartFactory.createStackedXYAreaChart(this,
                                                                        unit.getLabel(),
                                                                        null,
                                                                        "x",
                                                                        "y",
                                                                        new DefaultTableXYDataset(),
                                                                        null,
                                                                        new StandardXYToolTipGenerator(),
                                                                        null,
                                                                        chartSynchronizer));
    }

    @Override
    protected EventHandler<ActionEvent> getOnConfigureAction() {
        return null;
    }

    @Override
    public AxisScaler<AbstractIntervalXYDataset> getScaler() {
        switch (unit) {
            case OBJECTS:
                return new AxisScaler.JFreeChartAxisScaler.ObjectsScaler(this);
            case BYTES:
                return new AxisScaler.JFreeChartAxisScaler.BytesScaler(this);
            default:
                LOGGER.warning("No scaler defined for " + unit);
                return new AxisScaler.JFreeChartAxisScaler.NoAxisScaler(this);
        }
    }
}
