/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.lifetime;

import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.trace.gui.chart.base.AxisScaler;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.jfreechart.base.AntJFreeChart;
import at.jku.mevss.trace.gui.chart.jfreechart.base.JFreeChartFactory;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.base.StackedAreaJFreeChartPane;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.util.AddALotXYSeries;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.LifetimeVisualizationInfo;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.LifetimeVisualizationTab;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.table.GrowthTable;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.table.GrowthTableRow;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.visualization.LifetimeVisualizationDiagramInfo;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.ChartSelection;
import at.jku.mevss.trace.gui.model.SelectedClassifierInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.event.ActionEvent;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.scene.input.MouseButton;
import org.jfree.chart.ChartRenderingInfo;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.entity.ChartEntity;
import org.jfree.chart.entity.XYItemEntity;
import org.jfree.chart.fx.interaction.ChartMouseEventFX;
import org.jfree.chart.fx.interaction.ChartMouseListenerFX;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.XYDataItem;
import org.jfree.data.xy.XYDataset;

import java.awt.geom.Rectangle2D;
import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.atomic.AtomicReference;
import java.util.logging.Level;
import java.util.stream.Collectors;

public class LifetimeStackedAreaJFreeChart extends StackedAreaJFreeChartPane<AppInfo> {

    private Map<Long, GroupingNode> groupings;
    private LifetimeVisualizationDiagramInfo diagramInfo;
    private ArrayList<String> selectedSeriesKeys;
    private static String OTHER = "Other";
    private LifetimeVisualizationTab tab;
    private GrowthTable table = new GrowthTable();
    private AntJFreeChart stackedChart;
    private AntJFreeChart lineChart;

    public LifetimeStackedAreaJFreeChart() {
        FXMLUtil.load(this, LifetimeStackedAreaJFreeChart.class);
    }

    public void init(LifetimeVisualizationTab tab, LifetimeVisualizationDiagramInfo diagramInfo, ChartSynchronizer synchronizer, Map<Long, GroupingNode> groupings) {
        super.init(synchronizer);
        this.tab = tab;
        this.diagramInfo = diagramInfo;
        this.groupings = groupings;
        this.chartViewer.setOnScroll(Event::consume);
        table.init();
        setCorrectChart();
        chartSynchronizer.addSelectionListener(new ChartSynchronizer.SelectionListener() {
            @Override
            public void inCompareModeChanged(ChartSynchronizer sender) {

            }

            @Override
            public void selectionChanged(ChartSynchronizer sender) {
                tab.setExamineButtonsDisabled(chartSynchronizer.getSelection().size() != 1);
            }
        });
        this.chartViewer.addChartMouseListener(new ChartMouseListenerFX() {
            @Override
            public void chartMouseClicked(ChartMouseEventFX event) {
                if (event.getTrigger().getButton() == MouseButton.MIDDLE) {
                    Object selectedKey = getSelectedKeyAndVerifyPrerequisites(event);
                    if (selectedKey != null) {
                        openChartInNewTab(selectedKey);
                    }
                } else if (event.getTrigger().getButton() == MouseButton.PRIMARY) {
                    Object selectedKey = getSelectedKeyAndVerifyPrerequisites(event);
                    if (selectedKey != null) {
                        openSubChart(selectedKey);
                    } else if (!(event.getEntity() instanceof XYItemEntity)) {
                        //calculate the clicked value
                        double x = event.getTrigger().getX();
                        XYPlot plot = event.getChart().getXYPlot();
                        final ValueAxis domainAxis = plot.getDomainAxis();
                        ChartRenderingInfo renderingInfo = chartViewer.getCanvas().getRenderingInfo();
                        Rectangle2D dataArea = renderingInfo.getPlotInfo().getDataArea();
                        final double chartX = domainAxis.java2DToValue(x, dataArea, plot.getDomainAxisEdge());

                        //gather all possible values
                        //TODO: should we just use groupings.keyset?
                        XYDataset dataset = event.getChart().getXYPlot().getDataset();
                        int itemCount = dataset.getItemCount(0);
                        if (itemCount == 0) { return; }
                        double bestDiff = Double.MAX_VALUE;
                        double bestXVal = -1;
                        int itemNum = -1;
                        for (int i = 0; i < itemCount; i++) {
                            double curXVal = dataset.getXValue(0, i);
                            double curDiff = Math.abs(chartX - curXVal);
                            if (curDiff < bestDiff) {
                                bestDiff = curDiff;
                                bestXVal = curXVal;
                                itemNum = i;
                            }
                        }

                        double yValue = dataset.getYValue(0, itemNum);

                        chartSynchronizer.sync(new ChartSelection(0, itemNum, bestXVal, yValue, ((AntJFreeChart) event.getChart()).getChartID()));
                    }
                }
            }

            //returns null if no series was clicked or there are no sublevels anymore
            private Object getSelectedKeyAndVerifyPrerequisites(ChartMouseEventFX event) {
                ChartEntity entity = event.getEntity();
                if (entity instanceof XYItemEntity) {
                    XYItemEntity xyEntity = (XYItemEntity) event.getEntity();
                    DefaultTableXYDataset dataset = (DefaultTableXYDataset) xyEntity.getDataset();
                    Object selectedKeyString = dataset.getSeries(xyEntity.getSeriesIndex()).getKey();
                    if (selectedKeyString.equals(OTHER)) {
                        LOGGER.log(Level.INFO, "Cannot open diagram when clicking other");
                        return null;
                    }
                    Object selectedKey = selectedSeriesKeys.stream().filter(x -> x.equals(selectedKeyString)).findFirst().get();
                    //verify that there is still another sub level

                    if (!hasSubLevel()) {
                        LOGGER.log(Level.WARNING, "There is no sublevel anymore!");
                        return null;
                    }
                    return selectedKey;
                }
                return null;
            }

            @Override
            public void chartMouseMoved(ChartMouseEventFX event) {

            }
        });
    }

    public boolean hasSubLevel() {
        return diagramInfo.getSelectedKeys().size() < diagramInfo.getClassifierChain().length() - 1;
    }

    public void openChartInNewTab(Object selectedKey) {
        LifetimeVisualizationTab newTab = new LifetimeVisualizationTab();
        LifetimeVisualizationInfo oldLifetimeVisualizationInfo = LifetimeStackedAreaJFreeChart.this.tab.getLifetimeVisualizationInfo();
        LifetimeVisualizationInfo newLifetimeVisualizationInfo = new LifetimeVisualizationInfo(oldLifetimeVisualizationInfo.getAppInfo(),
                                                                                               oldLifetimeVisualizationInfo.getStartTime(),
                                                                                               oldLifetimeVisualizationInfo.getEndTime(),
                                                                                               new SelectedClassifierInfo(oldLifetimeVisualizationInfo
                                                                                                                                  .getSelectedClassifierInfo()
                                                                                                                                  .getSelectedClassifiers(),
                                                                                                                          oldLifetimeVisualizationInfo
                                                                                                                                  .getSelectedClassifierInfo()
                                                                                                                                  .getSelectedConditionalClassifier(),
                                                                                                                          oldLifetimeVisualizationInfo
                                                                                                                                  .getSelectedClassifierInfo()
                                                                                                                                  .getSelectedFilters()),
                                                                                               oldLifetimeVisualizationInfo.getAvailableClassifier(),
                                                                                               oldLifetimeVisualizationInfo.getAvailableFilter());
        newTab.init(newLifetimeVisualizationInfo, LifetimeStackedAreaJFreeChart.this.tab);
        newTab.closeConfigurationSelection();
        newTab.showScrollPane();

        LifetimeStackedAreaJFreeChart newChart = createChartForKey(selectedKey, new ChartSynchronizer(), newTab);
        if (newChart != null) {
            tab.openNewTabAndShowCharts(LifetimeStackedAreaJFreeChart.this, newChart, newTab);
        }
    }

    public void openSubChart(Object selectedKey) {
        LifetimeStackedAreaJFreeChart newChart = createChartForKey(selectedKey, chartSynchronizer, tab);
        if (newChart != null) {
            tab.removeAllSubDiagramsOf(LifetimeStackedAreaJFreeChart.this);
            tab.addDiagramToUi(newChart);
        }
    }

    private LifetimeStackedAreaJFreeChart createChartForKey(Object key, ChartSynchronizer chartSynchronizer, LifetimeVisualizationTab tab) {
        List<String> newSelectedKeys = new ArrayList<>(diagramInfo.getSelectedKeys());
        newSelectedKeys.add(key.toString());
        if (tab.isDiagramAlreadyOpen(newSelectedKeys)) {
            LOGGER.log(Level.INFO, "This diagram is already open!");
            return null;
        }

        LifetimeVisualizationDiagramInfo newDiagramInfo = new LifetimeVisualizationDiagramInfo(diagramInfo.getLifetimeVisualizationInfo(),
                                                                                               diagramInfo.isUseLineChart(),
                                                                                               diagramInfo.getSeriesCount(),
                                                                                               diagramInfo.getSeriesSort(),
                                                                                               diagramInfo.getDiagramMetric(),
                                                                                               newSelectedKeys,
                                                                                               diagramInfo.getClassifierChain(),
                                                                                               diagramInfo.getFilters(),
                                                                                               true);
        LifetimeStackedAreaJFreeChart newChart = new LifetimeStackedAreaJFreeChart();
        newChart.init(tab, newDiagramInfo, chartSynchronizer, groupings);
        newChart.update(newDiagramInfo.getLifetimeVisualizationInfo().getAppInfo());
        return newChart;
    }

    private long getAmountOfCurUnit(GroupingNode gn) {
        switch (diagramInfo.getDiagramMetric()) {
            case OBJECTS:
                return gn.getObjectCount();
            case BYTES:
                return gn.getByteCount();
            case RETAINED_SIZE:
                return gn.retainedSizeProperty().longValue();
            case TRANSITIVE_CLOSURE_SIZE:
                return gn.transitiveClosureSizeProperty().longValue();
            default:
                return 0;
        }
    }

    // Call with level = 0 from outside, and key = [] for root node
    private GroupingNode getNodeWithKey(GroupingNode root, int level, String[] key) {
        if (root == null) {
            return null;
        }
        if (level == key.length) {
            return root;
        } else {
            return getNodeWithKey(root.getChildBasedOnStringKey(key[level]), level + 1, key);
        }
    }

    private ArrayList<String> getSortedKeysBySortedTimes(List<Long> sortedTimes) {
        ArrayList<String> selectedSeriesKeys = new ArrayList<>();

        for (Long curTime : sortedTimes) {
            GroupingNode curRoot = groupings.get(curTime);
            GroupingNode targetNode = getNodeWithKey(curRoot, 0, diagramInfo.getSelectedKeysAsArray());
            if (targetNode == null) {
                continue;
            }
            List<GroupingNode> targetChildren = targetNode.getChildren();
            List<String> curKeys = targetChildren.stream()
                                                 .filter(x -> !selectedSeriesKeys.contains(x.getKey().toString()))
                                                 .sorted(Comparator.comparing(this::getAmountOfCurUnit)
                                                                   .reversed())
                                                 .limit(diagramInfo.getSeriesCount() - selectedSeriesKeys.size())
                                                 .map(x -> x.getKey().toString())
                                                 .collect(Collectors.toList());

            selectedSeriesKeys.addAll(curKeys);
            if (selectedSeriesKeys.size() >= diagramInfo.getSeriesCount()) {
                break;
            }
        }
        return selectedSeriesKeys;
    }

    private ArrayList<String> getSortedKeysByAvg() {
        HashMap<String, Long> amountPerKey = new HashMap<>();

        for (GroupingNode curRoot : groupings.values()) {
            GroupingNode targetNode = getNodeWithKey(curRoot, 0, diagramInfo.getSelectedKeysAsArray());
            if (targetNode == null) {
                continue;
            }
            List<GroupingNode> targetChildren = targetNode.getChildren();
            for (GroupingNode curKeyNode : targetChildren) {
                Long keyVal = amountPerKey.getOrDefault(curKeyNode.getKey(), 0L);
                keyVal += getAmountOfCurUnit(curKeyNode);
                amountPerKey.put(curKeyNode.getKey().toString(), keyVal);
            }
        }
        return amountPerKey.entrySet()
                           .stream()
                           .sorted(Comparator.comparingLong((Map.Entry<String, Long> x) -> x.getValue()).reversed())
                           .limit(diagramInfo.getSeriesCount())
                           .map(Map.Entry::getKey)
                           .collect(Collectors.toCollection(ArrayList::new));
    }

    private List<GroupingNode> getSortedGroupings() {
        return groupings.entrySet()
                        .stream()
                        .sorted(Comparator.comparingLong(x -> x.getKey()))
                        .sequential()
                        .map(x -> getNodeWithKey(x.getValue(), 0, diagramInfo.getSelectedKeysAsArray()))
                        .collect(Collectors.toList());
    }

    private HashMap<String, Double> calculateRelativeGrowth() {
        List<GroupingNode> sortedNodes = getSortedGroupings();

        HashMap<String, Double> growthPerKey = new HashMap<>();
        if (sortedNodes.size() == 0) {
            return growthPerKey;
        }
        GroupingNode startTree = sortedNodes.get(0);
        if (startTree == null) {
            return growthPerKey;
        }
        GroupingNode lastTree = sortedNodes.get(sortedNodes.size() - 1);
        for (GroupingNode child : startTree.getChildren()) {
            double amount = getAmountOfCurUnit(child);
            // we only put the key into the map if the amount is not 0, because if it is 0 is should be in the 'other'-series anyway
            if (amount != 0) {
                growthPerKey.put(child.getKey().toString(), amount);
            }
        }
        for (String key : growthPerKey.keySet()) {
            double amount = 0;
            if (lastTree != null) {
                GroupingNode child = getNodeWithKey(lastTree, 0, new String[]{key});
                if (child != null) {
                    amount = getAmountOfCurUnit(child);
                }
            }
            double relativeGrowth = (amount - growthPerKey.get(key)) / growthPerKey.get(key);
            //now we overwrite the old value with the actual relative growth
            growthPerKey.put(key, relativeGrowth);
        }
        return growthPerKey;
    }

    private ArrayList<String> getSortedKeysByRelativeGrowth() {
        HashMap<String, Double> growthPerKey = calculateRelativeGrowth();
        return growthPerKey.entrySet()
                           .stream()
                           .sorted(Comparator.comparingDouble((Map.Entry<String, Double> x) -> x.getValue()).reversed())
                           .limit(diagramInfo.getSeriesCount())
                           .map(Map.Entry::getKey)
                           .collect(Collectors.toCollection(ArrayList::new));
    }

    private HashMap<String, Long> calculateAbsoluteGrowth() {
        List<GroupingNode> sortedNodes = getSortedGroupings();

        HashMap<String, Long> changePerKey = new HashMap<>();
        if (sortedNodes.size() == 0) {
            return changePerKey;
        }
        GroupingNode startTree = sortedNodes.get(0);
        if (startTree != null) {
            for (GroupingNode child : startTree.getChildren()) {
                //we multiply it by -1, because if the key is not present in the last tree, it should be a negative value and we dont have to perform further steps
                changePerKey.put(child.getKey().toString(), -1 * getAmountOfCurUnit(child));
            }
        }

        GroupingNode lastTree = sortedNodes.get(sortedNodes.size() - 1);

        if (lastTree != null) {
            for (GroupingNode child : lastTree.getChildren()) {
                String curKey = child.getKey().toString();
                if (!changePerKey.containsKey(curKey)) {
                    changePerKey.put(curKey, getAmountOfCurUnit(child));
                } else {
                    Long startValue = changePerKey.get(curKey);
                    //we ADD the startValue to the end value as the start value is negative
                    Long diff = getAmountOfCurUnit(child) + startValue;
                    changePerKey.put(curKey, diff);
                }
            }
        }
        return changePerKey;
    }

    private ArrayList<String> getSortedKeysByAbsoluteGrowth() {
        HashMap<String, Long> changePerKey = calculateAbsoluteGrowth();
        return changePerKey.entrySet()
                           .stream()
                           .sorted(Comparator.comparingLong((Map.Entry<String, Long> x) -> x.getValue()).reversed())
                           .limit(diagramInfo.getSeriesCount())
                           .map(Map.Entry::getKey)
                           .collect(Collectors.toCollection(ArrayList::new));
    }

    private ArrayList<String> getSortedKeys() {
        switch (diagramInfo.getSeriesSort()) {
            case START:
                List<Long> ascSortedTimes = groupings.keySet().stream()
                                                     .sorted(Comparator.comparingLong(x -> x)).collect(Collectors.toList());
                return getSortedKeysBySortedTimes(ascSortedTimes);
            case AVG:
                return getSortedKeysByAvg();
            case END:
                List<Long> descSortedTimes = groupings.keySet().stream()
                                                      .sorted(Comparator.comparingLong((Long x) -> x).reversed())
                                                      .collect(Collectors.toList());

                return getSortedKeysBySortedTimes(descSortedTimes);
            case ABS_GROWTH:
                return getSortedKeysByAbsoluteGrowth();
            case REL_GROWTH:
                return getSortedKeysByRelativeGrowth();
            default:
                return null;
        }
    }

    @Override
    protected AbstractIntervalXYDataset createDataSet(AppInfo appInfo) {
        //chart generation
        selectedSeriesKeys = getSortedKeys();

        //create series
        Map<Object, AddALotXYSeries> seriesMap = new HashMap<>();
        Map<Object, List<XYDataItem>> dataMap = new HashMap<>();
        AddALotXYSeries otherSeries = new AddALotXYSeries(OTHER, false);
        List<XYDataItem> otherData = new ArrayList<>();

        for (String curKey : selectedSeriesKeys) {
            AddALotXYSeries curSeries = new AddALotXYSeries(curKey, false);
            List<XYDataItem> curData = new ArrayList<>();
            seriesMap.put(curKey, curSeries);
            dataMap.put(curKey, curData);
        }

        //add data
        AtomicReference<Boolean> otherHasData = new AtomicReference<>(false);

        groupings.entrySet().stream().sequential().sorted(Comparator.comparingLong(x -> x.getKey())).forEach(entry -> {
            long objectsOfAllSelectedKeys = 0;
            GroupingNode targetNode = getNodeWithKey(entry.getValue(), 0, diagramInfo.getSelectedKeysAsArray());
            long otherUnitAmount;
            if (targetNode == null) {
                //the targetNode is null all series have a count of 0
                otherUnitAmount = 0;
                for (String curKey : selectedSeriesKeys) {
                    dataMap.get(curKey).add(new XYDataItem((long) entry.getKey(), 0));
                }
            } else {
                for (String curKey : selectedSeriesKeys) {
                    List<GroupingNode> targetChildren = targetNode.getChildren();
                    long curUnitAmount = targetChildren.stream().filter(x -> x.getKey().toString().equals(curKey))
                                                       .map(this::getAmountOfCurUnit)
                                                       .findFirst().orElse(0L);

                    dataMap.get(curKey).add(new XYDataItem((long) entry.getKey(), curUnitAmount));
                    objectsOfAllSelectedKeys += curUnitAmount;
                }

                otherUnitAmount = getAmountOfCurUnit(targetNode) - objectsOfAllSelectedKeys;
                if (otherUnitAmount != 0) {
                    otherHasData.set(true);
                }
            }
            otherData.add(new XYDataItem((long) entry.getKey(), otherUnitAmount));
        });

        //set data
        for (String curKey : selectedSeriesKeys) {
            seriesMap.get(curKey).setData(dataMap.get(curKey), false);
        }
        otherSeries.setData(otherData, false);

        // create and init dataset
        // series should be displayed descending - so biggest on top and lowest at the bottom (exception: "Other" which is at the very bottom)
        // to achieve this we have to add them in reversed order
        DefaultTableXYDataset dataset = new DefaultTableXYDataset();

        if (diagramInfo.isShowOtherSeries() && otherHasData.get()) {
            dataset.addSeries(otherSeries);
        }

        //add all the selected series (on top of the 'other' series)
        for (int i = selectedSeriesKeys.size() - 1; i >= 0; i--) {
            Object curKey = selectedSeriesKeys.get(i);
            dataset.addSeries(seriesMap.get(curKey));
        }

        // update growth table
        table.getItems().clear();
        Set<String> allPossibleKeys = getAllPossibleKeys();
        HashMap<String, Long> absGrowth = calculateAbsoluteGrowth();
        HashMap<String, Double> relGrowth = calculateRelativeGrowth();
        for (String name : allPossibleKeys) {
            Double relVal = relGrowth.get(name);
            table.getItems().add(new GrowthTableRow(name, relVal == null ? "---" : formatDouble(relVal), absGrowth.getOrDefault(name, 0L)));
        }
        table.setSorting();

        // update sub chart key selection combobox
        tab.updateKeySelectionComboBox(this);

        return dataset;
    }

    private String formatDouble(double val) {
        return String.valueOf(new BigDecimal(val * 100).setScale(1, BigDecimal.ROUND_HALF_UP).doubleValue());
    }

    private Set<String> getAllPossibleKeys() {
        return groupings.values()
                        .stream()
                        .map(x -> getNodeWithKey(x, 0, diagramInfo.getSelectedKeysAsArray()))
                        .filter(Objects::nonNull)
                        .flatMap(x -> x.getChildren().stream())
                        .distinct()
                        .map(x -> x.getKey().toString())
                        .collect(Collectors.toSet());
    }

    @Override
    protected void initializeChart() {
        lineChart = JFreeChartFactory.createLineXYChart(this, "", null, "Time", "", new DefaultTableXYDataset(), null, chartSynchronizer);
        stackedChart = JFreeChartFactory.createStackedXYAreaChart(this, "", null, "Time", "", new DefaultTableXYDataset(), null,
                                                                  new StandardXYToolTipGenerator(), null, chartSynchronizer);
        setCorrectChart();
    }

    private void setCorrectChart() {
        if (diagramInfo != null) { //prevent error on first execution
            if (diagramInfo.isUseLineChart()) {
                chartViewer.setChart(lineChart);
            } else {
                chartViewer.setChart(stackedChart);
            }
        } else {
            chartViewer.setChart(stackedChart); //use stacked chart as default
        }
    }

    @Override
    public AxisScaler<AbstractIntervalXYDataset> getScaler() {
        switch (diagramInfo.getDiagramMetric()) {
            case OBJECTS:
                return new AxisScaler.JFreeChartAxisScaler.ObjectsScaler(this);
            case BYTES:
            case RETAINED_SIZE:
            case TRANSITIVE_CLOSURE_SIZE:
                return new AxisScaler.JFreeChartAxisScaler.BytesScaler(this);
            default:
                return new AxisScaler.JFreeChartAxisScaler.NoAxisScaler(this);
        }
    }

    @Override
    protected EventHandler<ActionEvent> getOnConfigureAction() {
        return null;
    }

    public LifetimeVisualizationDiagramInfo getDiagramInfo() {
        return diagramInfo;
    }

    public Map<Long, GroupingNode> getGroupings() {
        return groupings;
    }

    public ChartSynchronizer getChartSynchronizer() {
        return chartSynchronizer;
    }

    public void setGroupings(Map<Long, GroupingNode> groupings) {
        this.zoomX(Double.MIN_VALUE, Double.MAX_VALUE);
        this.groupings = groupings;
    }

    public GrowthTable getTable() {
        return table;
    }

    public ArrayList<String> getSelectedSeriesKeys() {
        return selectedSeriesKeys;
    }

    public void setUseLineChart(boolean useLineChart) {
        diagramInfo.setUseLineChart(useLineChart);
        setCorrectChart();
    }
}