package at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.objectkind;

import at.jku.mevss.anttracks.heap.statistics.Statistics;
import at.jku.mevss.trace.gui.chart.base.AxisScaler;
import at.jku.mevss.trace.gui.chart.jfreechart.base.JFreeChartFactory;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.stackedarea.base.StackedAreaJFreeChartPane;
import at.jku.mevss.trace.gui.chart.jfreechart.xy.util.AddALotXYSeries;
import at.jku.mevss.trace.gui.classification.classifier.ObjectKindsClassifier;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import org.jfree.chart.labels.StandardXYToolTipGenerator;
import org.jfree.data.xy.AbstractIntervalXYDataset;
import org.jfree.data.xy.DefaultTableXYDataset;
import org.jfree.data.xy.XYDataItem;

import java.util.ArrayList;
import java.util.List;

public class ObjectKindStackedAreaJFreeChart extends StackedAreaJFreeChartPane<AppInfo> {

    public ObjectKindStackedAreaJFreeChart() {
        FXMLUtil.load(this, ObjectKindStackedAreaJFreeChart.class);
    }

    public DefaultTableXYDataset createDataSet(AppInfo appInfo) {
        AddALotXYSeries instanceSeries = new AddALotXYSeries(ObjectKindsClassifier.INSTANCES, false);
        List<XYDataItem> instancesData = new ArrayList<>();
        AddALotXYSeries smallArraySeries = new AddALotXYSeries(ObjectKindsClassifier.SMALL_ARRAY, false);
        List<XYDataItem> smallArrayData = new ArrayList<>();
        AddALotXYSeries bigArraySeries = new AddALotXYSeries(ObjectKindsClassifier.BIG_ARRAY, false);
        List<XYDataItem> bigArrayData = new ArrayList<>();

        for (int i = 0; i < appInfo.getStatistics().size(); i++) {
            Statistics stat = appInfo.getStatistics().get(i);
            double time = stat.info.time;

            XYDataItem instanceDataItem = new XYDataItem(time,
                                                         stat.eden.objectTypes.getInstances() + stat.survivor.objectTypes.getInstances() + stat.old.objectTypes
                                                                 .getInstances());
            XYDataItem smallDataItem = new XYDataItem(time,
                                                      stat.eden.objectTypes.getSmallArrays() + stat.survivor.objectTypes.getSmallArrays() + stat.old.objectTypes
                                                              .getSmallArrays());
            XYDataItem bigDataItem = new XYDataItem(time,
                                                    stat.eden.objectTypes.getBigArrays() + stat.survivor.objectTypes.getBigArrays() + stat.old.objectTypes.getBigArrays
                                                            ());

            instancesData.add(instanceDataItem);
            smallArrayData.add(smallDataItem);
            bigArrayData.add(bigDataItem);
        }

        instanceSeries.setData(instancesData, false);
        smallArraySeries.setData(smallArrayData, false);
        bigArraySeries.setData(bigArrayData, false);

        DefaultTableXYDataset dataset = new DefaultTableXYDataset();
        dataset.addSeries(bigArraySeries);
        dataset.addSeries(smallArraySeries);
        dataset.addSeries(instanceSeries);

        return dataset;
    }

    @Override
    protected void initializeChart() {
        chartViewer.setChart(JFreeChartFactory.createStackedXYAreaChart(this,
                                                                        "Object Kinds",
                                                                        null,
                                                                        "set by scaler",
                                                                        "set by scaler",
                                                                        new DefaultTableXYDataset(),
                                                                        null,
                                                                        new StandardXYToolTipGenerator(),
                                                                        null,
                                                                        chartSynchronizer));
    }

    @Override
    protected EventHandler<ActionEvent> getOnConfigureAction() {
        return null;
    }

    @Override
    public AxisScaler<AbstractIntervalXYDataset> getScaler() {
        return new AxisScaler.JFreeChartAxisScaler.ObjectsScaler(this);
    }
}
