package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.util.ImagePack;
import at.jku.mevss.trace.gui.utils.ImageUtil;

import java.util.Arrays;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Allocation Site", desc = "This classifier distinguishes objects based on their allocation site. "
        + "The " +
        "allocation site can be splitted up by packages.", example = "at -> jku -> anttracks -> classifier", type = ClassifierType.HIERARCHY, collection =
        ClassifierSourceCollection
        .ALL)
public class AllocationSiteClassifier extends Classifier<String[]> {

    @ClassifierProperty(overviewLevel = 10)
    private boolean flatten = true;

    public boolean getFlatten() {
        return flatten;
    }

    public void setFlatten(boolean flatten) {
        this.flatten = flatten;
    }

    @Override
    protected String[] classify() throws Exception {
        String allocationSiteString = allocationSite().callSites[0].text;

        if (flatten) {
            return new String[]{allocationSiteString};
        } else {
            int paramStart = allocationSiteString.indexOf('(');

            String signature = allocationSiteString.substring(0, paramStart);
            String declaringType = signature.substring(0, signature.lastIndexOf('.'));
            String method = signature.substring(declaringType.length() + 1);

            String[] allocationSite = declaringType.split("\\.");
            allocationSite = Arrays.copyOf(allocationSite, allocationSite.length + 1);
            allocationSite[allocationSite.length - 1] = method;
            return allocationSite;
        }
    }

    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{ImageUtil.getResourceImagePack("Allocation site", "allocation_site.png")};
    }
}
