/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.heap.symbols.AllocationSite;
import at.jku.mevss.anttracks.util.ImagePack;
import at.jku.mevss.trace.gui.utils.ImageUtil;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Call Sites", desc = "This classifier distinguishes objects based on their call sites. " + "An " +
        "allocation " +
        "site may consist of multiple call sites, representing the method call chain to the allocation site.", example = "HashMap.put() <- Main.main()", type =
        ClassifierType
        .HIERARCHY, collection = ClassifierSourceCollection.ALL)
public class CallSitesClassifier extends Classifier<String[]> {

    @ClassifierProperty(overviewLevel = 10)
    private boolean includeAllocationSite = false;

    public boolean getIncludeAllocationSite() {
        return includeAllocationSite;
    }

    public void setIncludeAllocationSite(boolean includeAllocationSite) {
        this.includeAllocationSite = includeAllocationSite;
    }

    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{ImageUtil.getResourceImagePack("Allocation site", "allocation_site.png")};
    }

    @Override
    public String[] classify() {
        int startIndex = includeAllocationSite ? 0 : 1;

        AllocationSite allocationSite = allocationSite();
        String[] res = new String[allocationSite.callSites.length - startIndex];

        for (int i = startIndex, j = 0; i < allocationSite.callSites.length; i++, j++) {
            AllocationSite.Location loc = allocationSite.callSites[i];
            res[j] = loc.text;
        }

        return res;
    }
}
