/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.ClassifierException;
import at.jku.mevss.anttracks.classification.Transformer;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.heap.IndexBasedHeap;
import at.jku.mevss.anttracks.heap.datastructures.DataStructure;
import at.jku.mevss.anttracks.heap.symbols.Symbols;

import java.util.function.Supplier;

@at.jku.mevss.anttracks.classification.annotations.Classifier(
        name = "Datastructure Composition",
        desc = "This transformer is used to classify the objects based on its data structure composition",
        example = "",
        type = ClassifierType.ONE,
        collection = ClassifierSourceCollection.FASTHEAP)
public class DataStructureCompositionTransformer extends Transformer {
    private ClassifierChain typeClassifierChain = new ClassifierChain(new TypeClassifier());

    @Override
    public void setup(Supplier<Symbols> symbolsSupplier, Supplier<IndexBasedHeap> fastHeapSupplier) {
        super.setup(symbolsSupplier, fastHeapSupplier);
        typeClassifierChain.forEach(classifier -> classifier.setup(symbolsSupplier, fastHeapSupplier));
    }

    @Override
    public GroupingNode classify(GroupingNode transformerRoot) throws ClassifierException {
        DataStructure dataStructure = fastHeap().getHeadedDataStructure(index());

        if (dataStructure != null) {
            dataStructure.visitAllObjects(
                    containedIdx -> {
                        if (containedIdx != index()) {
                            ((GroupingNode.FastHeapGroupingNode) transformerRoot).classify(fastHeap(), containedIdx, typeClassifierChain, null);
                        }
                    },
                    fastHeap());
        }

        return transformerRoot;
    }

    @Override
    public String title() {
        return "Datastructure Composition";
    }
}
