/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.ClassifierException;
import at.jku.mevss.anttracks.classification.Transformer;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.heap.datastructures.DataStructure;
import at.jku.mevss.anttracks.util.ImagePack;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Data structures", desc = "This transformer is used to classify the data structures that contain " +
        "this " +
        "object", example = "(using \"Data structure and Type\") Data structures -> java.util.HashMap & java.util.ArrayList", type = ClassifierType.ONE, collection =
        ClassifierSourceCollection.FASTHEAP)
public class DataStructuresTransformer extends Transformer {

    @ClassifierProperty(overviewLevel = 10)
    protected ClassifierChain classifiers = new ClassifierChain();

    public ClassifierChain getClassifiers() {
        return classifiers;
    }

    public void setClassifiers(ClassifierChain classifiers) {
        this.classifiers = classifiers;
    }

    @ClassifierProperty(overviewLevel = 10)
    private boolean includeIndirectDataStructures = true;

    public boolean getIncludeIndirectDataStructures() {
        return includeIndirectDataStructures;
    }

    public void setIncludeIndirectDataStructures(boolean includeIndirectDataStructures) {
        this.includeIndirectDataStructures = includeIndirectDataStructures;
    }

    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{
                //                ImageUtil.getResourceImagePack("Data structures", "ds.png")
        };
    }

    @Override
    public GroupingNode classify(GroupingNode transformerRoot) throws ClassifierException {
        DataStructure[] dataStructures = fastHeap().getDataStructures(index(), includeIndirectDataStructures, false);
        if (dataStructures != null) {
            for (int i = 0; i < dataStructures.length; i++) {
                int dsHead = dataStructures[i].headIdx;
                ((GroupingNode.FastHeapGroupingNode) transformerRoot).classify(fastHeap(), dsHead, classifiers, null);
            }
        }
        return transformerRoot;
    }

    @Override
    public String title() {
        return "Data structures";
    }
}
