/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.heap.symbols.AllocatedType;
import at.jku.mevss.anttracks.util.Consts;
import at.jku.mevss.anttracks.util.ImagePack;
import at.jku.mevss.trace.gui.utils.ImageUtil;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Object Kind", desc = "This classifier distinguishes objects based on their object kind. This may " +
        "either be " +
        "\"Mirror\", i.e., Class<> objects, \"Instance\" for object " + "instances, \"Small Array\" for arrays with less than 255 elements, or \"Big Array\" for arrays" +
        " with at least" +
        " 255 elements", example = "Small Array", type = ClassifierType.ONE, collection = ClassifierSourceCollection.ALL)
public class ObjectKindsClassifier extends Classifier<String> {

    public static final String MIRROR = "Mirror";
    public static final String INSTANCES = "Instance";
    public static final String SMALL_ARRAY = "Small Array";
    public static final String BIG_ARRAY = "Big Array";

    @Override
    public String classify() {
        String id;

        if (isArray()) {
            if (arrayLength() < Consts.ARRAY_SIZE_MAX_SMALL) {
                id = SMALL_ARRAY;
            } else {
                id = BIG_ARRAY;
            }
        } else {
            if (type() instanceof AllocatedType.MirrorAllocatedType) {
                id = MIRROR;
            } else {
                id = INSTANCES;
            }
        }

        return id;
    }

    public ImagePack[] loadIcons() {
        return new ImagePack[]{ImageUtil.getResourceImagePack("Object kinds", "objectkinds.png")};
    }
}
