/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierException;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;

import java.util.Arrays;

import static at.jku.mevss.anttracks.heap.FastHeap.NULL_INDEX;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Out Degree", desc = "This classifier distinguishes objects based on the number of their pointers" +
        ".", example
        = "4", type = ClassifierType.ONE, collection = ClassifierSourceCollection.FASTHEAP)
public class OutDegreeClassifier extends Classifier<String> {

    @ClassifierProperty(overviewLevel = 10)
    private boolean alsoCountNullFields = false;

    public boolean getAlsoCountNullFields() {
        return alsoCountNullFields;
    }

    public void setAlsoCountNullFields(boolean showNulls) {
        this.alsoCountNullFields = showNulls;
    }

    @Override
    public String classify() throws ClassifierException {
        int[] toPointerIndices = pointsToIndices();
        return toPointerIndices == null ?
               "No pointer array available" :
               String.valueOf(alsoCountNullFields ? toPointerIndices.length : Arrays.stream(toPointerIndices).filter(x -> x != NULL_INDEX).count());
    }

    /*
    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{
                ImageUtil.getResourceImagePack("Address", "address.png")
        };
    }
    */
}
