/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.Transformer;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.classification.nodes.GroupingNode;
import at.jku.mevss.anttracks.heap.IndexBasedHeap;
import at.jku.mevss.anttracks.heap.objects.ObjectInfoCache;
import at.jku.mevss.anttracks.heap.symbols.Symbols;
import at.jku.mevss.anttracks.util.ImagePack;
import at.jku.mevss.trace.gui.utils.ImageUtil;

import java.util.function.Supplier;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Pointed From", desc = "This classifier is used to classify the objects that point to a given " +
        "object.",
        example = "(using \"Pointed from Type\") Pointed From -> HashMap#Node", type = ClassifierType.ONE, collection = ClassifierSourceCollection.FASTHEAP)
public class PointedFromTransformer extends Transformer {

    @ClassifierProperty(overviewLevel = 10)
    protected ClassifierChain classifiers = new ClassifierChain();

    public ClassifierChain getClassifiers() {
        return classifiers;
    }

    public void setClassifiers(ClassifierChain classifiers) {
        this.classifiers = classifiers;
        if (classifiers != null && classifiers.length() > 0) {
            classifiers.getList().forEach(classifier -> classifier.setup(() -> symbols(), () -> fastHeap()));
        }
    }

    ObjectInfoCache objInfoCache = new ObjectInfoCache();

    @Override
    public void setup(Supplier<Symbols> symbolsSupplier, Supplier<IndexBasedHeap> fastHeapSupplier) {
        super.setup(symbolsSupplier, fastHeapSupplier);
        if (classifiers != null && classifiers.length() > 0) {
            classifiers.getList().forEach(classifier -> classifier.setup(symbolsSupplier, fastHeapSupplier));
        }
    }

    @Override
    public GroupingNode classify(GroupingNode g) throws Exception {
        int[] pointedFrom = pointedFromIndices();
        if (pointedFrom != null && pointedFrom.length > 0) {
            for (int i = 0; i < pointedFrom.length; i++) {
                ((GroupingNode.FastHeapGroupingNode) g).classify(fastHeap(), pointedFrom[i], classifiers, null);
            }
        }
        return g;
    }

    @Override
    public String title() {
        return Classifier.CLASSIFICATION_MODE == ClassificationMode.LIST ? "Pointed From (# Objects)" : "Pointers From (# Pointers)";
    }

    public ImagePack[] loadIcons() {
        return new ImagePack[]{ImageUtil.getResourceImagePack("Pointed from", "pointed_from.png")};
    }
}
