/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.classifier;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.annotations.ClassifierProperty;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierSourceCollection;
import at.jku.mevss.anttracks.classification.enumerations.ClassifierType;
import at.jku.mevss.anttracks.heap.datastructures.DataStructure;
import at.jku.mevss.anttracks.util.ImagePack;

@at.jku.mevss.anttracks.classification.annotations.Classifier(name = "Role in data structure", desc = "This classifier distinguishes objects based what role they take " +
        "in one of " +
        "the recognized data structures", example = "'Backbone' or 'Not part of any data structure'", type = ClassifierType.ONE, collection =
        ClassifierSourceCollection.FASTHEAP)
public class RoleInDataStructureClassifier extends Classifier<String> {

    @ClassifierProperty(overviewLevel = 10)
    private boolean includeIndirectDataStructures = true;

    public boolean getIncludeIndirectDataStructures() {
        return includeIndirectDataStructures;
    }

    public void setIncludeIndirectDataStructures(boolean includeIndirectDataStructures) {
        this.includeIndirectDataStructures = includeIndirectDataStructures;
    }

    @Override
    public ImagePack[] loadIcons() {
        return new ImagePack[]{
                //                ImageUtil.getResourceImagePack("Data Structure Membership", "ds_mem.png")
        };
    }

    @Override
    public String classify() {
        if (DataStructure.isDataStructureHead(index(), fastHeap())) {
            return "Head";
        } else if (fastHeap().getDataStructures(index(), includeIndirectDataStructures, false) != null) {
            return "Backbone or data";
        } else {
            return "Not part of any data structure";
        }
    }
}
