/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.component.configurationpane;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierFactory;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.FilterFactory;
import at.jku.mevss.trace.gui.classification.classifier.component.selectedconditionalpane.SelectedConditionalObjectClassifierPane;
import at.jku.mevss.trace.gui.classification.classifier.component.selectionpane.ObjectClassifierSelectionPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane.ClassificationSelectionListener;
import at.jku.mevss.trace.gui.classification.filter.component.selectionpane.ObjectFilterSelectionPane;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.AvailableClassifierInfo;
import at.jku.mevss.trace.gui.model.SelectedClassifierInfo;
import at.jku.mevss.trace.gui.utils.Consts;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.ImageUtil;
import javafx.beans.DefaultProperty;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.image.ImageView;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.VBox;
import org.controlsfx.control.ToggleSwitch;

@DefaultProperty(value = "extension")
public abstract class ConfigurationPane extends BorderPane {

    @FXML
    protected ObjectFilterSelectionPane filterSelectionPane;
    @FXML
    protected ObjectClassifierSelectionPane classifierSelectionPane;
    @FXML
    protected SelectedConditionalObjectClassifierPane selectedConditionalClassifierPane;
    @FXML
    private Button configureButton;
    @FXML
    private VBox extension;
    @FXML
    private ToggleSwitch dataStructureSwitch;

    protected AppInfo appInfo;
    private AvailableClassifierInfo availableClassifierInfo;

    protected boolean isMaxMode;

    public ConfigurationPane() {
        FXMLUtil.load(this, ConfigurationPane.class);
    }

    public void init(AppInfo appInfo, AvailableClassifierInfo availableClassifierInfo, SelectedClassifierInfo selectedClassifierInfo) {
        init(appInfo, availableClassifierInfo, selectedClassifierInfo, false);
    }

    public void init(AppInfo appInfo, AvailableClassifierInfo availableClassifierInfo, SelectedClassifierInfo selectedClassifierInfo, boolean prohibitDuplicates) {
        this.appInfo = appInfo;
        this.availableClassifierInfo = availableClassifierInfo;

        ImageView configureIcon = ImageUtil.getIconNode(Consts.SETTINGS_IMAGE);
        configureIcon.setFitHeight(24);
        configureIcon.setFitWidth(24);
        configureButton.setGraphic(configureIcon);

        // vbox.setStyle("-fx-padding: 0px;");
        filterSelectionPane.init(availableClassifierInfo, 0, true);
        filterSelectionPane.addListener(createFilterListener());
        classifierSelectionPane.init(availableClassifierInfo, 0, prohibitDuplicates);
        classifierSelectionPane.addListener(createClassifierListener());
        selectedConditionalClassifierPane.init(selectedClassifierInfo);

        // Start maximized
        isMaxMode = false;
        switchMode();

        // define data structure switch behaviour
        Filter onlyDataStructureHeadsFilter = availableClassifierInfo.getAvailableFilter().get("Only data structure heads");
        dataStructureSwitch.selectedProperty().addListener((obs, wasSelected, isSelected) -> {
            if (!wasSelected && isSelected) {
                dataStructureSwitch.setText("Data structures");
                filterSelectionPane.addSelectedButNotShown(onlyDataStructureHeadsFilter);
            } else if (wasSelected && !isSelected) {
                dataStructureSwitch.setText("Objects");
                filterSelectionPane.removeSelectedButNotShown(onlyDataStructureHeadsFilter);
            }
        });
    }

    public ToggleSwitch getDataStructureSwitch() {
        return dataStructureSwitch;
    }

    public ObjectFilterSelectionPane getFilterSelectionPanel() {
        return filterSelectionPane;
    }

    public ObjectClassifierSelectionPane getClassifierSelectionPanel() {
        return classifierSelectionPane;
    }

    public ObservableList<Node> getExtension() {
        return extension.getChildren();
    }

    public void switchMode() {
        if (isMaxMode) {
            filterSelectionPane.switchToAnalysisPerspective();
            classifierSelectionPane.switchToAnalysisPerspective();
            selectedConditionalClassifierPane.setVisibleIfContainsContent();
            selectedConditionalClassifierPane.setManagedIfContainsContent();
            configureButton.setVisible(true);
            configureButton.setManaged(true);
        } else {
            filterSelectionPane.switchToConfigurationPerspective();
            classifierSelectionPane.switchToConfigurationPerspective();
            selectedConditionalClassifierPane.setVisible(false);
            selectedConditionalClassifierPane.setManaged(false);
            configureButton.setVisible(false);
            configureButton.setManaged(false);
        }
        isMaxMode = !isMaxMode;
        selectedConditionalClassifierPane.update();
    }

    public boolean isMaxMode() {
        return isMaxMode;
    }

    protected abstract ClassificationSelectionListener<Classifier<?>, ClassifierFactory> createClassifierListener();

    protected abstract ClassificationSelectionListener<Filter, FilterFactory> createFilterListener();

    public abstract void setLoadingMode(boolean isLoading);

    public abstract void setAppropriateConfigView();

    public AvailableClassifierInfo getAvailableClassifierInfo() {
        return availableClassifierInfo;
    }
}
