/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.component.treetable;

import javax.swing.*;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import java.awt.*;

public class TreeNodeTableCellEditor<T> extends AbstractCellEditor implements TableCellEditor {

    private final TableCellRenderer renderer;
    private final ListSelectionModel selection;

    public TreeNodeTableCellEditor(TableCellRenderer renderer, ListSelectionModel selection) {
        this.renderer = renderer;
        this.selection = selection;
    }

    @SuppressWarnings("unchecked")
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int r, int c) {
        SwingUtilities.invokeLater(() -> flipExpanded((TreeNode<T>) value, r));
        return renderer.getTableCellRendererComponent(table, value, isSelected, false, r, c);
    }

    private void flipExpanded(TreeNode<T> node, int row) {
        fireEditingCanceled();
        selection.clearSelection();
        node.setExpanded(!node.isExpanded());
        selection.addSelectionInterval(row, row);
    }

    @Override
    public Object getCellEditorValue() {
        return null;
    }

}
