/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.dialog.openfile;

import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.ClientInfo;
import javafx.scene.control.ButtonType;
import javafx.scene.control.Dialog;

import java.io.File;

public final class OpenFileDialog extends Dialog<Boolean> {

    private final OpenFileDialogPane dialogPane;

    public OpenFileDialog() {
        this(null);
    }

    public OpenFileDialog(AppInfo appInfo) {
        if (appInfo != null) {
            dialogPane = new OpenFileDialogPane(appInfo);
        } else {
            dialogPane = new OpenFileDialogPane();
        }

        setDialogPane(dialogPane);
        setResultConverter(value -> value == ButtonType.APPLY);

        dialogPane.getApplyButton().setOnAction(ae -> {
            if (getTraceFile() != null) {
                ClientInfo.traceDirectory = getTraceFile().getAbsolutePath().substring(0, getTraceFile().getAbsolutePath().lastIndexOf(File.separator));
            }
            if (getFeaturesFile() != null) {
                ClientInfo.featureDirectory = getFeaturesFile().getAbsolutePath().substring(0, getFeaturesFile().getAbsolutePath().lastIndexOf(File.separator));
            }
        });
    }

    public String getApplicationName() {
        return dialogPane.getApplicationName();
    }

    public File getTraceFile() {
        return dialogPane.getTraceFile();
    }

    public File getFeaturesFile() {
        return dialogPane.getFeaturesFile();
    }

}
