/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.component.maintabbedpane;

import at.jku.mevss.trace.gui.frame.main.tab.fallback.FallbackTab;
import at.jku.mevss.trace.gui.utils.AppLoader;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.Tab;
import javafx.scene.control.TabPane;
import javafx.scene.layout.AnchorPane;

import java.util.List;
import java.util.stream.Collectors;

public class MainTabbedPane extends AnchorPane {

    @FXML
    private TabPane tabPane;
    @FXML
    private FallbackTab fallbackTab;

    private Tab addTab;

    public MainTabbedPane() {
        FXMLUtil.load(this, MainTabbedPane.class);
    }

    public void init() {
        initAddTab();
    }

    public void addTab(Node icon, String title, Node content) {
        Tab tab = new Tab();

        tab.setGraphic(icon);
        tab.setText(title);
        tab.setContent(content);
        tab.setClosable(true);

        addTab(tab);
    }

    public void addSelectedItemListener(ChangeListener<Tab> listener) {
        tabPane.getSelectionModel().selectedItemProperty().addListener(listener);
    }

    public void addSelectedIndexListener(ChangeListener<Number> listener) {
        tabPane.getSelectionModel().selectedIndexProperty().addListener(listener);
    }

    public List<Tab> getTabs() {
        return tabPane.getTabs();
    }

    public List<Node> getTabContents() {
        return tabPane.getTabs().stream().map(t -> t.getContent()).filter(c -> c != null).collect(Collectors.toList());
    }

    public Tab getCurrentTab() {
        return tabPane.getSelectionModel().getSelectedItem();
    }

    public int getCloseableTabCount() {
        return tabPane.getTabs().filtered(t -> t.isClosable()).size();
    }

    private void initAddTab() {
        addTab = new Tab();
        addTab.setText("+");
        addTab.setClosable(false);
        addTab.selectedProperty().addListener((observable, oldValue, newValue) -> {
            if (!oldValue && newValue) {
                AppLoader.load();
            }
            if (tabPane.getSelectionModel().getSelectedIndex() == tabPane.getTabs().size() - 1) {
                tabPane.getSelectionModel().selectPrevious();
            }
        });
    }

    public void addTab(Tab tab) {
        Platform.runLater(() -> {
            tab.setOnCloseRequest(e -> {
                if (getCloseableTabCount() <= 1) {
                    tabPane.getTabs().add(0, fallbackTab);
                    tabPane.getTabs().remove(addTab);
                }
            });
            tabPane.getTabs().add(tabPane.getTabs().size() - 1, tab);
            if (tabPane.getTabs().contains(fallbackTab)) {
                tabPane.getTabs().add(addTab);
                tabPane.getTabs().remove(1);
            }
            tabPane.getSelectionModel().select(tab);
        });
    }
}
