/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.component.statuspane.component.operationpane;

import at.jku.mevss.trace.gui.utils.FXMLUtil;
import at.jku.mevss.trace.gui.utils.ImageUtil;
import at.jku.mevss.trace.gui.utils.Operation;
import javafx.application.Platform;
import javafx.beans.binding.Bindings;
import javafx.fxml.FXML;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressIndicator;
import javafx.scene.layout.BorderPane;

public final class OperationPane extends BorderPane {

    @FXML
    private Label titleLabel;
    @FXML
    private ProgressIndicator progressIndicator;
    @FXML
    private Label remainingLabel;
    @FXML
    private Button cancelButton;

    private final Operation operation;

    public OperationPane(Operation operation) {
        this.operation = operation;
        FXMLUtil.load(this, OperationPane.class);
    }

    public void initialize() {
        Platform.runLater(() -> {
            titleLabel.textProperty().bind(Bindings.format("%s: %s", operation.titleProperty(), operation.messageProperty()));
            remainingLabel.textProperty().bind(Bindings.format(" %s", operation.remainingTimeTextProperty()));
            progressIndicator.progressProperty().bind(operation.progressProperty());
        });

        cancelButton.setGraphic(ImageUtil.getResourceImagePack("Cancel", "delete.png").getAsNewNode());
    }

    // Interface for old works that do not implement AntTask
    public void progress(double progress) {
        operation.progress(progress);
    }

    public void cancel() {
        operation.cancel();
    }
}
