/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.application;

import at.jku.mevss.anttracks.util.ApplicationStatistics;
import at.jku.mevss.anttracks.util.ThreadUtil;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer;
import at.jku.mevss.trace.gui.chart.base.ChartSynchronizer.SelectionListener;
import at.jku.mevss.trace.gui.chart.base.SelectedItemsList;
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab;
import at.jku.mevss.trace.gui.frame.main.tab.application.component.rootchartpane.RootChartPane;
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.DataStructureDevelopmentTab;
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.parser.DataStructureDevelopmentParserTask;
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.DataStructureDevelopmentOverviewTab;
import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.HeapDiffingTask;
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.HeapStateTask;
import at.jku.mevss.trace.gui.frame.main.tab.heapvisualization.HeapVisualizationTask;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.LifetimeVisualizationInfo;
import at.jku.mevss.trace.gui.frame.main.tab.lifetimevisualization.LifetimeVisualizationTab;
import at.jku.mevss.trace.gui.frame.main.tab.timelapse.workers.TimelapseTask;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.AppInfo.ChangeType;
import at.jku.mevss.trace.gui.model.DiffInfo;
import at.jku.mevss.trace.gui.model.DiffStatisticsInfo;
import at.jku.mevss.trace.gui.model.SelectedClassifierInfo;
import at.jku.mevss.trace.gui.utils.*;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.TextArea;
import javafx.scene.layout.AnchorPane;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.Region;

import java.security.InvalidParameterException;
import java.util.Comparator;

import static at.jku.mevss.trace.gui.frame.main.GUI.CLIENT_INFO;

public class ApplicationTab extends ApplicationBaseTab {

    /*
    private static final String selectDiffingText = "Diff from";
    private static final String startDiffingText = "Examine diff";
    private static final String lifetimevisualizationText = "Lifetime Visualization (experimental)";
    private static final String diffLifetimevisualizationText = "Diff Lifetime Visualization (experimental)";
    */

    @FXML
    Button buttonDiffing;
    @FXML
    Button buttonCancelDiffing;
    @FXML
    ComboBox<String> heapStateComboBox;
    @FXML
    ComboBox<String> diffComboBox;
    @FXML
    AnchorPane rootChartPaneContainer;
    @FXML
    BorderPane borderPane;
    @FXML
    private TextArea metricText;

    private RootChartPane rootChartPane;

    ChartSynchronizer chartSync;

    public ApplicationTab() {
        FXMLUtil.load(this, ApplicationTab.class);
    }

    @Override
    public void init(AppInfo appInfo) {
        super.init(appInfo);

        initializeTab();
        initializeChartSynchronizer();
        initializeRootChartPane();
    }

    private void initializeTab() {
        setGraphic(ImageUtil.getIconNode(Consts.APP_IMAGE));
        setText("Application - " + getAppInfo().getAppName());
    }

    private void initializeChartSynchronizer() {
        chartSync = new ChartSynchronizer();
        chartSync.addSelectionListener(new SelectionListener() {

            @Override
            public void selectionChanged(ChartSynchronizer sender) {
                updateButtonState(sender);
            }

            @Override
            public void inCompareModeChanged(ChartSynchronizer sender) {
                updateButtonState(sender);
            }

            private void updateButtonState(ChartSynchronizer sender) {
                heapStateComboBox.setDisable(true);
                diffComboBox.setDisable(true);

                if (sender.isInCompareMode()) {
                    // Diffing
                    diffComboBox.setDisable(!chartSync.isInCompareMode() || chartSync.getSelection().size() != 2);
                    buttonDiffing.setVisible(false);
                    buttonDiffing.setManaged(false);
                    buttonCancelDiffing.setVisible(true);
                    buttonCancelDiffing.setManaged(true);
                } else {
                    // Heap State
                    heapStateComboBox.setDisable(chartSync.getSelection().size() != 1);
                    buttonDiffing.setVisible(true);
                    buttonDiffing.setManaged(true);
                    buttonDiffing.setDisable(chartSync.getSelection().size() != 1);
                    buttonCancelDiffing.setVisible(false);
                    buttonCancelDiffing.setManaged(false);
                }
            }
        });
    }

    private void initializeRootChartPane() {
        rootChartPane = new RootChartPane();
        rootChartPane.init(chartSync);
        rootChartPaneContainer.getChildren().add(rootChartPane);
    }

    @Override
    protected void cleanupOnClose() {

    }

    public void lifetimeVisualization() {
        if (!appInfo.isParsingCompleted()) {
            Alert notReady = new Alert(Alert.AlertType.ERROR);
            notReady.setTitle("Parsing not finished");
            notReady.getDialogPane().setMinHeight(Region.USE_PREF_SIZE);
            notReady.setContentText("Please wait until the parsing is finished!");
            WindowUtil.centerInMainFrame(notReady);
            notReady.showAndWait();
            return;
        }

        //calc start and end time
        long startTime;
        long endTime;
        if (chartSync.isInCompareMode()) {
            SelectedItemsList selection = chartSync.getSelection();
            startTime = Math.min((long) selection.get(0).x, (long) selection.get(1).x);
            endTime = Math.max((long) selection.get(0).x, (long) selection.get(1).x);
        } else {
            startTime = appInfo.getStatistics().stream().min(Comparator.comparingLong(x -> x.info.time)).get().info.time;
            endTime = appInfo.getStatistics().stream().max(Comparator.comparingLong(x -> x.info.time)).get().info.time;
        }

        LifetimeVisualizationTab lifetimeVisualizationTab = new LifetimeVisualizationTab();
        lifetimeVisualizationTab.init(new LifetimeVisualizationInfo(appInfo, startTime, endTime, new SelectedClassifierInfo(null, null, null)));
        CLIENT_INFO.getMainFrame().addTab(lifetimeVisualizationTab);
    }

    public void examine() {
        if (chartSync.getSelection().size() == 1) {
            ThreadUtil.startTask(new HeapStateTask(appInfo, (long) chartSync.getSelection().get(0).x));
        }
    }

    public void diff() {
        if (chartSync.isInCompareMode()) {
            chartSync.setInCompareMode(false);
        } else {
            chartSync.setInCompareMode(true);
        }
    }

    public void heapVisualization() {
        HeapVisualizationTask task = new HeapVisualizationTask(appInfo, (long) chartSync.getSelection().get(0).x);
        ThreadUtil.startTask(task);
    }

    public void timeLapse() {
        TimelapseTask worker = new TimelapseTask(getAppInfo(), null);
        ThreadUtil.startTask(worker);
    }

    public synchronized void plot() {
        ApplicationStatistics.Measurement m = ApplicationStatistics.getInstance().createMeasurement("Plot");
        rootChartPane.plot(getAppInfo());
        m.end();
    }

    @Override
    protected void appInfoChangeAction(ChangeType type) {
        Platform.runLater(() -> setText("Application - " + getAppInfo().getAppName()));

        if (type == ChangeType.SHOW_FEATURES || type == ChangeType.ALIVE_DEAD_PANEL_TYPE) {
            plot();
        } else if (type == ChangeType.FILE || type == ChangeType.TRACE || type == ChangeType.FEATURE) {
            chartSync.clear();
            AppLoader.parse(getAppInfo(), this);
        }
    }

    public void setMetricText(String text) {
        this.metricText.setText(text);
    }

    public void setTask(MetaParserTask task) {
        this.tasks.clear();
        this.tasks.add(task);
    }

    public void heapStateComboBoxSelection() {
        if (heapStateComboBox.getValue() == null) {
            return;
        }

        switch (heapStateComboBox.getValue()) {
            case "Open heap state inspection tab":
                examine();
                break;
            case "Visualize heap":
                heapVisualization();
                break;
            case "Timelapse (experimental)":
                break;
            default:
                throw new InvalidParameterException("Unknown value was selected in diffComboBox @ ApplicationTab: " + diffComboBox.getValue());
        }
        // Show prompt text again
        // TODO: Currently this throws a NullPointerException somewhere in the background, even though it is suggested to use this method:
        // http://www.java2s.com/Code/Java/JavaFX/SetvaluetonulltocleartheComboBox.htm
        // Okay, found the reason: https://stackoverflow.com/questions/32370394/javafx-combobox-change-value-causes-indexoutofboundsexception
        // In JavaFX, you cannot change the contents of an ObservableList while a change is already in progress
        // DIRTY DIRTY workaround: run "setValue"  a bit deferred
        Platform.runLater(() -> heapStateComboBox.setValue(null));
    }

    public void diffComboBoxSelection() {
        if (diffComboBox.getValue() == null) {
            return;
        }

        switch (diffComboBox.getValue()) {
            case "Perm/Born/Died/Temp diff":
                ThreadUtil.startTask(new HeapDiffingTask(new DiffStatisticsInfo(new DiffInfo(appInfo, chartSync.getSelection().get()), null, null, null)));
                break;
            case "Lifetime visualization":
                lifetimeVisualization();
                break;
            case "Timelapse (experimental)":
                timeLapse();
                break;
            case "Datastructure development analysis":
                DataStructureDevelopmentTab tab = new DataStructureDevelopmentTab();
                long fromTime = (long) chartSync.getSelection().get().get(0).x;
                long toTime = (long) chartSync.getSelection().get().get(1).x;

                tab.init(appInfo, fromTime, toTime);
                CLIENT_INFO.getMainFrame().addTab(tab);

                DataStructureDevelopmentParserTask parserTask = new DataStructureDevelopmentParserTask(appInfo, tab.getOverviewTab(), fromTime, toTime);
                tab.tasks.add(parserTask);
                ThreadUtil.startTask(parserTask);
                break;
            default:
                throw new InvalidParameterException("Unknown value was selected in diffComboBox @ ApplicationTab: " + diffComboBox.getValue());
        }
        // Show prompt text again
        // TODO: Currently this throws a NullPointerException somewhere in the background, even though it is suggested to use this method:
        // http://www.java2s.com/Code/Java/JavaFX/SetvaluetonulltocleartheComboBox.htm
        // Okay, found the reason: https://stackoverflow.com/questions/32370394/javafx-combobox-change-value-causes-indexoutofboundsexception
        // In JavaFX, you cannot change the contents of an ObservableList while a change is already in progress
        // DIRTY DIRTY workaround: run "setValue"  a bit deferred
        Platform.runLater(() -> diffComboBox.setValue(null));
    }
}
