/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.component;

import at.jku.mevss.trace.gui.frame.main.tab.heapdiffing.HeapDiffingTab;
import at.jku.mevss.trace.gui.model.DiffStatisticsInfo;
import javafx.geometry.Insets;
import javafx.scene.control.Button;
import javafx.scene.layout.HBox;

import javax.swing.*;

public class HeapDiffingOperationPane extends HBox {

    // Info objects
    private final DiffStatisticsInfo diffStatisticsInfo;

    // References to view and other panels
    private final HeapDiffingTab heapDiffingTab;

    // Buttons
    private final Button updateClassifierButton;
    private final Button cancelButton;
    private final Button exportAsGexfButton;

    public HeapDiffingOperationPane(DiffStatisticsInfo diffStatisticsInfo, HeapDiffingTab heapDiffingTab) {
        this.diffStatisticsInfo = diffStatisticsInfo;
        this.heapDiffingTab = heapDiffingTab;

        setPadding(new Insets(5));
        setSpacing(5);

        updateClassifierButton = new Button("Apply");
        updateClassifierButton.setOnAction(ae -> runWorker());
        cancelButton = new Button("Cancel");
        cancelButton.setOnAction(ae -> cancelEdit());
        exportAsGexfButton = new Button("Export as .gefx");
        exportAsGexfButton.setOnAction(ae -> JOptionPane.showMessageDialog(null,
                                                                           ".gexf export is not yet supported.",
                                                                           "Export currently not supported!",
                                                                           JOptionPane.INFORMATION_MESSAGE));
        getChildren().add(updateClassifierButton);
        getChildren().add(cancelButton);
        getChildren().add(exportAsGexfButton);
    }

    /**
     * Runs a {@link DetailedGroupingWorker} which updates the UI on finnishing
     */
    private void runWorker() {
        heapDiffingTab.acceptEdit();
        heapDiffingTab.setAppropriateConfigView();
        // If diffing is still running, new diffStatInfo settings will be used for next classification
        if (diffStatisticsInfo.getDiffInfo().getHeapDiff() != null) {
            // heap diff is non-null if diffing has finished. Need to run a reclassification
            // TODO: Create worker for this, currently UI is blocking
            diffStatisticsInfo.setGrouping(diffStatisticsInfo.getDiffInfo()
                                                             .getHeapDiff()
                                                             .classify(diffStatisticsInfo.getSelectedClassifiers(),
                                                                       diffStatisticsInfo.getSelectedConditionalClassifier(),
                                                                       diffStatisticsInfo.getSelectedFilters()));
            heapDiffingTab.updateTable();
        }
    }

    public void setClassifierUpdateEnabled(boolean b) {
        updateClassifierButton.setDisable(!b);
    }

    public void setExportEnabled(boolean b) {
        exportAsGexfButton.setDisable(!b);
    }

    private void cancelEdit() {
        heapDiffingTab.cancelEdit();
        heapDiffingTab.setAppropriateConfigView();
    }

}
