/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.heapvisualization.model;

import at.jku.mevss.trace.gui.frame.main.tab.heapvisualization.HeapVisualizationTab;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;
import java.awt.*;
import java.util.List;
import java.util.Map.Entry;
import java.util.concurrent.CopyOnWriteArrayList;

/**
 * @author Christina Rammerstorfer
 */
public class ClassificationColorTableModel implements TableModel {

    private final ObjectVisualizationData data;
    private final HeapVisualizationTab tab;
    private final List<TableModelListener> listeners;

    public ClassificationColorTableModel(ObjectVisualizationData data, HeapVisualizationTab tab) {
        this.data = data;
        this.tab = tab;
        listeners = new CopyOnWriteArrayList<TableModelListener>();
    }

    @Override
    public int getRowCount() {
        return data.getClassificationColors().size();
    }

    @Override
    public int getColumnCount() {
        return 2;
    }

    @Override
    public String getColumnName(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return "Classification";
            case 1:
                return "Color";
            default:
                return null;
        }
    }

    @Override
    public Class<?> getColumnClass(int columnIndex) {
        switch (columnIndex) {
            case 0:
                return Object.class;
            case 1:
                return Color.class;
            default:
                return null;
        }
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        switch (columnIndex) {
            case 1:
                return true;
            default:
                return false;
        }
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        Entry<Object, PixelDescription> row = data.getClassificationColors().entrySet().toArray(new Entry[0])[rowIndex];
        if (columnIndex == 0) {
            return row.getKey();
        } else if (columnIndex == 1) {
            return row.getValue().color;
        }
        return null;
    }

    @SuppressWarnings({"rawtypes", "unchecked"})
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        Color newColor = (Color) aValue;
        Entry<Object, PixelDescription> row = data.getClassificationColors().entrySet().toArray(new Entry[0])[rowIndex];
        tab.setColorForClassification(row.getKey(), newColor);
        listeners.forEach(l -> l.tableChanged(new TableModelEvent(this, rowIndex, rowIndex, columnIndex, TableModelEvent.UPDATE)));
    }

    @Override
    public void addTableModelListener(TableModelListener l) {
        listeners.add(l);
    }

    @Override
    public void removeTableModelListener(TableModelListener l) {
        listeners.remove(l);
    }

}
