/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.timelapse.components;

import at.jku.mevss.anttracks.classification.*;
import at.jku.mevss.trace.gui.classification.component.configurationpane.ConfigurationPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane;
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane.ClassificationSelectionListener;
import at.jku.mevss.trace.gui.frame.main.tab.timelapse.TimelapseTab;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.AvailableClassifierInfo;
import at.jku.mevss.trace.gui.model.SelectedClassifierInfo;
import at.jku.mevss.trace.gui.utils.FXMLUtil;
import javafx.fxml.FXML;
import javafx.scene.layout.HBox;

/**
 * @author Christina Rammerstorfer & Markus Weninger
 */
public class TimelapseConfigurationPane extends ConfigurationPane {

    @FXML
    private HBox operationPane;

    private TimelapseTab tab;

    public TimelapseConfigurationPane() {
        FXMLUtil.load(this, TimelapseConfigurationPane.class);
    }

    public void init(TimelapseTab tab, AppInfo appInfo, AvailableClassifierInfo availableClassifierInfo, SelectedClassifierInfo selectedClassifierInfo) {
        super.init(appInfo, availableClassifierInfo, selectedClassifierInfo);
        this.tab = tab;

        switchMode();
    }

    public void apply() {
        switchMode();
        tab.getStatisticsInfo().setSelectedClassifier(new ClassifierChain(getClassifierSelectionPanel().getSelected()));
        tab.getStatisticsInfo().setSelectedFilters(getFilterSelectionPanel().getSelected());
        tab.startNewWorker();
    }

    public void cancel() {
        switchMode();
        getClassifierSelectionPanel().resetSelected(tab.getStatisticsInfo().getSelectedClassifiers().getList());
        getFilterSelectionPanel().resetSelected(tab.getStatisticsInfo().getSelectedFilters());
    }

    @Override
    public void switchMode() {
        super.switchMode();
        if (operationPane != null) {
            operationPane.setVisible(isMaxMode);
            operationPane.setManaged(isMaxMode);
        }
    }

    @Override
    protected ClassificationSelectionListener<Classifier<?>, ClassifierFactory> createClassifierListener() {
        return new ClassificationSelectionListener<Classifier<?>, ClassifierFactory>() {

            @Override
            public void selected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                // TODO Auto-generated method stub
            }

            @Override
            public void deselected(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                // TODO Auto-generated method stub
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Classifier<?>, ClassifierFactory> sender, Classifier<?> x) {
                // TODO Auto-generated method stub
            }
        };
    }

    @Override
    protected ClassificationSelectionListener<Filter, FilterFactory> createFilterListener() {
        return new ClassificationSelectionListener<Filter, FilterFactory>() {
            @Override
            public void selected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                // TODO Auto-generated method stub
            }

            @Override
            public void deselected(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                // TODO Auto-generated method stub
            }

            @Override
            public void propertiesChanged(ClassificationSelectionPane<Filter, FilterFactory> sender, Filter x) {
                // TODO Auto-generated method stub
            }
        };
    }

    @Override
    public void setLoadingMode(boolean isLoading) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void setAppropriateConfigView() {
        throw new UnsupportedOperationException();
    }

}
