/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.timelapse.model;

import at.jku.mevss.trace.gui.model.DetailedHeapInfo;

import java.util.LinkedList;
import java.util.Queue;

/**
 * @author Christina Rammerstorfer
 */
public class HeapList {

    private final Queue<DetailedHeapInfo> heaps;
    private long minStartAddress = Long.MAX_VALUE;

    public HeapList() {
        heaps = new LinkedList<DetailedHeapInfo>();
    }

    /**
     * Adds a new DetailsInfo to the HeapList and sets the new minStartAddress if necessary
     *
     * @param d The new DetailsInfo containing the Heap to be added
     */
    public void add(DetailedHeapInfo d) {
        heaps.offer(d);
        long addr = d.getFastHeapSupplier().get().stream().mapToLong(x -> d.getFastHeapSupplier().get().getAddress(x)).min().getAsLong();
        if (addr < minStartAddress) {
            minStartAddress = addr;
        }
    }

    public DetailedHeapInfo getFirst() {
        return heaps.poll();
    }

    public long getMinStartAddress() {
        return minStartAddress;
    }

}
