/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.timelapse.model;

import java.awt.image.BufferedImage;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * @author Christina Rammerstorfer
 */
public class TimelapseModel {
    private Map<Long, BufferedImage> images;
    private Long currentSelection;

    public TimelapseModel() {
        images = new HashMap<>();
    }

    public void addImage(long time, BufferedImage image) {
        images.put(time, image);
    }

    public Long[] getTimes() {
        Long[] times = images.keySet().toArray(new Long[0]);
        Arrays.sort(times);
        return times;
    }

    //	public void setCurrentSelection(int timeIndex){
    //		Long[] times = images.keySet().toArray(new Long[0]);
    //		Arrays.sort(times);
    //		currentSelection = times[timeIndex];
    //	}

    public void setCurrentSelection(long time) {
        currentSelection = time;
    }

    public long getCurrentSelection() {
        return currentSelection;
    }

    public BufferedImage getCurrentImage() {
        return images.get(currentSelection);
    }

    public long getNextValue(long curValue) {
        Long[] times = images.keySet().toArray(new Long[0]);
        Arrays.sort(times);
        for (int i = 0; i < times.length; i++) {
            if (times[i] == curValue) {
                return times[i + 1];
            }
        }
        return -1;
    }

    public boolean containsKey(long value) {
        return images.containsKey(value);
    }

    public long getClosestValue(long value) {
        Long[] times = images.keySet().toArray(new Long[0]);
        Arrays.sort(times);
        long minDiff = Long.MAX_VALUE;
        int minIndex = 0;
        for (int i = 0; i < times.length; i++) {
            long timeDiff = Math.abs(value - times[i]);
            if (timeDiff < minDiff) {
                minDiff = timeDiff;
                minIndex = i;
            }
        }
        return times[minIndex];
    }

}
