/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.frame.main.tab.timelapse.workers;

import at.jku.mevss.anttracks.heap.DetailedHeap;
import at.jku.mevss.anttracks.heap.GarbageCollectionCause;
import at.jku.mevss.anttracks.heap.GarbageCollectionType;
import at.jku.mevss.anttracks.heap.HeapListener;
import at.jku.mevss.anttracks.parser.heap.HeapTraceParser;
import at.jku.mevss.anttracks.parser.heap.ListenerHeapTraceParser;
import at.jku.mevss.trace.gui.frame.main.tab.timelapse.TimelapseTab;
import at.jku.mevss.trace.gui.frame.main.tab.timelapse.model.HeapList;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.DetailedHeapInfo;
import at.jku.mevss.trace.gui.model.TimelapseInfo;
import at.jku.mevss.trace.gui.model.TimelapseStatisticsInfo;
import at.jku.mevss.trace.gui.utils.AntTask;

import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;

import static at.jku.mevss.trace.gui.frame.main.GUI.CLIENT_INFO;

/**
 * @author Christina Rammerstorfer
 */
public class TimelapseTask extends AntTask<HeapList> {

    public static final DetailedHeapInfo END = new DetailedHeapInfo(null, -1);

    private final AppInfo appInfo;
    private TimelapseTab tab;
    private final TimelapseStatisticsInfo statisticsInfo;

    public TimelapseTask(AppInfo appInfo, TimelapseTab tab) {
        this.appInfo = appInfo;
        this.tab = tab;
        if (tab == null) {
            statisticsInfo = new TimelapseStatisticsInfo(new TimelapseInfo(appInfo));
        } else {
            statisticsInfo = tab.getStatisticsInfo();
        }
    }

    @Override
    protected HeapList backgroundWork() throws Exception {
        updateTitle("Timelapse: Parsing trace and generating all available heaps");
        if (tab == null) {
            tab = new TimelapseTab();
            tab.init(appInfo, statisticsInfo);
            tab.setDefaultClassification();
            CLIENT_INFO.getMainFrame().addTab(tab);
        }
        tab.setWorker(this);
        HeapTraceParser parser;
        HeapList heapList = new HeapList();
        parser = new ListenerHeapTraceParser(appInfo.getSymbols(), new TimelapseHeapListener(heapList));
        parser.addTraceParserListener((from, to, position) -> updateProgress(position - from, to - from));
        parser.parse(cancelProperty);
        return heapList;
    }

    private class TimelapseHeapListener implements HeapListener {

        private final HeapList heaps;

        public TimelapseHeapListener(HeapList heaps) {
            this.heaps = heaps;
        }

        @Override
        public void phaseChanging(DetailedHeap heap,
                                  GarbageCollectionType from,
                                  GarbageCollectionType to,
                                  GarbageCollectionCause cause,
                                  boolean concurrent,
                                  boolean failed,
                                  long time,
                                  long position,
                                  long length,
                                  boolean inSelectedTimeWindow) {
            put(heap.clone(), time);
        }

        @Override
        public void phaseChanged(DetailedHeap heap,
                                 GarbageCollectionType from,
                                 GarbageCollectionType to,
                                 GarbageCollectionCause cause,
                                 boolean concurrent,
                                 boolean failed,
                                 long time,
                                 long position,
                                 long length,
                                 boolean inSelectedTimeWindow) {
            put(heap.clone(), time);
        }

        private void put(DetailedHeap heap, long time) {
            if (heap.getObjectCount() > 0) {
                DetailedHeapInfo info = new DetailedHeapInfo(appInfo, time);
                info.setHeap(heap);
                heaps.add(info);
            }
        }

        @Override
        public void close(DetailedHeap heap) {}

    }

    @Override
    protected void suceeded() {
        try {
            HeapList heapList = get();
            tab.setWorker(null);
            tab.startNewPaintWorker(heapList);
        } catch (InterruptedException | ExecutionException | CancellationException e) {
            e.printStackTrace();
        }
    }
}
