/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.model;

import at.jku.mevss.trace.gui.frame.main.MainFrame;
import at.jku.mevss.trace.gui.utils.OperationManager;
import com.sun.net.httpserver.HttpServer;
import io.micrometer.core.instrument.Clock;
import io.micrometer.core.instrument.MeterRegistry;
import io.micrometer.core.instrument.composite.CompositeMeterRegistry;
import io.micrometer.jmx.JmxConfig;
import io.micrometer.jmx.JmxMeterRegistry;
import io.micrometer.prometheus.PrometheusConfig;
import io.micrometer.prometheus.PrometheusMeterRegistry;
import javafx.application.Application;
import javafx.stage.Stage;

import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;

public class ClientInfo {

    public static String traceDirectory = System.getProperty("user.home");
    public static String featureDirectory = System.getProperty("user.home");

    public OperationManager operationManager;

    public MeterRegistry meterRegistry = new CompositeMeterRegistry();
    private JmxMeterRegistry jmxMeterRegistry = new JmxMeterRegistry(JmxConfig.DEFAULT, Clock.SYSTEM);
    private PrometheusMeterRegistry prometheusMeterRegistry = new PrometheusMeterRegistry(PrometheusConfig.DEFAULT);

    public final Application application;
    private final MainFrame mainframe;
    private final Stage stage;

    private boolean isExtendedMode = false;
    public boolean consistencyCheck;
    public String logSettingsFile;
    public String printStatisticsPath;
    public boolean assertionsEnabled;

    public ClientInfo(Application app, MainFrame frame, Stage stage) {
        application = app;
        mainframe = frame;
        this.stage = stage;

        ((CompositeMeterRegistry) meterRegistry).add(jmxMeterRegistry);
        ((CompositeMeterRegistry) meterRegistry).add(prometheusMeterRegistry);

        try {
            HttpServer server = HttpServer.create(new InetSocketAddress(7001), 0);
            server.createContext("/metrics", httpExchange -> {
                String response = prometheusMeterRegistry.scrape();
                httpExchange.sendResponseHeaders(200, response.length());
                OutputStream os = httpExchange.getResponseBody();
                os.write(response.getBytes());
                os.close();
            });

            new Thread(server::start).run();
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    public MainFrame getMainFrame() {
        return mainframe;
    }

    public Stage getStage() {
        return stage;
    }

    public boolean isExtendedChartVisibility() {
        return isExtendedMode;
    }

    public void setExtendedChartVisibility(boolean extendedMode) {
        isExtendedMode = extendedMode;
    }

    public AppInfo getCurrentAppInfo() {
        return mainframe.getAppInfo();
    }
}
