/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import at.jku.mevss.anttracks.util.ThreadUtil;
import at.jku.mevss.trace.gui.dialog.openfile.OpenFileDialog;
import at.jku.mevss.trace.gui.frame.main.GUI;
import at.jku.mevss.trace.gui.frame.main.tab.application.ApplicationLoadingTask;
import at.jku.mevss.trace.gui.frame.main.tab.application.ApplicationTab;
import at.jku.mevss.trace.gui.model.AppInfo;
import at.jku.mevss.trace.gui.model.ClientInfo;
import javafx.scene.control.Alert;
import javafx.scene.control.Alert.AlertType;
import javafx.scene.control.ButtonType;
import javafx.scene.layout.Region;
import javafx.stage.FileChooser;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class AppLoader {

    private final static Logger LOGGER = Logger.getLogger(AppLoader.class.getSimpleName());

    public static void preprocessDirectory() {
        // TODO TraceFileFilter filter = new TraceFileFilter();
        FileChooser fileChooser = new FileChooser();
        fileChooser.setInitialDirectory(new File(ClientInfo.traceDirectory));
        File traceFile = fileChooser.showOpenDialog(GUI.CLIENT_INFO.getStage());
        if (traceFile != null) {
            Alert terminateAntTracksAlert = new Alert(AlertType.CONFIRMATION,
                                                      "You are about to preprocess '" + traceFile.getAbsolutePath() + "' in the " + "background." + "\n" + "It is " +
                                                              "recommended " +
                                                              "that you terminate the application to increase the " + "available heap size." + "\n" + "Do you want to " +
                                                              "terminate now?",
                                                      ButtonType.YES,
                                                      ButtonType.NO,
                                                      ButtonType.CANCEL);
            terminateAntTracksAlert.setTitle("Preprocessing");
            terminateAntTracksAlert.getDialogPane().setMinHeight(Region.USE_PREF_SIZE);
            WindowUtil.centerInMainFrame(terminateAntTracksAlert);
            terminateAntTracksAlert.showAndWait().ifPresent(result -> {
                if (result != ButtonType.CANCEL) {
                    try {
                        Runtime.getRuntime()
                               .exec(new String[]{OsScript.getScriptExecutable(),
                                                  "runAntTracksPreprocess." + OsScript.getScriptExtension(),
                                                  traceFile.getAbsolutePath()});
                    } catch (IOException e) {
                        LOGGER.log(Level.SEVERE, "error occured", e);
                        Errors.display(e);
                    }
                    Alert preprocessStartedAlert = new Alert(AlertType.INFORMATION, "Preprocessing started in an external application.");
                    preprocessStartedAlert.setTitle("Preprocessing");
                    preprocessStartedAlert.getDialogPane().setMinHeight(Region.USE_PREF_SIZE);
                    WindowUtil.centerInMainFrame(preprocessStartedAlert);
                    preprocessStartedAlert.showAndWait();
                }

                if (result == ButtonType.YES) {
                    System.exit(0);
                } else {

                }
            });
        }
    }

    public static void load() {
        OpenFileDialog ofd = new OpenFileDialog();
        WindowUtil.centerInMainFrame(ofd);
        ofd.showAndWait().ifPresent(result -> {
            if (result) {
                AppInfo appInfo = new AppInfo(ofd.getApplicationName(), ofd.getTraceFile(), ofd.getFeaturesFile());
                parse(appInfo);
            }
        });
    }

    public static void reload() {
        // Get current app info
        AppInfo appInfo = GUI.CLIENT_INFO.getCurrentAppInfo();
        // User enters new definition
        OpenFileDialog ofd = new OpenFileDialog(appInfo);
        WindowUtil.centerInMainFrame(ofd);
        ofd.showAndWait().ifPresent(result -> {
            if (result) {
                // Update app info with newly entered values
                appInfo.setAppName(ofd.getApplicationName());
                appInfo.selectFeaturesAndTraceFile(ofd.getTraceFile(), ofd.getFeaturesFile());
            }
        });
    }

    public static void parse(AppInfo appInfo) {
        parse(appInfo, null);
    }

    public static void parse(AppInfo appInfo, ApplicationTab tab) {
        try {
            /*
             * SymbolsParser symbolsParser = new SymbolsParser(appInfo.getSymbolsFile(), appInfo.getClassDefinitionsFile(),
             * appInfo.getSelectedFeaturesFile(), Symbols.CALLCONTEXT_STATIC); appInfo.setSymbols(symbolsParser.parse());
             */

            // If view is not set, create new one
            if (tab == null) {
                tab = new ApplicationTab();
                tab.init(appInfo);
                GUI.CLIENT_INFO.getMainFrame().addTab(tab);
            }

            ThreadUtil.startTask(new ApplicationLoadingTask(tab));
        } catch (Exception e) {
            Logger.getLogger(AppLoader.class.getSimpleName()).log(Level.SEVERE, "error occured", e);
            Errors.display(e);
        }
    }
}
