/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import javax.swing.*;
import java.awt.*;

public class CollapsibleButton extends JButton {

    private static final long serialVersionUID = 1L;
    private final Polygon expandedArrow;
    private final Polygon collapsedArrow;
    private boolean expanded;

    public CollapsibleButton(int orientation, int size) {
        if (orientation == CollapsiblePanel.NORTH) {
            this.expandedArrow = arrowDown(size);
            this.collapsedArrow = arrowUp(size);
        } else if (orientation == CollapsiblePanel.SOUTH) {
            this.expandedArrow = arrowUp(size);
            this.collapsedArrow = arrowDown(size);
        } else if (orientation == CollapsiblePanel.EAST) {
            this.expandedArrow = arrowRight(size);
            this.collapsedArrow = arrowLeft(size);
        } else {
            this.expandedArrow = arrowLeft(size);
            this.collapsedArrow = arrowRight(size);
        }
        this.expanded = true;
    }

    @Override
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.setColor(Color.GRAY);
        if (expanded) {
            g.translate(getSize().width / 2, getSize().height);
            g.fillPolygon(expandedArrow);
            this.setCursor(Cursor.getPredefinedCursor(Cursor.N_RESIZE_CURSOR));
        } else {
            g.translate(getSize().width / 2, 0);
            g.fillPolygon(collapsedArrow);
            this.setCursor(Cursor.getPredefinedCursor(Cursor.S_RESIZE_CURSOR));
        }
        g.dispose();
    }

    public void updateExpanded(boolean expanded) {
        this.expanded = expanded;
    }

    private Polygon arrowDown(int size) {
        int[] xPoints = {0, size, -size};
        int[] yPoints = {0, -size, -size};
        return new Polygon(xPoints, yPoints, 3);
    }

    private Polygon arrowUp(int size) {
        int[] xPoints = {0, -size, size};
        int[] yPoints = {0, size, size};
        return new Polygon(xPoints, yPoints, 3);
    }

    private Polygon arrowLeft(int size) {
        int[] xPoints = {0, size, size};
        int[] yPoints = {0, size, -size};
        return new Polygon(xPoints, yPoints, 3);
    }

    private Polygon arrowRight(int size) {
        int[] xPoints = {0, -size, -size};
        int[] yPoints = {0, -size, size};
        return new Polygon(xPoints, yPoints, 3);
    }

}
