/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import at.jku.mevss.anttracks.util.ImagePack;
import javafx.scene.image.ImageView;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.logging.Logger;

public class Consts {
    public static final String HEADERS_META_FILE = at.jku.mevss.anttracks.util.Consts.HEADERS_META_FILE;
    public static final String HEAP_INDEX_META_FILE = at.jku.mevss.anttracks.util.Consts.HEAP_INDEX_META_FILE;
    public static final String STATISTICS_META_FILE = at.jku.mevss.anttracks.util.Consts.STATISTICS_META_FILE;
    ;
    public static final String ANT_META_DIRECTORY = at.jku.mevss.anttracks.util.Consts.ANT_META_DIRECTORY;
    public static final String FEATURES_META_FILE = at.jku.mevss.anttracks.util.Consts.FEATURES_META_FILE;
    public static final String CONFIG_DIRECTORY = System.getProperty("user.home") + File.separator + ".ant_tracks" + File.separator;
    public static final String SETTINGS_FILE = CONFIG_DIRECTORY + "tool_settings";
    public static final String CLASSIFIERS_DIRECTORY = CONFIG_DIRECTORY + "custom_classifiers";
    public static final String FILTERS_DIRECTORY = CONFIG_DIRECTORY + "custom_filters";

    public static final BufferedImage ANT_ICON_IMAGE;
    public static final BufferedImage ANT_IMAGE;
    public static final BufferedImage EXCLAMATION_MARK_IMAGE;
    public static final BufferedImage DELETE_IMAGE;
    public static final BufferedImage SETTINGS_IMAGE;
    public static final BufferedImage APP_IMAGE;
    public static BufferedImage DIFFING_IMAGE = null; // YET MISSING!
    public static final BufferedImage STATISTICS_IMAGE;
    public static final BufferedImage DETAILS_IMAGE;
    public static BufferedImage VISUALIZE_IMAGE = null; // YET MISSING!
    public static final BufferedImage DISABLE_MAGNIFIER_IMAGE;
    public static final BufferedImage BRUSH_IMAGE;
    public static final BufferedImage ARROW_IMAGE;
    public static final BufferedImage ZOOM_IN_IMAGE;
    public static final BufferedImage ZOOM_OUT_IMAGE;
    public static final BufferedImage PLAY_IMAGE;
    public static final BufferedImage PAUSE_IMAGE;
    public static final BufferedImage TYPE_IMAGE;
    public static final BufferedImage EXCHANGE_IMAGE;

    public static final ImageIcon LOADING_ICON;
    public static final ImageIcon EXCLAMATION_MARK_ICON;
    public static final ImageIcon DELETE_ICON;
    public static final ImageIcon APP_ICON;
    public static ImageIcon DIFFING_ICON = null; // YET MISSING!
    public static final ImageIcon STATISTICS_ICON;
    public static final ImageIcon DETAILS_ICON;
    public static ImageIcon VISUALIZE_ICON = null; // YET MISSING!
    public static final ImageIcon DISABLE_MAGNIFIER_ICON;
    public static final ImageIcon BRUSH_ICON;
    public static final ImageIcon ARROW_ICON;
    public static final ImageIcon ZOOM_IN_ICON;
    public static final ImageIcon ZOOM_OUT_ICON;
    public static final ImageIcon PLAY_ICON;
    public static final ImageIcon PAUSE_ICON;
    public static final ImageIcon TYPE_ICON;
    public static final ImageIcon EXCHANGE_ICON;

    public static final ImagePack EXCHANGE_PACK;

    public static Color[] BASE_COLOR_VALUES;

    // http://www.mulinblog.com/a-color-palette-optimized-for-data-visualization/
    /*
     * 4D4D4D (gray) 5DA5DA (blue) -> darker #5394c4 -> darker #4a84ae -> 1 FAA43A (orange) 60BD68 (green) -> 3 F17CB0 (pink) B2912F (brown)
     * -> 5 B276B2 (purple) DECF3F (yellow) -> 4 F15854 (red) -> 2
     */
    static {
        float alpha = 0.8f;
        BASE_COLOR_VALUES = new Color[]{new Color(0x4A / 255.0f, 0x84 / 255.0f, 0xAE / 255.0f, alpha),
                                        new Color(0xF1 / 255.0f, 0x58 / 255.0f, 0x54 / 255.0f, alpha),
                                        new Color(0x60 / 255.0f, 0xBD / 255.0f, 0x68 / 255.0f, alpha),
                                        new Color(0xDE / 255.0f, 0xCF / 255.0f, 0x3F / 255.0f, alpha),
                                        new Color(0xB2 / 255.0f, 0x91 / 255.0f, 0x2F / 255.0f, alpha)};
    }

    public static final int ICON_SIZE = 16;

    @SuppressWarnings("unused")
    private static final Logger LOGGER = Logger.getLogger(Consts.class.getSimpleName());

    static {
        ANT_ICON_IMAGE = ImageUtil.loadResourceImage("Ant Icon", "ant_icon.jpg");
        ANT_IMAGE = ImageUtil.loadResourceImage("Ant", "ant_tracks.jpg");
        EXCLAMATION_MARK_IMAGE = ImageUtil.loadResourceImage("Exclamation mark", "exclamationMark.png");
        DELETE_IMAGE = ImageUtil.loadResourceImage("Clear", "delete.png");
        SETTINGS_IMAGE = ImageUtil.loadResourceImage("Configure", "wrench.png");
        APP_IMAGE = ImageUtil.loadResourceImage("App", "app.png");
        STATISTICS_IMAGE = ImageUtil.loadResourceImage("Statistics", "statistics.png");
        DETAILS_IMAGE = ImageUtil.loadResourceImage("Details", "details.png");
        DISABLE_MAGNIFIER_IMAGE = ImageUtil.loadResourceImage("Disable Magnifier", "disableMagnifier.png");
        BRUSH_IMAGE = ImageUtil.loadResourceImage("Brush", "brush.png");
        ARROW_IMAGE = ImageUtil.loadResourceImage("Arrow", "arrow.png");
        ZOOM_IN_IMAGE = ImageUtil.loadResourceImage("Zoom In", "zoomIn.png");
        ZOOM_OUT_IMAGE = ImageUtil.loadResourceImage("Zoom Out", "zoomOut.png");
        PLAY_IMAGE = ImageUtil.loadResourceImage("Play", "play.png");
        PAUSE_IMAGE = ImageUtil.loadResourceImage("Pause", "pause.png");
        TYPE_IMAGE = ImageUtil.loadResourceImage("Type", "type.png");
        EXCHANGE_IMAGE = ImageUtil.loadResourceImage("Exchange Arrows", "exchangearrows.png");

        LOADING_ICON = new ImageIcon("./resources/img/loader.gif");
        EXCLAMATION_MARK_ICON = ImageUtil.asIcon(EXCLAMATION_MARK_IMAGE);
        DELETE_ICON = ImageUtil.asIcon(DELETE_IMAGE);
        APP_ICON = ImageUtil.asIcon(APP_IMAGE);
        STATISTICS_ICON = ImageUtil.asIcon(STATISTICS_IMAGE);
        DETAILS_ICON = ImageUtil.asIcon(DETAILS_IMAGE);
        DISABLE_MAGNIFIER_ICON = ImageUtil.asIcon(DISABLE_MAGNIFIER_IMAGE);
        BRUSH_ICON = ImageUtil.asIcon(BRUSH_IMAGE);
        ARROW_ICON = ImageUtil.asIcon(ARROW_IMAGE);
        ZOOM_IN_ICON = ImageUtil.asIcon(ZOOM_IN_IMAGE);
        ZOOM_OUT_ICON = ImageUtil.asIcon(ZOOM_OUT_IMAGE);
        PLAY_ICON = ImageUtil.asIcon(PLAY_IMAGE);
        PAUSE_ICON = ImageUtil.asIcon(PAUSE_IMAGE);
        TYPE_ICON = ImageUtil.asIcon(TYPE_IMAGE);
        EXCHANGE_ICON = ImageUtil.asIcon(EXCHANGE_IMAGE);

        EXCHANGE_PACK = new ImagePack() {
            @Override
            public ImageView getAsNewNode() {
                return ImageUtil.getIconNode(EXCHANGE_ICON);
            }

            @Override
            public BufferedImage getImage() {
                return EXCHANGE_IMAGE;
            }

            @Override
            public ImageIcon getIcon() {
                return EXCHANGE_ICON;
            }
        };

        // try {
        // img = ImageIO.read(new File("./resources/img/diff.png"));
        // DIFFING_ICON = new ImageIcon(img.getScaledInstance(ICON_SIZE,
        // ICON_SIZE, Image.SCALE_SMOOTH));
        // } catch (Exception e) {
        // LOGGER.warning("Could not load \"Diffing\" icon\n" + e);
        // }
        // try {
        // img = ImageIO.read(new File("./resources/img/visualize.png"));
        // VISUALIZE_ICON = new ImageIcon(img.getScaledInstance(ICON_SIZE,
        // ICON_SIZE, Image.SCALE_SMOOTH));
        // } catch (Exception e) {
        // LOGGER.warning("Could not load \"Visualize\" icon\n" + e);
        // }
    }
}
