/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import at.jku.mevss.trace.gui.frame.main.component.statuspane.component.operationpane.OperationPane;
import javafx.beans.property.DoubleProperty;
import javafx.beans.property.SimpleDoubleProperty;
import javafx.beans.property.SimpleStringProperty;
import javafx.beans.property.StringProperty;
import javafx.concurrent.Task;

import java.util.ArrayList;
import java.util.List;

public class Operation {

    private SimpleStringProperty title = new SimpleStringProperty();
    private SimpleStringProperty message = new SimpleStringProperty();
    private SimpleStringProperty remainingTimeText = new SimpleStringProperty();
    private SimpleDoubleProperty progress = new SimpleDoubleProperty();

    private Task<?> task;
    private OperationPane operationPane;
    private List<Runnable> onClosedListeners;
    private ProgressCalc progressCalculator;

    // Interface for old workers that do not implement AntTask
    public Operation(String title) {
        this.title.set(title);
        this.message.set("");
        this.progressCalculator = new ProgressCalc();
        this.remainingTimeText.bind(progressCalculator.remainingTimeTextProperty);
        this.progress.bind(progressCalculator.progressProperty);

        setup();
    }

    // Interface for old workers that do not implement AntTask
    public void progress(double progress) {
        progressCalculator.update(progress);
    }

    /**
     * Creates a new operation based on a task's title, message and progress.
     * May only be called from within the JavaFX thread.
     *
     * @param task The task to bind to
     */
    public Operation(Task<?> task) {
        this.task = task;
        this.title.bind(task.titleProperty());
        this.message.bind(task.messageProperty());
        this.progressCalculator = new ProgressCalc(task.progressProperty());
        this.remainingTimeText.bind(progressCalculator.remainingTimeTextProperty);
        this.progress.bind(progressCalculator.progressProperty);

        setup();
    }

    private void setup() {
        onClosedListeners = new ArrayList<>();
        operationPane = new OperationPane(this);
        operationPane.initialize();
    }

    public Task<?> getTask() {
        return task;
    }

    public OperationPane getOperationPane() {
        return operationPane;
    }

    public void addClosedListener(Runnable listener) {
        onClosedListeners.add(listener);
    }

    public void removeClosedListener(Runnable listener) {
        onClosedListeners.remove(listener);
    }

    /**
     * Notifies all listeners, then clears the listener list (since close should only be called once)
     */
    public void close() {
        progressCalculator.close();
        onClosedListeners.forEach(Runnable::run);
        onClosedListeners.clear();
    }

    public void cancel() {
        task.cancel();
    }

    public StringProperty titleProperty() {
        return title;
    }

    public StringProperty messageProperty() {
        return message;
    }

    public StringProperty remainingTimeTextProperty() {
        return remainingTimeText;
    }

    public DoubleProperty progressProperty() {
        return progress;
    }
}
