/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import javafx.application.Platform;
import javafx.beans.value.ChangeListener;
import javafx.collections.ObservableList;
import javafx.concurrent.Task;
import javafx.concurrent.Worker.State;
import javafx.scene.Node;

import java.util.ArrayList;
import java.util.List;

public class OperationManager {

    private final ObservableList<Node> operationPaneList;
    private final List<Operation> operations = new ArrayList<>();

    public OperationManager(ObservableList<Node> operationPaneList) {
        this.operationPaneList = operationPaneList;
    }

    public Operation addNewOperation(String title) {
        Operation o = new Operation(title);
        addOperation(o);
        return o;
    }

    public Operation addNewOperation(Task<?> task) {
        Operation o = new Operation(task);
        addOperation(o);
        return o;
    }

    public void addOperation(Operation operation) {
        // End on manual operation closing
        operation.addClosedListener(() -> {
            closeOperation(operation);
        });

        Platform.runLater(() -> {
            // End on task end
            operation.getTask().stateProperty().addListener((ChangeListener<State>) (observable, oldValue, newValue) -> {
                switch (newValue) {
                    case CANCELLED:
                        closeOperation(operation);
                        break;
                    case FAILED:
                        closeOperation(operation);
                        break;
                    case READY:
                        break;
                    case RUNNING:
                        break;
                    case SCHEDULED:
                        break;
                    case SUCCEEDED:
                        closeOperation(operation);
                        break;
                    default:
                        break;
                }
            });

            // Add panel to view
            operationPaneList.add(operation.getOperationPane());
        });
    }

    private void closeOperation(Operation operation) {
        // Remove panel from view
        Platform.runLater(() -> operationPaneList.remove(operation.getOperationPane()));
    }

    public void cancelAll() {
        operations.forEach(operation -> operation.getTask().cancel());
    }
}
