/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.utils;

import at.jku.mevss.trace.gui.frame.main.GUI;
import javafx.beans.value.ChangeListener;
import javafx.beans.value.ObservableDoubleValue;
import javafx.beans.value.ObservableValue;
import javafx.geometry.Bounds;
import javafx.scene.Node;
import javafx.scene.control.Dialog;

import java.awt.*;
import java.util.logging.Logger;

public class WindowUtil {

    private final static Logger LOGGER = Logger.getLogger(WindowUtil.class.getSimpleName());

    public static void centerOnDefaultScreen(Window frame) {
        GraphicsDevice myScreen = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
        centerInBounds(frame, myScreen.getDefaultConfiguration().getBounds());
    }

    public static void centerInBounds(Window frame, Rectangle bounds) {
        if (frame == null || bounds == null) {
            return;
        }
        final int x = bounds.x + (bounds.width - frame.getWidth()) / 2;
        final int y = bounds.y + (bounds.height - frame.getHeight()) / 2;
        frame.setLocation(x, y);
        LOGGER.fine("Frame opened on screen (" + bounds.x + ", " + bounds.y + ", " + bounds.width + ", " + bounds.height + ") at (" + x + ", " + y + ")\n" + frame);
    }

    public static void centerInBounds(Dialog<?> dialog, Bounds bounds) {
        if (dialog == null || bounds == null) {
            throw new IllegalArgumentException();
        }

        if (Double.isNaN(dialog.getHeight()) || Double.isNaN(dialog.getWidth())) {
            // cant calculate center position now because properties not calculated yet!
            // trigger calculation once values are present
            ObservableDoubleValue width = dialog.widthProperty();
            ChangeListener<Number> widthListener = new ChangeListener<Number>() {
                @Override
                public void changed(ObservableValue<? extends Number> widthProperty, Number oldWidth, Number newWidth) {
                    if (newWidth.doubleValue() != Double.NaN) {
                        dialog.setX(bounds.getMinX() + (bounds.getWidth() - newWidth.doubleValue()) / 2);
                        width.removeListener(this);     // only trigger ONCE!
                    }
                }
            };
            width.addListener(widthListener);

            ObservableDoubleValue height = dialog.heightProperty();
            ChangeListener<Number> heightListener = new ChangeListener<Number>() {
                @Override
                public void changed(ObservableValue<? extends Number> heightProperty, Number oldHeight, Number newHeight) {
                    if (newHeight.doubleValue() != Double.NaN) {
                        dialog.setY(bounds.getMinY() + (bounds.getHeight() - newHeight.doubleValue()) / 2);
                        height.removeListener(this);     // only trigger ONCE!
                    }
                }
            };
            height.addListener(heightListener);

            LOGGER.fine("Dialog opened on screen (" + bounds.getMinX() + ", " + bounds.getMinY() + ", " + bounds.getWidth() + ", " + bounds.getHeight() + ") - " +
                                "coordinates to be " +
                                "calculated!");
        } else {
            final double x = bounds.getMinX() + (bounds.getWidth() - dialog.getWidth()) / 2;
            final double y = bounds.getMinY() + (bounds.getHeight() - dialog.getHeight()) / 2;

            dialog.setX(x);
            dialog.setY(y);
            dialog.initOwner(GUI.CLIENT_INFO.getStage());

            LOGGER.fine("Dialog opened on screen (" + bounds.getMinX() + ", " + bounds.getMinY() + ", " + bounds.getWidth() + ", " + bounds.getHeight() + ") at (" + x
                                + ", " + y
                                + ")\n" + dialog);
        }
    }

    public static Dimension getSizeRelativeToWindowScreen(int percent) {
        GraphicsDevice myScreen = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
        return new Dimension((int) (myScreen.getDefaultConfiguration().getBounds().getWidth() * percent / 100.0),
                             (int) (myScreen.getDefaultConfiguration().getBounds().getHeight() * percent / 100.0));
    }

    public static void centerInMainFrame(Dialog<?> dialog) {
        Node root = GUI.CLIENT_INFO.getMainFrame().getRoot();
        WindowUtil.centerInBounds(dialog, root.localToScreen(root.getBoundsInLocal()));
    }
}

