package at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.configurationpane

import at.jku.mevss.anttracks.classification.ClassifierChain
import at.jku.mevss.anttracks.classification.ConditionalClassifierList
import at.jku.mevss.anttracks.classification.Filter
import at.jku.mevss.anttracks.classification.trees.ClassificationTree
import at.jku.mevss.anttracks.classification.trees.ListClassificationTree
import at.jku.mevss.anttracks.heap.ObjectStream
import at.jku.mevss.anttracks.util.Counter
import at.jku.mevss.trace.gui.classification.classifier.AddressClassifier
import at.jku.mevss.trace.gui.frame.main.GUI.CLIENT_INFO
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.DataStructureDevelopmentOverviewTab
import at.jku.mevss.trace.gui.model.DataStructureDevelopmentInfo
import at.jku.mevss.trace.gui.utils.AntTask
import javafx.concurrent.Task
import java.util.*
import java.util.concurrent.ExecutionException

class DataStructureDevelopmentClassificationTask(private val info: DataStructureDevelopmentInfo,
                                                 private val tab: DataStructureDevelopmentOverviewTab?) : AntTask<ListClassificationTree>() {
    override fun backgroundWork(): ListClassificationTree? {
        updateTitle(String.format("Classifying %d data structure head objects",
                                  info.developments.stream().filter { dsd -> dsd.startIndex >= 0 && dsd.endIndex >= 0 }.count()))
        val c = Counter()

        val filters: Array<Filter> = info.selectedClassifierInfo.selectedFilters.toTypedArray()
        val classifiers: ClassifierChain = info.selectedClassifierInfo.selectedClassifiers
        val conditionalClassifiers: ConditionalClassifierList = info.selectedClassifierInfo.selectedConditionalClassifier

        val startFilters: Array<Filter> = Arrays.copyOf(filters, filters.size + 1)
        startFilters[startFilters.size - 1] = object : Filter() {
            // There is only an entry in the developments list if the object already lived at the beginning of the selected time frame.
            // End index is >= 0 if the object survived the time frame
            override fun classify(): Boolean = info.developments.find { dsd -> dsd.startIndex == index() }?.endIndex ?: -1 >= 0
        }

        val endFilters: Array<Filter> = Arrays.copyOf(filters, filters.size + 1)
        endFilters[endFilters.size - 1] = object : Filter() {
            override fun classify(): Boolean = info.developments.any { dsd -> dsd.endIndex == index() }
        }

        val replaceStartWithEndAddressClassifier: AddressClassifier = object : AddressClassifier() {
            override fun address(): Long = info.developments.stream().filter { x -> x.startIndex == index() }.findAny().get().endAddress
        }

        val replaceStartWithEndAddressClassifierChain = ClassifierChain(classifiers.list.map {
            if (it is AddressClassifier) {
                replaceStartWithEndAddressClassifier.additionalClassifier = it.additionalClassifier
                replaceStartWithEndAddressClassifier.prepend = it.prepend
                replaceStartWithEndAddressClassifier
            } else {
                it
            }
        })

        val listener: ObjectStream.IterationListener = ObjectStream.IterationListener {
            c.add(it)
            updateProgress(c.get(), info.fastHeapSupplier.get()!!.objectCount.toLong())
        }

        info.startClassificationTree = info.getStartFastHeapSupplier().get()!!.groupListParallel(startFilters,
                                                                                                 replaceStartWithEndAddressClassifierChain,
                                                                                                 conditionalClassifiers,
                                                                                                 listener,
                                                                                                 cancelProperty)

        // Run closure initialization sequentially
        var closureInitTask: Task<*> = info.startClassificationTree!!.initClosureTask()
        CLIENT_INFO.operationManager.addNewOperation(closureInitTask)
        tab!!.tasks.add(closureInitTask)
        closureInitTask.run()

        info.endClassificationTree = info.fastHeapSupplier.get()!!.groupListParallel(endFilters,
                                                                                     classifiers,
                                                                                     conditionalClassifiers,
                                                                                     listener,
                                                                                     cancelProperty)

        // Run closure initialization sequentially
        closureInitTask = info.endClassificationTree!!.initClosureTask()
        CLIENT_INFO.operationManager.addNewOperation(closureInitTask)
        tab.tasks.add(closureInitTask)
        closureInitTask.run()

        info.classificationTreeDifference = info.endClassificationTree!!.root.subtractRecursive(info.startClassificationTree!!.root)
        return info.endClassificationTree
    }

    override fun suceeded() {
        try {
            val tree: ClassificationTree = get()
            if (tab != null) {
                tab.updateClassificationResult(tree)
            }
        } catch (e: InterruptedException) {
            e.printStackTrace()
        } catch (e: ExecutionException) {
            e.printStackTrace()
        }
    }
}
