package at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.configurationpane

import at.jku.mevss.anttracks.classification.*
import at.jku.mevss.anttracks.util.ThreadUtil
import at.jku.mevss.trace.gui.classification.classifier.AddressClassifier
import at.jku.mevss.trace.gui.classification.component.configurationpane.ConfigurationPane
import at.jku.mevss.trace.gui.classification.component.selectionpane.ClassificationSelectionPane
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.DataStructureDevelopmentOverviewTab
import at.jku.mevss.trace.gui.model.DataStructureDevelopmentInfo
import at.jku.mevss.trace.gui.utils.FXMLUtil
import javafx.fxml.FXML
import javafx.scene.control.Button

class DataStructureDevelopmentConfigurationPane : ConfigurationPane() {

    @FXML
    private lateinit var applyButton: Button
    @FXML
    private lateinit var cancelButton: Button

    private var dataStructureDevelopmentOverviewTab: DataStructureDevelopmentOverviewTab? = null
    private var dataStructureDevelopmentInfo: DataStructureDevelopmentInfo? = null

    init {
        FXMLUtil.load(this, DataStructureDevelopmentConfigurationPane::class.java)
    }

    fun init(dataStructureDevelopmentOverviewTab: DataStructureDevelopmentOverviewTab,
             dataStructureDevelopmentInfo: DataStructureDevelopmentInfo) {
        super.init(dataStructureDevelopmentInfo.appInfo, dataStructureDevelopmentInfo, dataStructureDevelopmentInfo.selectedClassifierInfo)
        this.dataStructureDevelopmentOverviewTab = dataStructureDevelopmentOverviewTab
        this.dataStructureDevelopmentInfo = dataStructureDevelopmentInfo

        // default classifier is address
        val addressClassifier = dataStructureDevelopmentInfo.getClassifier("Address") as AddressClassifier
        addressClassifier.additionalClassifier = dataStructureDevelopmentInfo.getClassifier("Type")
        classifierSelectionPane.addSelectedButNotShown(addressClassifier)

        // permanent hidden filter is only data structure heads
        val onlyDataStructureHeadsFilter = dataStructureDevelopmentInfo.availableFilter.get("Only data structure heads")
        filterSelectionPane.addSelectedButNotShown(onlyDataStructureHeadsFilter)

        dataStructureSwitch.isSelected = true
        dataStructureSwitch.isDisable = true

        // button behaviour
        cancelButton.setOnAction {
            classifierSelectionPane.resetSelected(dataStructureDevelopmentInfo.selectedClassifierInfo.selectedClassifiers.list)
            filterSelectionPane.resetSelected(dataStructureDevelopmentInfo.selectedClassifierInfo.selectedFilters)
            switchMode()
        }
        applyButton.setOnAction {
            classify()
            switchMode()
        }

        // start in minimized mode
        switchMode()
    }

    fun classify() {
        dataStructureDevelopmentInfo!!.selectedClassifierInfo.setSelectedClassifier(ClassifierChain(classifierSelectionPane.selected))
        dataStructureDevelopmentInfo!!.selectedClassifierInfo.selectedFilters = filterSelectionPane.selected

        // perform classification
        val groupingTask = DataStructureDevelopmentClassificationTask(dataStructureDevelopmentInfo!!, dataStructureDevelopmentOverviewTab)
        dataStructureDevelopmentOverviewTab!!.tasks.add(groupingTask)
        ThreadUtil.startTask(groupingTask)
    }

    override fun createClassifierListener(): ClassificationSelectionPane.ClassificationSelectionListener<Classifier<*>, ClassifierFactory> {
        return object : ClassificationSelectionPane.ClassificationSelectionListener<Classifier<*>, ClassifierFactory> {
            override fun selected(sender: ClassificationSelectionPane<Classifier<*>, ClassifierFactory>, x: Classifier<*>) {
                applyButton.isDisable = false
            }

            override fun deselected(sender: ClassificationSelectionPane<Classifier<*>, ClassifierFactory>, x: Classifier<*>) {
                applyButton.isDisable = sender.selected.isEmpty()
            }

            override fun propertiesChanged(sender: ClassificationSelectionPane<Classifier<*>, ClassifierFactory>, x: Classifier<*>) {
                // Nothing to do if property changed
            }
        }
    }

    override fun createFilterListener(): ClassificationSelectionPane.ClassificationSelectionListener<Filter, FilterFactory> {
        return object : ClassificationSelectionPane.ClassificationSelectionListener<Filter, FilterFactory> {
            override fun selected(sender: ClassificationSelectionPane<Filter, FilterFactory>, x: Filter) {}

            override fun deselected(sender: ClassificationSelectionPane<Filter, FilterFactory>, x: Filter) {}

            override fun propertiesChanged(sender: ClassificationSelectionPane<Filter, FilterFactory>, x: Filter) {
                // Nothing to do if property changed
            }
        }
    }

    override fun switchMode() {
        super.switchMode()
        applyButton.isVisible = isMaxMode
        applyButton.isManaged = isMaxMode
        cancelButton.isVisible = isMaxMode
        cancelButton.isManaged = isMaxMode
    }

    override fun setLoadingMode(isLoading: Boolean) {

    }

    override fun setAppropriateConfigView() {

    }
}
