package at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.parser

import at.jku.mevss.anttracks.heap.IndexBasedHeap
import at.jku.mevss.anttracks.heap.datastructures.DataStructureDevelopmentTraceParser
import at.jku.mevss.anttracks.heap.datastructures.MovingObject
import at.jku.mevss.anttracks.heap.io.MetaDataReaderConfig
import at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.DataStructureDevelopmentOverviewTab
import at.jku.mevss.trace.gui.model.AppInfo
import at.jku.mevss.trace.gui.utils.AntTask
import at.jku.mevss.trace.gui.utils.Consts
import java.io.File

class DataStructureDevelopmentParserTask(val appInfo: AppInfo,
                                         val tab: DataStructureDevelopmentOverviewTab?,
                                         private val startTime: Long,
                                         private val endTime: Long) : AntTask<DataStructureDevelopmentParserTaskResult>() {
    override fun backgroundWork(): DataStructureDevelopmentParserTaskResult {
        updateTitle("Parse trace file to calculate data structure development")
        return DataStructureDevelopmentTraceParser(appInfo.symbols,
                                                   MetaDataReaderConfig(appInfo.symbols.root + File.separator + Consts.ANT_META_DIRECTORY),
                                                   startTime,
                                                   endTime)
                .run {
                    addTraceParserListener { from, to, position ->
                        updateProgress(position - from, to - from)
                    }
                    parse()
                    val beforeIndexBasedHeap: IndexBasedHeap = initialIndexHeap!!
                    val afterIndexBasedHeap: IndexBasedHeap = workspace.toIndexBasedHeap()
                    val dataStructureDevelopmentInfos: Array<MovingObject> = addresses!!
                    DataStructureDevelopmentParserTaskResult(beforeIndexBasedHeap,
                                                             afterIndexBasedHeap,
                                                             dataStructureDevelopmentInfos)
                }
    }

    override fun suceeded() {
        val result: DataStructureDevelopmentParserTaskResult = get()
        tab?.updateParserResult(result)
    }
}