package at.jku.mevss.trace.gui.frame.main.tab.datastructuredevelopment.tableview

import at.jku.mevss.anttracks.classification.Classifier
import at.jku.mevss.anttracks.classification.ClassifierChain
import at.jku.mevss.anttracks.classification.nodes.GroupingNode
import at.jku.mevss.trace.gui.frame.main.component.applicationbase.ApplicationBaseTab
import at.jku.mevss.trace.gui.frame.main.tab.heapstate.tab.statistics.component.classificationtreetableview.ClassificationTreeTableView
import at.jku.mevss.trace.gui.model.ApproximateDouble
import at.jku.mevss.trace.gui.model.DataStructureDevelopmentInfo
import at.jku.mevss.trace.gui.model.Percentage
import javafx.beans.property.ReadOnlyLongWrapper
import javafx.beans.property.ReadOnlyObjectWrapper
import javafx.beans.value.ObservableValue
import javafx.scene.control.TreeTableColumn

class DataStructureDevelopmentClassificationTreeTableView : ClassificationTreeTableView() {

    private val objectCountBeforeColumn = TreeTableColumn<GroupingNode, ApproximateDouble>("Before")
    private val objectCountAfterColumn = TreeTableColumn<GroupingNode, ApproximateDouble>("After")
    private val objectCountAbsoluteGrowthColumn = TreeTableColumn<GroupingNode, ApproximateDouble>("Absolute Growth")
    private val objectCountRelativeGrowthColumn = TreeTableColumn<GroupingNode, Percentage>("Relative Growth [%]")

    private val shallowSizeBeforeColumn = TreeTableColumn<GroupingNode, Number>("Before")
    private val shallowSizeAfterColumn = TreeTableColumn<GroupingNode, Number>("After")
    private val shallowSizeAbsoluteGrowthColumn = TreeTableColumn<GroupingNode, Number>("Absolute Growth")
    private val shallowSizeRelativeGrowthColumn = TreeTableColumn<GroupingNode, Percentage>("Relative Growth [%]")

    private val transitiveSizeBeforeColumn = TreeTableColumn<GroupingNode, Number>("Before")
    private val transitiveSizeAfterColumn = TreeTableColumn<GroupingNode, Number>("After")
    private val transitiveClosureAbsoluteGrowthColumn = TreeTableColumn<GroupingNode, Number>("Absolute Growth")
    private val transitiveClosureRelativeGrowthColumn = TreeTableColumn<GroupingNode, Percentage>("Relative Growth [%]")
    private val transitiveClosureHeapGrowthPortion = TreeTableColumn<GroupingNode, Percentage>("Heap Growth Portion [%]")

    private val retainedSizeBeforeColumn = TreeTableColumn<GroupingNode, Number>("Before")
    private val retainedSizeAfterColumn = TreeTableColumn<GroupingNode, Number>("After")
    private val retainedClosureAbsoluteGrowthColumn = TreeTableColumn<GroupingNode, Number>("Absolute Growth")
    private val retainedClosureRelativeGrowthColumn = TreeTableColumn<GroupingNode, Percentage>("Relative Growth [%]")

    private val dataStructureBeforeColumn = TreeTableColumn<GroupingNode, Number>("Before")
    private val dataStructureAfterColumn = TreeTableColumn<GroupingNode, Number>("After")
    private val dataStructureAbsoluteGrowthColumn = TreeTableColumn<GroupingNode, Number>("Absolute Growth")
    private val dataStructureRelativeGrowthColumn = TreeTableColumn<GroupingNode, Percentage>("Relative Growth [%]")

    private val deepDataStructureBeforeColumn = TreeTableColumn<GroupingNode, Number>("Before")
    private val deepDataStructureAfterColumn = TreeTableColumn<GroupingNode, Number>("After")
    private val deepDataStructureAbsoluteGrowthColumn = TreeTableColumn<GroupingNode, Number>("Absolute Growth")
    private val deepDataStructureRelativeGrowthColumn = TreeTableColumn<GroupingNode, Percentage>("Relative Growth [%]")

    fun init(info: DataStructureDevelopmentInfo,
             selectLocalClassifiers: (Classifier<*>) -> Boolean,
             addTab: (ApplicationBaseTab) -> Unit,
             switchToTab: (ApplicationBaseTab) -> Unit,
             showTTVOnScreen: () -> Unit,
             setSelectedClassifiers: (ClassifierChain) -> Unit) {
        super.init(info.appInfo, info, selectLocalClassifiers, addTab, switchToTab, showTTVOnScreen, setSelectedClassifiers)
        columns.forEach { topColumn ->
            topColumn.prefWidthProperty().unbind()
            topColumn.columns.forEach { firstChildLevelColumn -> firstChildLevelColumn.prefWidthProperty().unbind() }
        }

        objectCountBeforeColumn.setCellValueFactory { ReadOnlyObjectWrapper(ApproximateDouble(getBefore(info, it, { it.objectCount }).doubleValue(), true)) }
        objectCountAfterColumn.setCellValueFactory { ReadOnlyObjectWrapper(ApproximateDouble(getAfter(info, it, { it.objectCount }).doubleValue(), true)) }
        objectCountAbsoluteGrowthColumn.setCellValueFactory { ReadOnlyObjectWrapper(ApproximateDouble(getAbsoluteGrowth(info, it, { it.objectCount }).value.toDouble(), true)) }
        objectCountRelativeGrowthColumn.setCellValueFactory { getRelativeGrowth(info, it, { it.objectCount }) }
        objectsColumn.columns.add(objectCountBeforeColumn)
        objectsColumn.columns.add(objectCountAfterColumn)
        objectsColumn.columns.add(objectCountAbsoluteGrowthColumn)
        objectsColumn.columns.add(objectCountRelativeGrowthColumn)

        shallowSizeBeforeColumn.setCellValueFactory { getBefore(info, it, { it.byteCount }) }
        shallowSizeAfterColumn.setCellValueFactory { getAfter(info, it, { it.byteCount }) }
        shallowSizeAbsoluteGrowthColumn.setCellValueFactory { getAbsoluteGrowth(info, it, { it.byteCount }) }
        shallowSizeRelativeGrowthColumn.setCellValueFactory { getRelativeGrowth(info, it, { it.byteCount }) }
        shallowSizeColumn.columns.add(shallowSizeBeforeColumn)
        shallowSizeColumn.columns.add(shallowSizeAfterColumn)
        shallowSizeColumn.columns.add(shallowSizeAbsoluteGrowthColumn)
        shallowSizeColumn.columns.add(shallowSizeRelativeGrowthColumn)

        transitiveSizeBeforeColumn.setCellValueFactory { getBefore(info, it, { it.transitiveClosureSizeProperty().longValue() }) }
        transitiveSizeAfterColumn.setCellValueFactory { getAfter(info, it, { it.transitiveClosureSizeProperty().longValue() }) }
        transitiveClosureAbsoluteGrowthColumn.setCellValueFactory { getAbsoluteGrowth(info, it, { it.transitiveClosureSizeProperty().longValue() }) }
        transitiveClosureRelativeGrowthColumn.setCellValueFactory { getRelativeGrowth(info, it, { it.transitiveClosureSizeProperty().longValue() }) }
        transitiveClosureHeapGrowthPortion.setCellValueFactory {
            ReadOnlyObjectWrapper(Percentage((getAbsoluteGrowth(info, it, { it.transitiveClosureSizeProperty().longValue() }).value.toLong() * 100.0 / info.absoluteHeapGrowth())))
        }
        transitiveClosureSizeColumn.columns.add(transitiveSizeBeforeColumn)
        transitiveClosureSizeColumn.columns.add(transitiveSizeAfterColumn)
        transitiveClosureSizeColumn.columns.add(transitiveClosureAbsoluteGrowthColumn)
        transitiveClosureSizeColumn.columns.add(transitiveClosureRelativeGrowthColumn)
        transitiveClosureSizeColumn.columns.add(transitiveClosureHeapGrowthPortion)

        retainedSizeBeforeColumn.setCellValueFactory { getBefore(info, it, { it.retainedSizeProperty().longValue() }) }
        retainedSizeAfterColumn.setCellValueFactory { getAfter(info, it, { it.retainedSizeProperty().longValue() }) }
        retainedClosureAbsoluteGrowthColumn.setCellValueFactory { getAbsoluteGrowth(info, it, { it.retainedSizeProperty().longValue() }) }
        retainedClosureRelativeGrowthColumn.setCellValueFactory { getRelativeGrowth(info, it, { it.retainedSizeProperty().longValue() }) }
        retainedSizeColumn.columns.add(retainedSizeBeforeColumn)
        retainedSizeColumn.columns.add(retainedSizeAfterColumn)
        retainedSizeColumn.columns.add(retainedClosureAbsoluteGrowthColumn)
        retainedSizeColumn.columns.add(retainedClosureRelativeGrowthColumn)

        dataStructureBeforeColumn.setCellValueFactory { getBefore(info, it, { it.dataStructureSizeProperty().longValue() }) }
        dataStructureAfterColumn.setCellValueFactory { getAfter(info, it, { it.dataStructureSizeProperty().longValue() }) }
        dataStructureAbsoluteGrowthColumn.setCellValueFactory { getAbsoluteGrowth(info, it, { it.dataStructureSizeProperty().longValue() }) }
        dataStructureRelativeGrowthColumn.setCellValueFactory { getRelativeGrowth(info, it, { it.dataStructureSizeProperty().longValue() }) }
        dataStructureSizeColumn.columns.add(dataStructureBeforeColumn)
        dataStructureSizeColumn.columns.add(dataStructureAfterColumn)
        dataStructureSizeColumn.columns.add(dataStructureAbsoluteGrowthColumn)
        dataStructureSizeColumn.columns.add(dataStructureRelativeGrowthColumn)

        deepDataStructureBeforeColumn.setCellValueFactory { getBefore(info, it, { it.deepDataStructureSizeProperty().longValue() }) }
        deepDataStructureAfterColumn.setCellValueFactory { getAfter(info, it, { it.deepDataStructureSizeProperty().longValue() }) }
        deepDataStructureAbsoluteGrowthColumn.setCellValueFactory { getAbsoluteGrowth(info, it, { it.deepDataStructureSizeProperty().longValue() }) }
        deepDataStructureRelativeGrowthColumn.setCellValueFactory { getRelativeGrowth(info, it, { it.deepDataStructureSizeProperty().longValue() }) }
        deepDataStructureSizeColumn.columns.add(deepDataStructureBeforeColumn)
        deepDataStructureSizeColumn.columns.add(deepDataStructureAfterColumn)
        deepDataStructureSizeColumn.columns.add(deepDataStructureAbsoluteGrowthColumn)
        deepDataStructureSizeColumn.columns.add(deepDataStructureRelativeGrowthColumn)

        // hide less interesting columns
        columns.forEach { topColumn ->
            topColumn.prefWidthProperty().unbind()
            topColumn.isVisible = true
            topColumn.columns.forEach { firstChildLevelColumn ->
                firstChildLevelColumn.prefWidthProperty().unbind()
                firstChildLevelColumn.setVisible(true)
            }
        }
        objectCountAbsoluteGrowthColumn.isVisible = false
        objectCountRelativeGrowthColumn.isVisible = false
        objectsCountColumn.isVisible = false
        objectsPercentColumn.isVisible = false
        shallowSizeColumn.isVisible = false
        closureSizeCountColumn.isVisible = false
        closureSizePercentColumn.isVisible = false
        transitiveClosureRelativeGrowthColumn.isVisible = false
        gcSizeCountColumn.isVisible = false
        gcSizePercentColumn.isVisible = false
        retainedClosureRelativeGrowthColumn.isVisible = false
        dataStructureSizeColumn.isVisible = false
        deepDataStructureSizeColumn.isVisible = false

        // set column widths
        nameColumn.prefWidthProperty().bind(widthProperty().multiply(13).divide(35).subtract(23))   // subtract scrollbar width
        objectCountBeforeColumn.prefWidthProperty().bind(widthProperty().multiply(2).divide(35))
        objectCountAfterColumn.prefWidthProperty().bind(widthProperty().multiply(2).divide(35))
        transitiveSizeBeforeColumn.prefWidthProperty().bind(widthProperty().multiply(2).divide(35))
        transitiveSizeAfterColumn.prefWidthProperty().bind(widthProperty().multiply(2).divide(35))
        transitiveClosureAbsoluteGrowthColumn.prefWidthProperty().bind(widthProperty().multiply(3).divide(35))
        transitiveClosureHeapGrowthPortion.prefWidthProperty().bind(widthProperty().multiply(4).divide(35))
        retainedSizeBeforeColumn.prefWidthProperty().bind(widthProperty().multiply(2).divide(35))
        retainedSizeAfterColumn.prefWidthProperty().bind(widthProperty().multiply(2).divide(35))
        retainedClosureAbsoluteGrowthColumn.prefWidthProperty().bind(widthProperty().multiply(3).divide(35))
    }

    private fun getBefore(info: DataStructureDevelopmentInfo,
                          param: TreeTableColumn.CellDataFeatures<GroupingNode, *>,
                          valueExtractor: (GroupingNode) -> Long): ReadOnlyLongWrapper {
        val key = param.value.value.fullKey
        val matchingChild = info.startClassificationTree!!.root.getChildWithFullKey(key) ?: return ReadOnlyLongWrapper(0)
        return ReadOnlyLongWrapper(valueExtractor(matchingChild))
    }

    private fun getAfter(info: DataStructureDevelopmentInfo,
                         param: TreeTableColumn.CellDataFeatures<GroupingNode, *>,
                         valueExtractor: (GroupingNode) -> Long): ReadOnlyLongWrapper {
        val key = param.value.value.fullKey
        val matchingChild = info.endClassificationTree!!.root.getChildWithFullKey(key) ?: return ReadOnlyLongWrapper(0)
        return ReadOnlyLongWrapper(valueExtractor(matchingChild))
    }

    private fun getAbsoluteGrowth(info: DataStructureDevelopmentInfo,
                                  param: TreeTableColumn.CellDataFeatures<GroupingNode, *>,
                                  valueExtractor: (GroupingNode) -> Long): ObservableValue<Number> {
        val key = param.value.value.fullKey
        val matchingChild = info.classificationTreeDifference!!.getChildWithFullKey(key)
        if (matchingChild == null) {
            // TODO: Do something here, this should not happen...
        }
        return ReadOnlyLongWrapper(valueExtractor(matchingChild))
    }

    private fun getRelativeGrowth(info: DataStructureDevelopmentInfo,
                                  param: TreeTableColumn.CellDataFeatures<GroupingNode, *>,
                                  valueExtractor: (GroupingNode) -> Long): ObservableValue<Percentage> {
        var growth = getAbsoluteGrowth(info, param, valueExtractor).value.toFloat() * 100.0f
        growth /= valueExtractor(info.startClassificationTree!!.root.getChildWithFullKey(param.value.value.fullKey)).toFloat()
        return ReadOnlyObjectWrapper(Percentage(growth.toDouble()))
    }

}