/*
Copyright (c) 2014, 2015, 2016, 2017, dynatrace and/or its affiliates. All rights reserved.
This file is part of the AntTracks Tool.

This code is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License version 2 only, as
published by the Free Software Foundation.

This code is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
version 2 for more details (a copy is included in the LICENSE file that
accompanied this code).

You should have received a copy of the GNU General Public License
along with with this work.  If not, see <http://www.gnu.org/licenses/>.
*/
package at.jku.mevss.trace.gui.classification.test;

import at.jku.mevss.anttracks.classification.Classifier;
import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.ClassifierFactoryList;
import at.jku.mevss.anttracks.diff.DiffGrouping;
import at.jku.mevss.anttracks.diff.HeapDiff;
import at.jku.mevss.anttracks.heap.PreprocessedHeap;
import at.jku.mevss.anttracks.util.TestUtility;
import at.jku.mevss.anttracks.util.TraceException;
import jku.anttracks.example.heapdiffing.artificial.born.Born;
import jku.anttracks.example.heapdiffing.artificial.died.Died;
import jku.anttracks.example.heapdiffing.artificial.perm.Perm;
import jku.anttracks.example.heapdiffing.artificial.temp.Temp;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;

public class Classifier_HeapDiff_Test {
    private double delta = 0.000001;

    private static PreprocessedHeap preprocessedHeap;

    private ClassifierFactoryList classifiers;

    @BeforeClass
    public static void initClass() throws IOException, TraceException, Exception {
        String traceFolder = TestUtility.getExampleTrace("PointerTraces/HeapDiffing-Artificial/trace", "./downloadedTrace");
        preprocessedHeap = TestUtility.preprocess(traceFolder, true);
    }

    private void initClassifiers(HeapDiff diff) {
        classifiers = new ClassifierFactoryList(() -> diff.getIntervals().get(0).getLastHeap().getSymbols(), null, null);
    }

    @Test
    public void test_Type_PermDiedCreation() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffEndToEnd(preprocessedHeap, 0, 1);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals("Born count must match", 1_000, grouping.getBorn().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals("Born count must match", 1_000, grouping.getBorn().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_PermDiedCreation successful");
    }

    @Test
    public void test_Type_Gc1EndToGc2End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffEndToEnd(preprocessedHeap, 1, 2);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.out.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1EndToGc2End successful");
    }

    @Test
    public void test_Type_Gc1EndToGc3End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffEndToEnd(preprocessedHeap, 1, 3);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1EndToGc3End successful");
    }

    @Test
    public void test_Type_Gc1EndToGc4End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffEndToEnd(preprocessedHeap, 1, 4);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getTemp().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1EndToGc4End successful");
    }

    @Test
    public void test_Type_Gc1EndToGc5End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffEndToEnd(preprocessedHeap, 1, 5);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getTemp().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1EndToGc5End successful");
    }

    @Test
    public void test_Type_Gc1StartToGc2End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToEnd(preprocessedHeap, 1, 2);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc2End successful");
    }

    @Test
    public void test_Type_Gc1StartToGc3End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToEnd(preprocessedHeap, 1, 3);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc3End successful");
    }

    @Test
    public void test_Type_Gc1StartToGc4End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToEnd(preprocessedHeap, 1, 4);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getTemp().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc4End successful");
    }

    @Test
    public void test_Type_Gc1StartToGc5End() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToEnd(preprocessedHeap, 1, 5);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getTemp().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc5End successful");
    }

    @Test
    public void test_Type_Gc1StartToGc2Start() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToStart(preprocessedHeap, 1, 2);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc2Start successful");
    }

    @Test
    public void test_Type_Gc1StartToGc3Start() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToStart(preprocessedHeap, 1, 3);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc3Start successful");
    }

    @Test
    public void test_Type_Gc1StartToGc4Start() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToStart(preprocessedHeap, 1, 4);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.out.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc4Start successful");
    }

    @Test
    public void test_Type_Gc1StartToGc5Start() throws Exception {
        HeapDiff diff = TestUtility.getHeapDiffStartToStart(preprocessedHeap, 1, 5);
        initClassifiers(diff);
        Classifier<?> classifier = classifiers.get("Type");
        DiffGrouping grouping = diff.getIntervals().get(0).info.classify(new ClassifierChain(classifier), null, null);
        // System.err.println(grouping);
        Assert.assertEquals(1_000, grouping.getPerm().getChild(Perm.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getBorn().getChild(Born.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getTemp().getChild(Temp.class.getName()).getObjectCount(), delta);
        Assert.assertEquals(1_000, grouping.getDied().getChild(Died.class.getName()).getObjectCount(), delta);
        System.err.println("Test test_Type_Gc1StartToGc5Start successful");
    }
}
