package at.jku.mevss.trace.gui.classification.test;

import at.jku.mevss.anttracks.classification.ClassifierChain;
import at.jku.mevss.anttracks.classification.ClassifierFactoryList;
import at.jku.mevss.anttracks.classification.Filter;
import at.jku.mevss.anttracks.classification.FilterFactoryList;
import at.jku.mevss.anttracks.classification.trees.ListClassificationTree;
import at.jku.mevss.anttracks.heap.*;
import at.jku.mevss.anttracks.parser.EventType;
import at.jku.mevss.anttracks.util.TestUtility;
import at.jku.mevss.anttracks.util.TraceException;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

public class FilterTest {
    private static PreprocessedHeap preprocessedHeap;

    private ClassifierFactoryList classifiers;
    private FilterFactoryList filters;

    @BeforeClass
    public static void initClass() throws IOException, TraceException, Exception {
        String traceFolder = TestUtility.getExampleTrace("PointerTraces/FlatDataStructures/trace", "./downloadedTrace");
        preprocessedHeap = TestUtility.preprocess(traceFolder, true);
    }

    private void initClassifiersAndFilters(IndexBasedHeap fastHeap) {
        classifiers = new ClassifierFactoryList(() -> fastHeap.getSymbols(), () -> fastHeap, null);
        filters = new FilterFactoryList(() -> fastHeap.getSymbols(), () -> fastHeap, null);
    }

    @Test
    public void testDataStructureHeadFilter_Maps() throws Exception {
        if (IndexBasedHeap.REDUCE_TIMING_MEASUREMENT_DISTORTION || !IndexBasedHeap.CALCULATE_DATA_STRUCTURES) {
            return;
        }
        DetailedHeap heap = TestUtility.getHeap(preprocessedHeap, new GarbageCollectionLookup(EventType.GC_END, GarbageCollectionType.MAJOR, null, 0));
        IndexBasedHeap fastHeap = heap.toIndexBasedHeap();
        initClassifiersAndFilters(fastHeap);

        ListClassificationTree classificationTree = fastHeap.groupListParallel(new Filter[]{filters.get("Only data structure heads")},
                                                                               new ClassifierChain(classifiers.get("Type")),
                                                                               null,
                                                                               null,
                                                                               null);
        // HashMap is a datastructure head
        Assert.assertNotNull(classificationTree.getRoot().getChild(HashMap.class.getName()));
        // HashMap$Entry is not a data structure head, but is part of a data structure
        Assert.assertNull(classificationTree.getRoot().getChild(HashMap.Entry.class.getName()));

        Assert.assertNotNull(classificationTree.getRoot().getChild(LinkedHashMap.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(LinkedHashMap.SimpleEntry.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(LinkedHashMap.SimpleImmutableEntry.class.getName()));

        Assert.assertNotNull(classificationTree.getRoot().getChild(ConcurrentHashMap.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(ConcurrentHashMap.KeySetView.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(ConcurrentHashMap.SimpleEntry.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(ConcurrentHashMap.SimpleImmutableEntry.class.getName()));

        Assert.assertNotNull(classificationTree.getRoot().getChild(IdentityHashMap.class.getName()));
        Assert.assertNotNull(classificationTree.getRoot().getChild(WeakHashMap.class.getName()));
        Assert.assertNotNull(classificationTree.getRoot().getChild(TreeMap.class.getName()));
        Assert.assertNotNull(classificationTree.getRoot().getChild(Hashtable.class.getName()));

        // We do not expect any of the basic data types
        Assert.assertNull(classificationTree.getRoot().getChild(String.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(Integer.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(Long.class.getName()));
        Assert.assertNull(classificationTree.getRoot().getChild(Character.class.getName()));
    }
}
